@@ -902,19 +902,19 @@ F2x_valrem(GEN x, GEN *Z)
   l -= i-2;
   y = cgetg(l, t_VECSMALL); y[1] = x[1];
   if (v2 == 0)
-    for (i=2; i<l; i++) y[i] = x[i+v];
+    for (i=2; i<l; i++) uel(y,i) = uel(x,i+v);
   else if (l == 3)
-    y[2] = ((ulong)x[2+v]) >> v2;
+    y[2] = uel(x,2+v) >> v2;
   else
   {
     const ulong sh = BITS_IN_LONG - v2;
-    ulong r = x[2+v];
+    ulong r = uel(x,2+v);
     for (i=3; i<l; i++)
     {
-      y[i-1] = (x[i+v] << sh) | (r >> v2);
-      r = x[i+v];
+      uel(y,i-1) = (uel(x,i+v)<< sh) | (r >> v2);
+      r = uel(x,i+v);
     }
-    y[l-1] = r >> v2;
+    uel(y,l-1) = r >> v2;
     (void)F2x_renormalize(y,l);
   }
   *Z = y; return (v << TWOPOTBITS_IN_LONG) + v2;
@@ -2677,11 +2677,15 @@ F2xqX_halfres_basecase(GEN a, GEN b, GEN T, GEN *pa, GEN *pb, struct F2xqX_res *
     if (gc_needed(av,2))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"F2xqX_halfgcd (d = %ld)",degpol(b));
-      gerepileall(av,res ? 8: 6, &a,&b,&u1,&v1,&u,&v,&res->res,&res->lc);
+      if (res)
+        gerepileall(av, 8, &a,&b,&u1,&v1,&u,&v,&res->res,&res->lc);
+      else
+        gerepileall(av, 6, &a,&b,&u1,&v1,&u,&v);
     }
   }
   M = mkmat22(u,v,u1,v1); *pa = a; *pb = b;
-  return gc_all(av, res ? 5: 3, &M, pa, pb, &res->res, &res->lc);
+  return res ? gc_all(av, 5, &M, pa, pb, &res->res, &res->lc)
+             : gc_all(av, 3, &M, pa, pb);
 }
 
 static GEN F2xqX_halfres_i(GEN x, GEN y, GEN T, GEN *a, GEN *b, struct F2xqX_res *res);
@@ -2713,7 +2717,8 @@ F2xqX_halfres_split(GEN x, GEN y, GEN T, GEN *a, GEN *b, struct F2xqX_res *res)
   if (lgpol(y1) <= n)
   {
     *a = x1; *b = y1;
-    return gc_all(av, res ? 5: 3, &R, a, b, &res->res, &res->lc);
+    return res ? gc_all(av, 5, &R, a, b, &res->res, &res->lc)
+               : gc_all(av, 3, &R, a, b, &res->res, &res->lc);
   }
   k = 2*n-degpol(y1);
   q = F2xqX_divrem(x1, y1, T, &r);
@@ -2744,7 +2749,8 @@ F2xqX_halfres_split(GEN x, GEN y, GEN T, GEN *a, GEN *b, struct F2xqX_res *res)
   V2 = F2xqXM_F2xqX_mul2(S, F2xXn_red(y1,k), F2xXn_red(r,k), T);
   *a = F2xX_add(F2xX_shift(*a,k,vT), gel(V2,1));
   *b = F2xX_add(F2xX_shift(*b,k,vT), gel(V2,2));
-  return gc_all(av, res ? 5: 3, &Q, a, b, &res->res, &res->lc);
+  return res ? gc_all(av, 5, &Q, a, b, &res->res, &res->lc)
+             : gc_all(av, 3, &Q, a, b, &res->res, &res->lc);
 }
 
 static GEN
