@@ -32,6 +32,10 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
  * the school-book method used in FpE_mulu().
  */
 
+static GEN
+ellinf_FpJ(void)
+{ return mkvec3(gen_1, gen_1, gen_0); }
+
 /*
  * Cost: 1M + 8S + 1*a + 10add + 1*8 + 2*2 + 1*3.
  * Source: http://www.hyperelliptic.org/EFD/g1p/auto-shortw-jacobian.html#doubling-dbl-2007-bl
@@ -43,8 +47,7 @@ FpJ_dbl(GEN P, GEN a4, GEN p)
   GEN X1, Y1, Z1;
   GEN XX, YY, YYYY, ZZ, S, M, T, Q;
 
-  if (signe(gel(P,3)) == 0)
-    return gcopy(P);
+  if (signe(gel(P,3)) == 0) return ellinf_FpJ();
 
   X1 = gel(P,1); Y1 = gel(P,2); Z1 = gel(P,3);
 
@@ -52,16 +55,13 @@ FpJ_dbl(GEN P, GEN a4, GEN p)
   YY = Fp_sqr(Y1, p);
   YYYY = Fp_sqr(YY, p);
   ZZ = Fp_sqr(Z1, p);
-  S = Fp_mulu(Fp_sub(Fp_sqr(Fp_add(X1, YY, p), p),
-                       Fp_add(XX, YYYY, p), p), 2, p);
+  S = Fp_double(Fp_sub(Fp_sqr(Fp_add(X1,YY,p), p), Fp_add(XX,YYYY,p), p), p);
   M = Fp_addmul(Fp_mulu(XX, 3, p), a4, Fp_sqr(ZZ, p),  p);
-  T = Fp_sub(Fp_sqr(M, p), Fp_mulu(S, 2, p), p);
+  T = Fp_sub(Fp_sqr(M, p), Fp_double(S, p), p);
   Q = cgetg(4, t_VEC);
   gel(Q,1) = T;
-  gel(Q,2) = Fp_sub(Fp_mul(M, Fp_sub(S, T, p), p),
-                Fp_mulu(YYYY, 8, p), p);
-  gel(Q,3) = Fp_sub(Fp_sqr(Fp_add(Y1, Z1, p), p),
-                Fp_add(YY, ZZ, p), p);
+  gel(Q,2) = Fp_sub(Fp_mul(M, Fp_sub(S, T, p), p), Fp_mulu(YYYY, 8, p), p);
+  gel(Q,3) = Fp_sub(Fp_sqr(Fp_add(Y1, Z1, p), p), Fp_add(YY, ZZ, p), p);
   return Q;
 }
 
@@ -89,7 +89,7 @@ FpJ_add(GEN P, GEN Q, GEN a4, GEN p)
   S1 = mulii(Y1, Fp_mul(Z2, Z2Z2, p));
   S2 = mulii(Y2, Fp_mul(Z1, Z1Z1, p));
   H = Fp_sub(U2, U1, p);
-  r = Fp_mulu(Fp_sub(S2, S1, p), 2, p);
+  r = Fp_double(Fp_sub(S2, S1, p), p);
 
   /* If points are equal we must double. */
   if (signe(H)== 0) {
@@ -97,12 +97,12 @@ FpJ_add(GEN P, GEN Q, GEN a4, GEN p)
       /* Points are equal so double. */
       return FpJ_dbl(P, a4, p);
     else
-      return mkvec3(gen_1, gen_1, gen_0);
+      return ellinf_FpJ();
   }
-  I = Fp_sqr(Fp_mulu(H, 2, p), p);
+  I = Fp_sqr(Fp_double(H, p), p);
   J = Fp_mul(H, I, p);
   V = Fp_mul(U1, I, p);
-  W = Fp_sub(Fp_sqr(r, p), Fp_add(J, Fp_mulu(V, 2, p), p), p);
+  W = Fp_sub(Fp_sqr(r, p), Fp_add(J, Fp_double(V, p), p), p);
   R = cgetg(4, t_VEC);
   gel(R,1) = W;
   gel(R,2) = Fp_sub(mulii(r, subii(V, W)),
@@ -120,8 +120,9 @@ FpJ_neg(GEN Q, GEN p)
 
 GEN
 FpE_to_FpJ(GEN P)
-{ return ell_is_inf(P) ? mkvec3(gen_1, gen_1, gen_0):
-                         mkvec3(icopy(gel(P,1)),icopy(gel(P,2)), gen_1);
+{
+  return ell_is_inf(P) ? ellinf_FpJ()
+       : mkvec3(icopy(gel(P,1)),icopy(gel(P,2)), gen_1);
 }
 
 GEN
@@ -155,8 +156,8 @@ _FpJ_mul(void *E, GEN P, GEN n)
   pari_sp av = avma;
   struct _FpE *e=(struct _FpE *) E;
   long s = signe(n);
-  if (!s || signe(gel(P,3))==0) return mkvec3(gen_1, gen_1, gen_0);
-  if (s<0) P = FpJ_neg(P, e->p);
+  if (!s || signe(gel(P,3))==0) return ellinf_FpJ();
+  if (s < 0) P = FpJ_neg(P, e->p);
   if (is_pm1(n)) return s > 0 ? gcopy(P): P;
   return gerepilecopy(av, gen_pow_i(P, n, e, &_FpJ_dbl, &_FpJ_add));
 }
@@ -237,10 +238,15 @@ FpE_changepointinv(GEN P, GEN ch, GEN p)
 }
 
 static GEN
-nonsquare_Fp(GEN p)
+random_nonsquare_Fp(GEN p)
 {
   pari_sp av = avma;
   GEN a;
+  switch(mod8(p))
+  { /* easy special cases */
+    case 3: case 5: return gen_2;
+    case 7: return subiu(p, 1);
+  }
   do
   {
     set_avma(av);
@@ -252,7 +258,7 @@ nonsquare_Fp(GEN p)
 void
 Fp_elltwist(GEN a4, GEN a6, GEN p, GEN *pt_a4, GEN *pt_a6)
 {
-  GEN d = nonsquare_Fp(p), d2 = Fp_sqr(d, p), d3 = Fp_mul(d2, d, p);
+  GEN d = random_nonsquare_Fp(p), d2 = Fp_sqr(d, p), d3 = Fp_mul(d2, d, p);
   *pt_a4 = Fp_mul(a4, d2, p);
   *pt_a6 = Fp_mul(a6, d3, p);
 }
@@ -600,7 +606,7 @@ FpE_weilpairing(GEN P, GEN Q, GEN m, GEN a4, GEN p)
     ulong pp = p[2];
     GEN Pp = ZV_to_Flv(P, pp), Qp = ZV_to_Flv(Q, pp);
     ulong w = Fle_weilpairing(Pp, Qp, itou(m), umodiu(a4, pp), pp);
-    set_avma(av); return utoi(w);
+    return gc_utoi(av, w);
   }
   N = FpE_Miller(P, Q, m, a4, p);
   D = FpE_Miller(Q, P, m, a4, p);
@@ -619,7 +625,7 @@ FpE_tatepairing(GEN P, GEN Q, GEN m, GEN a4, GEN p)
     ulong pp = p[2];
     GEN Pp = ZV_to_Flv(P, pp), Qp = ZV_to_Flv(Q, pp);
     ulong w = Fle_tatepairing(Pp, Qp, itou(m), umodiu(a4, pp), pp);
-    set_avma(av); return utoi(w);
+    return gc_utoi(av,w);
   }
   return FpE_Miller(P, Q, m, a4, p);
 }
@@ -684,7 +690,17 @@ FqX_quad_root(GEN x, GEN T, GEN p)
   GEN D = Fq_sub(Fq_sqr(b, T, p), Fq_mulu(c,4, T, p), T, p);
   GEN s = Fq_sqrt(D,T, p);
   if (!s) return NULL;
-  return Fq_Fp_mul(Fq_sub(s, b, T, p), shifti(addiu(p, 1),-1),T, p);
+  return Fq_halve(Fq_sub(s, b, T, p), T, p);
+}
+
+static GEN
+FpX_quad_root(GEN x, GEN p)
+{
+  GEN s, b = gel(x,3), c = gel(x,2);
+  GEN D = Fp_sub(Fp_sqr(b, p), shifti(c,2), p);
+  if (kronecker(D,p) == -1) return NULL;
+  s = Fp_sqrt(D,p);
+  return Fp_halve(Fp_sub(s, b, p), p);
 }
 
 /*
@@ -693,37 +709,81 @@ FqX_quad_root(GEN x, GEN T, GEN p)
  * (T, p) defines the field FF_{p^2} in which j_prev and j live.
  */
 static long
-path_extends_to_floor(GEN j_prev, GEN j, GEN T, GEN p, GEN Phi2, ulong max_len)
+Fq_path_extends_to_floor(GEN j_prev, GEN j, GEN T, GEN p, GEN Phi2, long max_len)
 {
   pari_sp ltop = avma;
-  GEN Phi2_j;
-  ulong mult, d;
+  long d, i, l = lg(j);
 
   /* A path made its way to the floor if (i) its length was cut off
    * before reaching max_path_len, or (ii) it reached max_path_len but
    * only has one neighbour. */
-  for (d = 1; d < max_len; ++d) {
-    GEN j_next;
-
-    Phi2_j = FqX_div_by_X_x(FqXY_evalx(Phi2, j, T, p), j_prev, T, p, NULL);
-    j_next = FqX_quad_root(Phi2_j, T, p);
-    if (!j_next)
-    { /* j is on the floor */
-      set_avma(ltop);
-      return 1;
+  for (d = 1; d <= max_len; ++d)
+  {
+    for (i = 1; i < l; i++)
+    {
+      GEN Phi2_j = FqX_div_by_X_x(FqXY_evalx(Phi2, gel(j,i), T, p), gel(j_prev,i), T, p, NULL);
+      GEN j_next = FqX_quad_root(Phi2_j, T, p);
+      if (!j_next)
+        return  gc_long(ltop, 1);
+      gel(j_prev,i) = gel(j, i); gel(j,i) = j_next;
     }
-
-    j_prev = j; j = j_next;
     if (gc_needed(ltop, 2))
       gerepileall(ltop, 2, &j, &j_prev);
   }
+  return gc_long(ltop, 0);
+}
 
-  /* Check that we didn't end up at the floor on the last step (j will
-   * point to the last element in the path. */
-  Phi2_j = FqX_div_by_X_x(FqXY_evalx(Phi2, j, T, p), j_prev, T, p, NULL);
-  mult = FqX_nbroots(Phi2_j, T, p);
-  set_avma(ltop);
-  return mult == 0;
+static long
+Fp_path_extends_to_floor(GEN j_prev, GEN j, GEN p, GEN Phi2, long max_len, GEN *pt_j, GEN *pt_j_prev)
+{
+  pari_sp ltop = avma;
+  long d, i, l = lg(j);
+
+  /* A path made its way to the floor if (i) its length was cut off
+   * before reaching max_path_len, or (ii) it reached max_path_len but
+   * only has one neighbour. */
+  for (d = 1; d <= max_len; ++d)
+  {
+    for (i = 1; i < l; i++)
+    {
+      GEN Phi2_j = FpX_div_by_X_x(FpXY_evalx(Phi2, gel(j,i), p), gel(j_prev,i), p, NULL);
+      GEN j_next = FpX_quad_root(Phi2_j, p);
+      if (!j_next)
+      {
+        *pt_j = gel(j,i);
+        *pt_j_prev = gel(j_prev,i);
+        return 1;
+      }
+      gel(j_prev,i) = gel(j, i); gel(j,i) = j_next;
+    }
+    if (gc_needed(ltop, 2))
+      gerepileall(ltop, 2, &j, &j_prev);
+  }
+  return gc_long(ltop, 0);
+}
+
+
+static int
+Fp_jissupersingular(GEN j, GEN p)
+{
+  long max_path_len = expi(p)+1;
+  GEN Phi2 = FpXX_red(polmodular_ZXX(2,0,0,1), p);
+  GEN Phi2_j = FpXY_evalx(Phi2, j, p);
+  GEN roots = FpX_roots(Phi2_j, p);
+  long nbroots = lg(roots)-1;
+  GEN S, j_prev = NULL;
+
+  /* Every node in a supersingular L-volcano has L + 1 neighbours. */
+  /* Note: a multiple root only occur when j has CM by sqrt(-15). */
+  if (nbroots==0)
+    return 0;
+  S = deg2pol_shallow(gen_1, gen_0, Fp_neg(Fp_2gener(p),p),1);
+  if (nbroots==1 && FpX_is_squarefree(Phi2_j, p))
+  { j_prev = j; j = FqX_quad_root(FpX_div_by_X_x(Phi2_j, gel(roots,1), p, NULL), S, p); }
+  else
+    if (!Fp_path_extends_to_floor(const_vec(nbroots,j), roots, p, Phi2, max_path_len, &j, &j_prev))
+      return 1;
+  return !Fq_path_extends_to_floor(mkvec(j_prev), mkvec(j), S, p, Phi2, max_path_len);
 }
 
 static int
@@ -734,40 +794,24 @@ jissupersingular(GEN j, GEN S, GEN p)
   GEN Phi2_j = FqXY_evalx(Phi2, j, S, p);
   GEN roots = FpXQX_roots(Phi2_j, S, p);
   long nbroots = lg(roots)-1;
-  int res = 1;
 
   /* Every node in a supersingular L-volcano has L + 1 neighbours. */
   /* Note: a multiple root only occur when j has CM by sqrt(-15). */
   if (nbroots==0 || (nbroots==1 && FqX_is_squarefree(Phi2_j, S, p)))
-    res = 0;
-  else {
-    long i, l = lg(roots);
-    for (i = 1; i < l; ++i) {
-      if (path_extends_to_floor(j, gel(roots, i), S, p, Phi2, max_path_len)) {
-        res = 0;
-        break;
-      }
-    }
-  }
-  /* If none of the paths reached the floor, then the j-invariant is
-   * supersingular. */
-  return res;
+    return 0;
+  else
+    return !Fq_path_extends_to_floor(const_vec(nbroots,j), roots, S, p, Phi2, max_path_len);
 }
 
 int
 Fp_elljissupersingular(GEN j, GEN p)
 {
-  pari_sp ltop = avma;
   long CM;
   if (abscmpiu(p, 5) <= 0) return signe(j) == 0; /* valid if p <= 5 */
   CM = Fp_ellj_get_CM(j, gen_1, p);
   if (CM < 0) return krosi(CM, p) < 0; /* valid if p > 3 */
   else
-  {
-    GEN S = init_Fq(p, 2, fetch_var());
-    int res = jissupersingular(j, S, p);
-    (void)delete_var(); return gc_bool(ltop, res);
-  }
+    return Fp_jissupersingular(j, p);
 }
 
 /***********************************************************************/
@@ -1344,6 +1388,22 @@ Fp_ellj(GEN a4, GEN a6, GEN p)
   return gerepileuptoint(av,Fp_div(gel(z,1),gel(z,2),p));
 }
 
+void
+Fp_ellj_to_a4a6(GEN j, GEN p, GEN *pt_a4, GEN *pt_a6)
+{
+  j = modii(j, p);
+  if (signe(j) == 0)    { *pt_a4 = gen_0; *pt_a6 = gen_1; }
+  else if (equaliu(j,umodui(1728,p))) { *pt_a4 = gen_1; *pt_a6 = gen_0; }
+  else
+  {
+    GEN k = Fp_sub(utoi(1728), j, p);
+    GEN kj = Fp_mul(k, j, p);
+    GEN k2j = Fp_mul(kj, k, p);
+    *pt_a4 = Fp_mulu(kj, 3, p);
+    *pt_a6 = Fp_double(k2j, p);
+  }
+}
+
 static GEN /* Only compute a mod p, so assume p>=17 */
 Fp_ellcard_CM(GEN a4, GEN a6, GEN p)
 {
@@ -1490,7 +1550,7 @@ FpXQE_changepointinv(GEN x, GEN ch, GEN T, GEN p)
 }
 
 static GEN
-nonsquare_FpXQ(GEN T, GEN p)
+random_nonsquare_FpXQ(GEN T, GEN p)
 {
   pari_sp av = avma;
   long n = degpol(T), v = varn(T);
@@ -1499,7 +1559,7 @@ nonsquare_FpXQ(GEN T, GEN p)
   {
     GEN z = cgetg(3, t_POL);
     z[1] = evalsigne(1) | evalvarn(v);
-    gel(z,2) = nonsquare_Fp(p); return z;
+    gel(z,2) = random_nonsquare_Fp(p); return z;
   }
   do
   {
@@ -1512,7 +1572,7 @@ nonsquare_FpXQ(GEN T, GEN p)
 void
 FpXQ_elltwist(GEN a4, GEN a6, GEN T, GEN p, GEN *pt_a4, GEN *pt_a6)
 {
-  GEN d = nonsquare_FpXQ(T, p);
+  GEN d = random_nonsquare_FpXQ(T, p);
   GEN d2 = FpXQ_sqr(d, T, p), d3 = FpXQ_mul(d2, d, T, p);
   *pt_a4 = FpXQ_mul(a4, d2, T, p);
   *pt_a6 = FpXQ_mul(a6, d3, T, p);
@@ -1876,6 +1936,21 @@ FpXQ_ellj(GEN a4, GEN a6, GEN T, GEN p)
   }
 }
 
+static GEN
+FpXQ_is_quad(GEN x, GEN T, GEN p)
+{
+  pari_sp av = avma;
+  GEN K;
+  long d = degpol(T);
+  x = FpXQ_red(x,T,p);
+  if (lgpol(x)<=1) return NULL;
+  if (d==2) return FpXQ_minpoly(x, T, p);
+  if (odd(degpol(T))) return NULL;
+  K = FpM_ker(FpXQ_matrix_pow(x, d, 3, T, p), p);
+  if (lg(K)!=2) return gc_NULL(av);
+  return RgV_to_RgX(gel(K,1), get_FpX_var(T));
+}
+
 int
 FpXQ_elljissupersingular(GEN j, GEN T, GEN p)
 {
@@ -1887,30 +1962,82 @@ FpXQ_elljissupersingular(GEN j, GEN T, GEN p)
    * the j-invariants are in FF_{p^{2 - e}}. */
   ulong d = get_FpX_degree(T);
   GEN S;
-
   if (degpol(j) <= 0) return Fp_elljissupersingular(constant_coeff(j), p);
-  if (abscmpiu(p, 5) <= 0) return 0; /* j != 0*/
-
+  j = FpXQ_red(j, T, p);
+  if (degpol(j) <= 0) return gc_bool(ltop, Fp_elljissupersingular(constant_coeff(j), p));
+  /* Now j is not in F_p */
+  if (abscmpiu(p, 5) <= 0) return gc_bool(ltop,0); /* j != 0*/
+  if (odd(d)) return 0;
   /* Set S so that FF_p[T]/(S) is isomorphic to FF_{p^2}: */
   if (d == 2)
     S = T;
-  else { /* d > 2 */
-    /* We construct FF_{p^2} = FF_p[t]/((T - j)(T - j^p)) which
-     * injects into FF_{p^d} via the map T |--> j. */
-    GEN j_pow_p = FpXQ_pow(j, p, T, p);
-    GEN j_sum = FpX_add(j, j_pow_p, p), j_prod;
-    long var = varn(T);
-    if (degpol(j_sum) > 0) return gc_bool(ltop,0); /* j not in Fp^2 */
-    j_prod = FpXQ_mul(j, j_pow_p, T, p);
-    if (degpol(j_prod) > 0 ) return gc_bool(ltop,0); /* j not in Fp^2 */
-    j_sum = constant_coeff(j_sum); j_prod = constant_coeff(j_prod);
-    S = mkpoln(3, gen_1, Fp_neg(j_sum, p), j_prod);
-    setvarn(S, var);
-    j = pol_x(var);
+  else /* d > 2 */
+  {
+    S = FpXQ_is_quad(j, T, p);
+    if (!S) return gc_bool(ltop,0);
+    j = pol_x(varn(S));
   }
   return gc_bool(ltop, jissupersingular(j,S,p));
 }
 
+int
+Fq_elljissupersingular(GEN j, GEN T, GEN p)
+{ return typ(j)==t_INT? Fp_elljissupersingular(j, p)
+                      : FpXQ_elljissupersingular(j, T, p); }
+
+/* p > 5 prime; return d such that (-d/p) = -1 */
+static ulong
+find_inert_disc(GEN p)
+{
+  long s = mod4(p) == 1? -1: 1; /* - (-1/p) */
+  ulong d = 3;
+  while(1)
+  {
+    if (kroui(d,p) == s) return d; /* = 3 mod (16) */
+    d++;
+    if (kroui(d>>2,p) == s) return d; /* = 4 mod (16) */
+    d += 3;
+    if (kroui(d,p) == s) return d; /* = 7 mod (16) */
+    d++;
+    if (kroui(d>>2,p) == s) return d; /* = 8 mod (16) */
+    d += 3;
+    if (kroui(d,p) == s) return d; /* = 11 mod (16) */
+    d += 4;
+    if (kroui(d,p) == s) return d; /* = 15 mod (16) */
+    d += 4;
+  }
+}
+
+/* p > 5 */
+static GEN
+ellsupersingularj_easy_FpXQ(GEN T, GEN p)
+{
+  long d = find_inert_disc(p);
+  GEN R = FpXQX_roots(polclass(stoi(-d), 0, 0), T, p);
+  return gel(R,1);
+}
+
+GEN
+ellsupersingularj_FpXQ(GEN T, GEN p)
+{
+  GEN j, j2, R, Phi2;
+  long i, ep, lp;
+  if (cmpiu(p, 5) <= 0) return pol_0(get_FpX_var(T));
+  j2 = ellsupersingularj_easy_FpXQ(T, p);
+  Phi2 = polmodular_ZXX(2,0,0,1);
+  R = FpXQX_roots(FqXY_evalx(Phi2, j2, T, p), T, p);
+  j = gel(R,1+random_Fl(lg(R)-1));
+  ep = expi(p); lp = ep + random_Fl(ep);
+  for (i = 1; i <= lp; i++)
+  {
+    GEN Phi2_j = FqX_div_by_X_x(FqXY_evalx(Phi2, j, T, p), j2, T, p, NULL);
+    R = FqX_quad_root(Phi2_j, T, p);
+    if (!R) pari_err_PRIME("ellsupersingularj",p);
+    j2 = j; j = random_bits(1) ? R: Fq_neg(Fq_add(gel(Phi2_j,3), R, T, p), T, p);
+  }
+  return j;
+}
+
 /***********************************************************************/
 /**                                                                   **/
 /**                           Point counting                          **/
@@ -1994,30 +2121,99 @@ FpXQ_ellcardj(GEN a4, GEN a6, GEN j, GEN T, GEN q, GEN p, long n)
   {
     GEN g = Fp_div(j, Fp_sub(utoi(1728), j, p), p);
     GEN l = FpXQ_div(FpX_mulu(a6,3,p),FpX_mulu(a4,2,p),T,p);
-    GEN N = Fp_ffellcard(Fp_mulu(g,3,p),Fp_mulu(g,2,p),q,n,p);
+    GEN N = Fp_ffellcard(Fp_mulu(g,3,p),Fp_double(g,p),q,n,p);
     if (FpXQ_issquare(l,T,p)) return N;
     return subii(shifti(q1,1),N);
   }
 }
 
+static GEN
+FpXQ_ffellcard(GEN a4, GEN a6, GEN M, GEN q, GEN T, GEN p, long n)
+{
+  long m = degpol(M);
+  GEN j = pol_x(get_FpX_var(T));
+  GEN g = FpXQ_div(j, Fp_FpX_sub(utoi(1728), j, p), M, p);
+  GEN N = FpXQ_ellcard(FpX_mulu(g,3,p),FpX_mulu(g,2,p),M,p);
+  GEN qm = powiu(p, m), q1 = addiu(q, 1), qm1 = addiu(qm, 1);
+  GEN l = FpXQ_mul(FpX_mulu(a6,3,p),FpX_mulu(a4,2,p),T,p);
+  GEN te = elltrace_extension(subii(qm1, N), n/m, qm);
+  return FpXQ_issquare(l,T,p) ? subii(q1, te): addii(q1, te);
+}
+
+static int
+FpXQ_is4power(GEN x, GEN T, GEN p)
+{
+  long d = get_FpX_degree(T);
+  if (lg(x) == 2 || absequalui(2, p)) return 1;
+  if (Mod4(p)==1)
+    return equali1(Fp_pow(FpXQ_norm(x,T,p),shifti(p,-2), p));
+  if (odd(d))
+    return FpXQ_issquare(x, T, p);
+  return ZX_equal1(FpXQ_pow(x, shifti(powiu(p, d),-2), T, p));
+}
+
+/* http://www.numdam.org/article/ASENS_1969_4_2_4_521_0.pdf */
+
+GEN
+FpXQ_ellcard_supersingular(GEN a4, GEN a6, GEN T, GEN p)
+{
+  pari_sp av = avma;
+  long d = get_FpX_degree(T);
+  GEN r;
+  if (equaliu(p,3))
+    r = Flxq_ellcard(ZX_to_Flx(a4,3), ZX_to_Flx(a6,3), ZXT_to_FlxT(T,3), 3);
+  else if (signe(a4)==0)
+    r = FpXQ_ellcardj(a4, a6, gen_0, T, powiu(p, d), p, d);
+  else if (signe(a6)==0)
+    r = FpXQ_ellcardj(a4, a6, modsi(1728,p), T, powiu(p, d), p, d);
+  else
+  {
+    GEN q, q2, t, D;
+    long qm4 = (odd(d>>1) && Mod4(p)==3);
+    if (odd(d)) return gen_0;
+    q2 = powiu(p, d>>1); q = sqri(q2);
+    t = shifti(q2, 1);
+    D = FpX_sub(FpX_Fp_mul(FpXQ_powu(a4,3,T,p), stoi(-4), p),
+                FpX_mulu(FpXQ_sqr(a6,T,p), 27, p), p);
+    r = qm4 ^ FpXQ_is4power(D, T, p) ? subii(addiu(q, 1), t)
+                                     : addii(addiu(q, 1), t);
+  }
+  return gerepileuptoint(av, r);
+}
+
+GEN
+Fq_ellcard_supersingular(GEN a4, GEN a6, GEN T, GEN p)
+{ return T ? FpXQ_ellcard_supersingular(a4, a6, T, p) : addiu(p, 1); }
+
+static GEN
+FpXQ_ellcard_i(GEN a4, GEN a6, GEN T, GEN p)
+{
+  long n = get_FpX_degree(T);
+  GEN q = powiu(p, n);
+  if (degpol(a4)<=0 && degpol(a6)<=0)
+    return Fp_ffellcard(constant_coeff(a4),constant_coeff(a6),q,n,p);
+  if (lgefint(p)==3)
+  {
+    ulong pp = p[2];
+    return Flxq_ellcard(ZX_to_Flx(a4,pp),ZX_to_Flx(a6,pp),ZX_to_Flx(T,pp),pp);
+  }
+  else
+  {
+    GEN J = FpXQ_ellj(a4,a6,T,p), M;
+    if (degpol(J) <= 0)
+      return FpXQ_ellcardj(a4,a6,constant_coeff(J),T,q,p,n);
+    M = FpXQ_minpoly(J,T,p);
+    if (degpol(M) < degpol(T))
+      return FpXQ_ffellcard(a4, a6, M, q, T, p, n);
+    return Fq_ellcard_SEA(a4, a6, q, T, p, 0);
+  }
+}
+
 GEN
 FpXQ_ellcard(GEN a4, GEN a6, GEN T, GEN p)
 {
   pari_sp av = avma;
-  long n = get_FpX_degree(T);
-  GEN q = powiu(p, n), r, J;
-  if (degpol(a4)<=0 && degpol(a6)<=0)
-    r = Fp_ffellcard(constant_coeff(a4),constant_coeff(a6),q,n,p);
-  else if (lgefint(p)==3)
-  {
-    ulong pp = p[2];
-    r =  Flxq_ellcard(ZX_to_Flx(a4,pp),ZX_to_Flx(a6,pp),ZX_to_Flx(T,pp),pp);
-  }
-  else if (degpol(J=FpXQ_ellj(a4,a6,T,p))<=0)
-    r = FpXQ_ellcardj(a4,a6,constant_coeff(J),T,q,p,n);
-  else
-    r = Fq_ellcard_SEA(a4, a6, q, T, p, 0);
-  return gerepileuptoint(av, r);
+  return gerepileuptoint(av, FpXQ_ellcard_i(a4, a6, T, p));
 }
 
 static GEN
