@@ -29,10 +29,9 @@ RgX_check_ZX(GEN x, const char *s)
 long
 ZX_max_lg(GEN x)
 {
-  long i, prec = 0, lx = lg(x);
-
-  for (i=2; i<lx; i++) { long l = lgefint(gel(x,i)); if (l > prec) prec = l; }
-  return prec;
+  long i, l = 0, lx = lg(x);
+  for (i = 2; i < lx; i++) l = maxss(l, lgefint(gel(x,i)));
+  return l;
 }
 
 GEN
@@ -802,7 +801,7 @@ ZX_rem(GEN x, GEN y)
     set_avma(av);
   }
   lr=i+3; rem -= lr;
-  rem[0] = evaltyp(t_POL) | evallg(lr);
+  rem[0] = evaltyp(t_POL) | _evallg(lr);
   rem[1] = z[-1];
   p1 = gerepileuptoint((pari_sp)rem, p1);
   rem += 2; gel(rem,i) = p1;
