@@ -117,6 +117,11 @@ charpoly0(GEN x, long v, long flag)
   return NULL; /* LCOV_EXCL_LINE */
 }
 
+/* (v - x)^d */
+static GEN
+caract_const(pari_sp av, GEN x, long v, long d)
+{ return gerepileupto(av, gpowgs(deg1pol_shallow(gen_1, gneg_i(x), v), d)); }
+
 /* characteristic pol. Easy cases. Return NULL in case it's not so easy. */
 static GEN
 easychar(GEN x, long v)
@@ -150,10 +155,13 @@ easychar(GEN x, long v)
     case t_POLMOD:
     {
       GEN A = gel(x,2), T = gel(x,1);
-      if (typ(A)==t_POL && RgX_is_QX(A) && RgX_is_ZX(T))
-        return QXQ_charpoly(A, T, v);
-      else
-        return RgXQ_charpoly(A, T, v);
+      long vx, vp;
+      if (typ(A) != t_POL) return caract_const(avma, A, v, degpol(T));
+      vx = varn(A);
+      vp = varn(T);
+      if (varncmp(vx, vp) > 0) return caract_const(avma, A, v, degpol(T));
+      if (varncmp(vx, vp) < 0) pari_err_PRIORITY("charpoly", x, "<", vp);
+      return RgXQ_charpoly(A, T, v);
     }
     case t_MAT:
       lx=lg(x);
@@ -817,24 +825,37 @@ minpoly(GEN x, long v)
 /*******************************************************************/
 static int
 relative0(GEN a, GEN a0, long bit)
-{ return gequal0(a) || (bit && gexpo(a)-gexpo(a0) < bit); }
-/* assume x a nonempty square t_MAT */
+{
+  if (gequal0(a)) return 1;
+  if (gequal0(a0)) return gexpo(a) < bit;
+  return (gexpo(a)-gexpo(a0) < bit);
+}
+/* x0 a nonempty square t_MAT that can be written to */
 static GEN
 RgM_hess(GEN x0, long prec)
 {
-  pari_sp av;
-  long lx = lg(x0), bit = prec? 8 - bit_accuracy(prec): 0, m, i, j;
-  GEN x;
+  pari_sp av = avma;
+  long lx = lg(x0), bit = prec? 8 - prec2nbits(prec): 0, m, i, j;
+  GEN x = bit? RgM_shallowcopy(x0): x0;
 
-  if (bit) x0 = RgM_shallowcopy(x0);
-  av = avma; x = RgM_shallowcopy(x0);
   for (m=2; m<lx-1; m++)
   {
     GEN t = NULL;
-    for (i=m+1; i<lx; i++)
-    {
-      t = gcoeff(x,i,m-1);
-      if (!relative0(t, gcoeff(x0,i,m-1), bit)) break;
+    if (!bit)
+    { /* first non-zero pivot */
+      for (i=m+1; i<lx; i++)
+        if (!gequal0(t = gcoeff(x,i,m-1))) break;
+    }
+    else
+    { /* maximal pivot */
+      long E = -(long)HIGHEXPOBIT, k = lx;
+      for (i=m+1; i<lx; i++)
+      {
+        long e = gexpo(gcoeff(x,i,m-1));
+        if (e > E) { E = e; k = i; t = gcoeff(x,i,m-1); }
+      }
+      if (k != lx && relative0(t, gcoeff(x0,k,m-1), bit)) k = lx;
+      i = k;
     }
     if (i == lx) continue;
     for (j=m-1; j<lx; j++) swap(gcoeff(x,i,j), gcoeff(x,m,j));
@@ -881,7 +902,7 @@ hess(GEN x)
     case t_COMPLEX: break;
     default: prec = 0;
   }
-  return gerepilecopy(av, RgM_hess(x,prec));
+  return gerepilecopy(av, RgM_hess(RgM_shallowcopy(x),prec));
 }
 
 GEN
@@ -1388,6 +1409,7 @@ gsupnorm_aux(GEN x, GEN *m, GEN *msq, long prec)
     case t_QUAD:  z = cxquadnorm(x,prec); store(z, msq); return;
     case t_INT: case t_REAL: z = mpabs(x); store(z,m); return;
     case t_FRAC: z = absfrac(x); store(z,m); return;
+    case t_INFINITY: store(mkoo(), m);
 
     case t_POL: lx = lg(x)-1; x++; break;
 
@@ -1554,6 +1576,23 @@ qfgaussred_positive(GEN a)
   return gerepilecopy(av,b);
 }
 
+GEN
+RgM_Cholesky(GEN M, long prec)
+{
+  pari_sp av = avma;
+  long i, j, lM = lg(M);
+  GEN R, L = qfgaussred_positive(M);
+  if (!L) return gc_NULL(av);
+  R = cgetg(lM, t_MAT); for (j = 1; j < lM; j++) gel(R,j) = cgetg(lM, t_COL);
+  for (i = 1; i < lM; i++)
+  {
+    GEN r = gsqrt(gcoeff(L,i,i), prec);
+    for (j = 1; j < lM; j++)
+      gcoeff(R, i, j) = (i == j) ? r: gmul(r, gcoeff(L, i, j));
+  }
+  return gerepileupto(av, R);
+}
+
 /* Maximal pivot strategy: x is a suitable pivot if it is non zero and either
  * - an exact type, or
  * - it is maximal among remaining nonzero (t_REAL) pivots */
@@ -1679,6 +1718,46 @@ gaussred(GEN a, long signature)
 GEN
 qfgaussred(GEN a) { return gaussred(a,0); }
 
+GEN
+qfgaussred2(GEN a)
+{
+  pari_sp av = avma;
+  GEN M = qfgaussred(a);
+  long i, l = lg(M);
+  GEN D = cgetg(l, t_VEC);
+  for (i = 1; i < l; i++)
+  {
+    gel(D,i) = gcoeff(M,i,i);
+    gcoeff(M,i,i) = gen_1;
+  }
+  return gerepilecopy(av, mkvec2(M,D));
+}
+
+GEN
+qfgaussred0(GEN a, long flag)
+{
+  switch (flag)
+  {
+    case 0: return qfgaussred(a);
+    case 1: return qfgaussred2(a);
+    default: pari_err_FLAG("qfgaussred");
+  }
+  return NULL; /* LCOV_EXCL_LINE */
+}
+
+GEN
+qfcholesky(GEN a, long prec)
+{
+  GEN M;
+  long n = lg(a);
+  if (typ(a) != t_MAT) pari_err_TYPE("qfcholesky",a);
+  if (n == 1) return cgetg(1, t_MAT);
+  if (lgcols(a) != lg(a)) pari_err_DIM("qfcholesky");
+  M =  RgM_Cholesky(a, prec);
+  if (!M) return cgetg(1, t_VEC);
+  return M;
+}
+
 GEN
 qfsign(GEN a) { return gaussred(a,1); }
 
