@@ -40,7 +40,6 @@ etilde(GEN nf, GEN pr, GEN T)
     v = u_pval(pr_get_f(pr), gp);
     return itou( mului(e, powiu(gp, v)) );
   }
-  nf = checknf(nf);
   p = itou(gp);
   k = e / (p-1) + 1;
   /* log Norm_{F_P/Q_p} (1 + P^k) = Tr(P^k) = p^[(k + v(Diff))/ e] Z_p */
@@ -63,8 +62,9 @@ etilde(GEN nf, GEN pr, GEN T)
 }
 static long
 ftilde_from_e(GEN pr, long e) { return pr_get_e(pr) * pr_get_f(pr) / e; }
+/* true nf */
 static long
-ftilde(GEN K, GEN pr, GEN T) { return ftilde_from_e(pr, etilde(K,pr, T)); }
+ftilde(GEN nf, GEN pr, GEN T) { return ftilde_from_e(pr, etilde(nf, pr, T)); }
 
 static long
 get_ZpX_index(GEN K, GEN pr, GEN T)
@@ -248,7 +248,7 @@ rnfislocalcyclo(GEN rnf)
     long lSL = lg(SL);
     for (j = 1; j < lSL; j++)
     {
-      long iS = gen_search(SL2, gel(SL,j), 0, (void*)&cmp_prime_over_p,
+      long iS = gen_search(SL2, gel(SL,j), (void*)&cmp_prime_over_p,
                 &cmp_nodata);
       long eL = etilde(L, gel(SL,j), gel(TL,iS));
       if (dvdui(eL/eK, ell)) return gc_long(av,0);
@@ -413,10 +413,10 @@ bnflog_i(GEN bnf, GEN ell)
     {
       GEN a, b, bi, A = gel(Ap,i), d = gel(cycAp,i);
       bi = isprincipal(bnf, A);
-      a = vecmodii(ZC_Z_mul(bi,d), Kcyc);
+      a = ZV_ZV_mod(ZC_Z_mul(bi,d), Kcyc);
       /* a in subgroup generated by S = Rell; hence b integral */
       b = hnf_invimage(Rell, a);
-      b = vecmodii(ZM_ZC_mul(Uell, ZC_neg(b)), ordS);
+      b = ZV_ZV_mod(ZM_ZC_mul(Uell, ZC_neg(b)), ordS);
       A = mkvec2(A, trivial_fact());
       A = idealpowred(nf, A, d);
       /* find a principal representative of A_i^cycA_i up to elements of S */
@@ -477,7 +477,7 @@ bnflogdegree(GEN nf, GEN A, GEN ell)
 
   if (typ(ell) != t_INT) pari_err_TYPE("bnflogdegree", ell);
   nf = checknf(nf);
-  A = idealhnf(nf, A);
+  A = idealhnf_shallow(nf, A);
   AZ = gcoeff(A,1,1);
   vAZ = Z_pvalrem(AZ, ell, &A0Z);
   if (is_pm1(A0Z))
