@@ -43,7 +43,7 @@ struct buch_quad
   ulong limhash;
   long KC, KC2, PRECREG;
   long *primfact, *exprimfact, **hashtab;
-  GEN FB, numFB;
+  GEN FB, numFB, prodFB;
   GEN powsubFB, vperm, subFB, badprim;
   struct qfr_data *q;
 };
@@ -176,17 +176,20 @@ qfi_random(struct buch_quad *B,GEN ex) { return random_form(B, ex, &QFI_comp); }
 /*                     Common subroutines                          */
 /*                                                                 */
 /*******************************************************************/
-long
-bnf_increase_LIMC(long LIMC, long LIMCMAX)
+/* We assume prime ideals of norm < D generate Cl(K); failure with
+ * a factor base of primes with norm < LIMC <= D. Suggest new value.
+ * All values of the form c * log^2 (disc K) [where D has c = 4
+ * (Grenie-Molteni, under GRH)]. A value of c in [0.3, 1] should be OK for most
+ * fields. No example is known where c >= 2 is necessary. */
+ulong
+bnf_increase_LIMC(ulong LIMC, ulong D)
 {
-  if (LIMC >= LIMCMAX) pari_err_BUG("Buchmann's algorithm");
-  if (LIMC <= LIMCMAX/40) /* cbach <= 0.3 */
-    LIMC *= 2;
-  else if (LIMCMAX < 60) /* \Delta_K <= 9 */
-    LIMC++;
+  if (LIMC >= D) pari_err_BUG("Buchmann's algorithm");
+  if (LIMC <= D / 13.333)
+    LIMC *= 2; /* tiny c <= 0.3 : double it */
   else
-    LIMC += LIMCMAX / 60; /* cbach += 0.2 */
-  if (LIMC > LIMCMAX) LIMC = LIMCMAX;
+    LIMC += maxuu(1, D / 20); /* large c, add 0.2 to it */
+  if (LIMC > D) LIMC = D;
   return LIMC;
 }
 
@@ -197,13 +200,27 @@ isless_iu(GEN q, ulong p) {
   return l==2 || (l == 3 && uel(q,2) <= p);
 }
 
+static GEN
+Z_isquasismooth_prod(GEN N, GEN P)
+{
+  P = gcdii(P,N);
+  while (!is_pm1(P))
+  {
+    N = diviiexact(N, P);
+    P = gcdii(N, P);
+  }
+  return N;
+}
+
 static long
 factorquad(struct buch_quad *B, GEN f, long nFB, ulong limp)
 {
   ulong X;
   long i, lo = 0;
-  GEN x = gel(f,1), FB = B->FB, P = B->primfact, E = B->exprimfact;
-
+  GEN F, x = gel(f,1), FB = B->FB, P = B->primfact, E = B->exprimfact;
+  if (B->badprim && !is_pm1(gcdii(x, B->badprim))) return 0;
+  F =  Z_isquasismooth_prod(x, B->prodFB);
+  if (cmpiu(F, B->limhash) > 0) return 0;
   for (i=1; lgefint(x) > 3; i++)
   {
     ulong p = uel(FB,i), r;
@@ -237,12 +254,7 @@ factorquad(struct buch_quad *B, GEN f, long nFB, ulong limp)
   }
 END:
   if (X > B->limhash) return 0;
-  if (X != 1 && X <= limp)
-  {
-    if (B->badprim && ugcdui(X, B->badprim) > 1) return 0;
-    lo++; P[lo] = X; E[lo] = 1;
-    X = 1;
-  }
+  if (X != 1 && X <= limp) { lo++; P[lo] = X; E[lo] = 1; X = 1; }
   P[0] = lo; return X;
 }
 
@@ -455,6 +467,7 @@ FBquad(struct buch_quad *B, ulong C2, ulong C1, GRHcheck_t *S)
   {
     B->badprim = NULL; set_avma(av);
   }
+  B->prodFB = zv_prod_Z(B->FB);
 }
 
 /* create B->vperm, return B->subFB */
@@ -698,6 +711,13 @@ imag_be_honest(struct buch_quad *B)
   return 1;
 }
 
+static GEN
+dist(GEN e, GEN d, long prec)
+{
+  GEN t = qfr5_dist(e, d, prec);
+  return signe(d) < 0 ? mkcomplex(t, gen_1): t;
+}
+
 /* Real Quadratic fields */
 
 static void
@@ -812,7 +832,7 @@ CYCLE:
         for (i=1; i<lgsub; i++) col[B->subFB[i]] += fpd[i]-ex[i];
         sub_fact(B,col, form2);
         if (fpd[-2]) col[fpd[-2]]++;
-        d = qfr5_dist(subii(gel(form1,4),gel(form2,4)),
+        d = dist(subii(gel(form1,4),gel(form2,4)),
                       divrr(gel(form1,5),gel(form2,5)), prec);
       }
       else
@@ -820,7 +840,7 @@ CYCLE:
         for (i=1; i<lgsub; i++) col[B->subFB[i]] += -fpd[i]-ex[i];
         add_fact(B, col, form2);
         if (fpd[-2]) col[fpd[-2]]--;
-        d = qfr5_dist(addii(gel(form1,4),gel(form2,4)),
+        d = dist(addii(gel(form1,4),gel(form2,4)),
                       mulrr(gel(form1,5),gel(form2,5)), prec);
       }
       if (DEBUGLEVEL>2) err_printf(" %ldP",s);
@@ -839,10 +859,10 @@ CYCLE:
       col = gel(mat,++s);
       for (i=1; i<lgsub; i++) col[B->subFB[i]] = -ex[i];
       add_fact(B, col, form1);
-      d = qfr5_dist(gel(form1,4), gel(form1,5), prec);
+      d = dist(gel(form1,4), gel(form1,5), prec);
       if (DEBUGLEVEL>2) err_printf(" %ld",s);
     }
-    affrr(d, gel(C,s));
+    gaffect(d, gel(C,s));
     if (first)
     {
       if (s >= lim) continue;
@@ -883,29 +903,28 @@ real_be_honest(struct buch_quad *B)
   return 1;
 }
 
+static GEN
+crabs(GEN a)
+{
+  return signe(real_i(a)) < 0 ? gneg(a): a;
+}
+
 static GEN
 gcdreal(GEN a,GEN b)
 {
-  if (!signe(a)) return mpabs_shallow(b);
-  if (!signe(b)) return mpabs_shallow(a);
-  if (typ(a)==t_INT)
-  {
-    if (typ(b)==t_INT) return gcdii(a,b);
-    a = itor(a, lg(b));
-  }
-  else if (typ(b)==t_INT)
-    b = itor(b, lg(a));
-  if (expo(a)<-5) return absr(b);
-  if (expo(b)<-5) return absr(a);
-  a = absr(a); b = absr(b);
-  while (expo(b) >= -5  && signe(b))
+  if (!signe(real_i(a))) return crabs(b);
+  if (!signe(real_i(b))) return crabs(a);
+  if (expo(real_i(a))<-5) return crabs(b);
+  if (expo(real_i(b))<-5) return crabs(a);
+  a = crabs(a); b = crabs(b);
+  while (expo(real_i(b)) >= -5  && signe(real_i(b)))
   {
     long e;
-    GEN r, q = gcvtoi(divrr(a,b),&e);
+    GEN r, q = gcvtoi(divrr(real_i(a),real_i(b)),&e);
     if (e > 0) return NULL;
-    r = subrr(a, mulir(q,b)); a = b; b = r;
+    r = gsub(a, gmul(q,b)); a = b; b = r;
   }
-  return mpabs_shallow(a);
+  return crabs(a);
 }
 
 static int
@@ -914,23 +933,22 @@ get_R(struct buch_quad *B, GEN C, long sreg, GEN z, GEN *ptR)
   GEN R = gen_1;
   double c;
   long i;
-
   if (B->PRECREG)
   {
-    R = mpabs_shallow(gel(C,1));
+    R = crabs(gel(C,1));
     for (i=2; i<=sreg; i++)
     {
       R = gcdreal(gel(C,i), R);
       if (!R) return fupb_PRECI;
     }
-    if (gexpo(R) <= -3)
+    if (gexpo(real_i(R)) <= -3)
     {
       if (DEBUGLEVEL>2) err_printf("regulator is zero.\n");
       return fupb_RELAT;
     }
     if (DEBUGLEVEL>2) err_printf("#### Tentative regulator: %Ps\n",R);
   }
-  c = gtodouble(gmul(z, R));
+  c = gtodouble(gmul(z, real_i(R)));
   if (c < 0.8 || c > 1.3) return fupb_RELAT;
   *ptR = R; return fupb_NONE;
 }
@@ -973,7 +991,7 @@ Buchquad_i(GEN D, double cbach, double cbach2, long prec)
       retmkvec4(gen_1, cgetg(1,t_VEC), cgetg(1,t_VEC), gen_1);
     prec = BQ.PRECREG = 0;
   } else {
-    BQ.PRECREG = maxss(prec+EXTRAPRECWORD, nbits2prec(2*expi(q.D) + 128));
+    BQ.PRECREG = maxss(prec+EXTRAPREC64, nbits2prec(2*expi(q.D) + 128));
   }
   if (DEBUGLEVEL>2) timer_start(&T);
   BQ.primfact   = new_chunk(100);
@@ -1000,7 +1018,7 @@ Buchquad_i(GEN D, double cbach, double cbach2, long prec)
   minSFB = (expi(D) > 15)? 3: 2;
   init_GRHcheck(&GRHcheck, 2, BQ.PRECREG? 2: 0, LOGD);
   high = low = LIMC0 = maxss((long)(cbach2*LOGD2), 1);
-  LIMCMAX = (long)(6.*LOGD2);
+  LIMCMAX = (long)(4.*LOGD2);
   /* 97/1223 below to ensure a good enough approximation of residue */
   cache_prime_quad(&GRHcheck, expi(D) < 16 ? 97: 1223, D);
   while (!quadGRHchk(D, &GRHcheck, high))
@@ -1087,7 +1105,7 @@ START:
     if (BQ.PRECREG) {
       for (i = triv+1; i<=need; i++) {
         gel(mat,i) = zero_zv(BQ.KC);
-        gel(extraC,i) = cgetr(BQ.PRECREG);
+        gel(extraC,i) = mkcomplex(cgetr(BQ.PRECREG), cgeti(3));
       }
       real_relations(&BQ, need - triv, &current, s,LIMC,mat + triv,extraC + triv);
     } else {
@@ -1097,7 +1115,6 @@ START:
       }
       imag_relations(&BQ, need - triv, &current, LIMC,mat + triv);
     }
-
     if (!W)
       W = hnfspec_i(mat,BQ.vperm,&dep,&B,&C,nsubFB);
     else
@@ -1138,7 +1155,10 @@ START:
   gen = get_clgp(&BQ,W,&cyc);
   gunclone(BQ.subFB);
   gunclone(BQ.powsubFB);
-  return mkvec4(h, cyc, gen, R);
+  if (BQ.PRECREG)
+    return mkvec5(h, cyc, gen, real_i(R), mpodd(imag_i(R)) ? gen_m1:gen_1);
+  else
+    return mkvec4(h, cyc, gen, real_i(R));
 }
 GEN
 Buchquad(GEN D, double c, double c2, long prec)
