@@ -656,6 +656,7 @@ sumprimeloop(forprime_t *pT, GEN s, long N, GEN data, GEN S, GEN W, GEN WB,
     {
       ks = f(E, p, prec1);
       if (gequal0(ks)) ks = NULL;
+      if (ks && gequal1(ks)) ks = gen_1;
     }
     gp[2] = p;
     if (needlog)
@@ -855,14 +856,16 @@ mycallvec(void *f, ulong n, long prec)
 GEN
 parsumprimefun_worker(GEN gk, GEN s, GEN zerf, GEN data, GEN vW, GEN f)
 {
+  pari_sp av = avma;
   forprime_t T;
-  GEN W, WB;
+  GEN W, WB, S;
   long k = itou(gk), sq, N = data[4], STEP = data[5];
 
   v2unpack(vW, &W, &WB); sq = lg(W)-1;
   if (isintzero(f)) f = NULL;
   u_forprime_init(&T, k * STEP + sq + 1, minss(N, (k + 1) * STEP + sq));
-  return sumprimeloop(&T, s, N, data, zerf, W, WB, (void*)f, mycallvec);
+  S = sumprimeloop(&T, s, N, data, zerf, W, WB, (void*)f, mycallvec);
+  return gerepilecopy(av, S);
 }
 
 static GEN
@@ -873,6 +876,10 @@ vR_get_vW(GEN vR)
   return v2pack(W, WB);
 }
 
+static GEN
+halfconj(long both, GEN V)
+{ return both ? mkvec2(gel(V, 1), gconj(gel(V, 2))) : V; }
+
 static GEN
 pardirpowerssumfun_i(GEN f, ulong N, GEN s, long both, long prec)
 {
@@ -886,8 +893,9 @@ pardirpowerssumfun_i(GEN f, ulong N, GEN s, long both, long prec)
   worker = snm_closure(is_entry("_parsumprimefun_worker"),
                        mkvecn(5, s, zerf, data, vR_get_vW(vR), f? f: gen_0));
   worker2 = snm_closure(is_entry("_parsqf_worker"), mkvec2(vR, utoi(N)));
-  return gadd(parsum(gen_0, utoipos((N-1) / data[5]), worker),
-              parsum(gen_0, utoipos(maxss((N-1) / step - 1, 0)), worker2));
+  return halfconj(both,
+                  gadd(parsum(gen_0, utoipos((N-1) / data[5]), worker),
+                       parsum(gen_0, utoipos(maxss((N-1) / step - 1, 0)), worker2)));
 }
 
 GEN
