@@ -492,7 +492,7 @@ cert_get_s(GEN x) { return gel(x,3); }
 INLINE GEN
 cert_get_q(GEN x) { return diviiexact(cert_get_m(x), cert_get_s(x)); }
 INLINE GEN
-cert_get_qlast(GEN x) { return cert_get_q( gel(x, lg(x)-1) ); }
+cert_get_qlast(GEN x) { return cert_get_q(veclast(x)); }
 INLINE GEN
 cert_get_a4(GEN x) { return gel(x,4); }
 INLINE GEN
@@ -516,20 +516,6 @@ cert_get_J(GEN x)
   return Fp_ellj(a, b, N);
 }
 
-/* Given J, N, set A = 3*J*(1728-J) mod N, B = 2*J*(1728-J)^2 mod N */
-static void
-Fp_ellfromj(GEN j, GEN N, GEN *A, GEN *B)
-{
-  GEN k, jk;
-  j = Fp_red(j, N);
-  if (isintzero(j)) { *A = gen_0; *B = gen_1; return; }
-  if (absequalui(umodui(1728,N), j)) { *A = gen_1; *B = gen_0; return; }
-  k = Fp_sub(utoi(1728), j, N);
-  jk = Fp_mul(j, k, N);
-  *A = Fp_mulu(jk, 3, N);
-  *B = Fp_mulu(Fp_mul(j, Fp_sqr(k,N), N), 2, N);
-}
-
 /* "Twist factor". Does not cover J = 0, 1728 */
 static GEN
 cert_get_lambda(GEN z)
@@ -539,7 +525,7 @@ cert_get_lambda(GEN z)
   N = cert_get_N(z);
   a = cert_get_a4(z);
   b = cert_get_a6(z);
-  Fp_ellfromj(J, N, &A, &B);
+  Fp_ellj_to_a4a6(J, N, &A, &B);
   return Fp_div(Fp_mul(a,B,N), Fp_mul(b,A,N), N);
 }
 
@@ -609,7 +595,8 @@ j0_find_g(GEN N)
 static GEN
 find_EP(GEN N, long D, GEN q, GEN g, GEN J, GEN s)
 {
-  GEN A0, B0; Fp_ellfromj(J, N, &A0, &B0);
+  GEN A0, B0;
+  Fp_ellj_to_a4a6(J, N, &A0, &B0);
   for(;;)
   { /* expect one iteration: not worth saving the A's and B's */
     GEN gg, v, A = A0, B = B0;
@@ -1540,13 +1527,14 @@ primecertisvalid_ecpp_worker(GEN certi)
   if (typ(P) != t_VEC || lg(P) != 3) return gen_0;
   P = FpE_to_FpJ(P);
 
-  /* Check mP == 0 */
-  mP = FpJ_mul(P, m, a, N);
-  if (!FpJ_is_inf(mP)) return gen_0;
-
   /* Check sP != 0 and third component is coprime to N */
   sP = FpJ_mul(P, s, a, N);
   if (!isint1(gcdii(gel(sP, 3), N))) return gen_0;
+
+  /* Check mP == 0 */
+  mP = FpJ_mul(sP, q, a, N);
+  if (!FpJ_is_inf(mP)) return gen_0;
+
   return q;
 }
 
@@ -1564,7 +1552,7 @@ ecppisvalid_i(GEN cert)
     return BPSW_psp(cert);
   }
   if (typ(cert) != t_VEC || lgcert <= 1) return 0;
-  ql = gel(cert, lgcert-1); if (lg(ql)-1 != 5) return 0;
+  ql = veclast(cert); if (lg(ql)-1 != 5) return 0;
   ql = cert_get_q(ql);
   if (expi(ql) >= trustbits || !BPSW_psp(ql)) return 0;
   worker = strtofunction("_primecertisvalid_ecpp_worker");
