@@ -141,6 +141,102 @@ ellmodulareqn(long ell, long vx, long vy)
   return gerepilecopy(av, mkvec2(meqn.eq, meqn.type=='A'? gen_1: gen_0));
 }
 
+/***********************************************************************/
+/**                                                                   **/
+/**                           FqE_group                               **/
+/**                                                                   **/
+/***********************************************************************/
+
+static GEN
+Fq_to_Flx(GEN a4, GEN T, ulong p)
+{ return typ(a4)==t_INT ? Z_to_Flx(a4, p, get_Flx_var(T)): ZX_to_Flx(a4, p); }
+
+/*FIXME: the name of the function does not quite match what it does*/
+static const struct bb_group *
+get_FqE_group(void ** pt_E, GEN a4, GEN a6, GEN T, GEN p)
+{
+  if (!T) return get_FpE_group(pt_E,a4,a6,p);
+  else if (lgefint(p)==3)
+  {
+    ulong pp = uel(p,2);
+    GEN Tp = ZXT_to_FlxT(T,pp);
+    return get_FlxqE_group(pt_E, Fq_to_Flx(a4, Tp, pp), Fq_to_Flx(a6, Tp, pp),
+                           Tp, pp);
+  }
+  return get_FpXQE_group(pt_E,a4,a6,T,p);
+}
+
+/***********************************************************************/
+/**                                                                   **/
+/**   Handle curves with CM by small order                            **/
+/**                                                                   **/
+/***********************************************************************/
+
+/* l odd prime. Return the list of discriminants D such that
+ *   polclass(D) | poldisc(polmodular(l)) */
+static GEN
+list_singular_discs(long l)
+{
+  const long _4l2 = 4*l*l;
+  long v;
+  GEN V = zero_F2v(_4l2);
+  /* special cased for efficiency + remove factor l^2 from conductor */
+  F2v_set(V, 4); /* v = 0 */
+  F2v_set(V, 3); /* v = l */
+  for (v = 1; v < 2*l; v++)
+    if (v != l)
+    { /* l does not divide _4l2 - v*v */
+      GEN F = factoru(_4l2 - v*v), P, E, c;
+      ulong d = coredisc2u_fact(F, -1, &P, &E);
+      long i, lc;
+      c = divisorsu_fact(mkvec2(P,E));
+      lc = lg(c);
+      for (i = 1; i < lc; i++)
+        F2v_set(V, d * uel(c,i)*uel(c,i));
+    }
+  return V;
+}
+
+/* l odd prime. Find D such that j has CM by D, assuming
+ * subst(polmodular(l),x,j) has a double root */
+static long
+find_CM(long l, GEN j, GEN T, GEN p)
+{
+  const long inv = 0;
+  GEN v = list_singular_discs(l);
+  long i, n = v[1];
+  GEN db = polmodular_db_init(inv);
+  for (i = 1; i < n; i++)
+    if (F2v_coeff(v,i))
+    {
+      GEN C = polclass0(-i, inv, 0, &db);
+      GEN F = FqX_eval(C, j, T, p);
+      if (signe(F)==0) break;
+    }
+  gunclone_deep(db); return i < n ? -i: 0;
+}
+
+static GEN
+vecpoints_to_vecx(GEN x, GEN q1)
+{
+  pari_APPLY_type(t_COL, gadd(q1, signe(gmael(x,i,2)) > 0 ? gmael(x,i,1)
+                                                          : negi(gmael(x,i,1))));
+}
+
+static GEN
+Fq_ellcard_CM(long disc, GEN a4, GEN a6, GEN T, GEN p)
+{
+  const struct bb_group *grp;
+  void *E;
+  long d = T ? degpol(T): 1;
+  GEN q = powiu(p, d), q1 = addiu(q, 1), Q, S;
+  Q = qfbsolve(Qfb0(gen_1,gen_0,stoi(-disc)), mkmat22(gen_2, gen_2, p, utoi(d)), 3);
+  if (lg(Q)==1) return q1;
+  S = vecpoints_to_vecx(Q, q1);
+  grp = get_FqE_group(&E, a4, a6, T, p);
+  return gen_select_order(S, E, grp);
+}
+
 /***********************************************************************/
 /**                                                                   **/
 /**                      n-division polynomial                        **/
@@ -349,6 +445,19 @@ Flxq_elldivpolmod_init(struct divpolmod_red *d, GEN a4, GEN a6, long n, GEN h, G
   divpolmod_init(d, D3, D4, RHS, n, E, ff);
 }
 
+/*Computes the n-division polynomial modulo the polynomial h \in Fq[x] */
+GEN
+Flxq_elldivpolmod(GEN a4, GEN a6, long n, GEN h, GEN T, ulong p)
+{
+  struct divpolmod_red d;
+  pari_sp ltop = avma;
+  GEN res;
+  Flxq_elldivpolmod_init(&d, a4, a6, n, h, T, p);
+  res = gcopy(divpol(d.t,d.r2,n,d.E,d.ff));
+  divpol_free(d.t);
+  return gerepileupto(ltop, res);
+}
+
 /*Computes the n-division polynomial modulo the polynomial h \in Fq[x] */
 GEN
 Fq_elldivpolmod(GEN a4, GEN a6, long n, GEN h, GEN T, GEN p)
@@ -356,6 +465,14 @@ Fq_elldivpolmod(GEN a4, GEN a6, long n, GEN h, GEN T, GEN p)
   struct divpolmod_red d;
   pari_sp ltop = avma;
   GEN res;
+  if (lgefint(p)==3 && T)
+  {
+    ulong pp = p[2];
+    GEN a4p = ZX_to_Flx(a4,pp), a6p = ZX_to_Flx(a6,pp);
+    GEN hp = ZXX_to_FlxX(h, pp, get_FpX_var(T)), Tp = ZXT_to_FlxT(T , pp);
+    res = Flxq_elldivpolmod(a4p, a6p, n, hp, Tp, pp);
+    return gerepileupto(ltop, FlxX_to_ZXX(res));
+  }
   Fq_elldivpolmod_init(&d, a4, a6, n, h, T, p);
   res = gcopy(divpol(d.t,d.r2,n,d.E,d.ff));
   divpol_free(d.t);
@@ -617,10 +734,6 @@ Zq_ellj(GEN a4, GEN a6, GEN T, GEN p, GEN pp, long e)
 /*                              EIGENVALUE                                  */
 /****************************************************************************/
 
-static GEN
-Fq_to_Flx(GEN a4, GEN T, ulong p)
-{ return typ(a4)==t_INT ? Z_to_Flx(a4, p, get_Flx_var(T)): ZX_to_Flx(a4, p); }
-
 static GEN
 Flxq_find_eigen_Frobenius(GEN a4, GEN a6, GEN h, GEN T, ulong p)
 {
@@ -733,8 +846,7 @@ static GEN
 find_kernel(GEN a4, GEN a6, long l, GEN b4, GEN b6, GEN pp1, GEN T, GEN p, GEN pp, long e)
 {
   GEN Ge, S, Sd;
-  const long ext = 1;
-  long d = (l+1)/2+ext;
+  long d = ((l+1)>>1)+1;
   if(l==3)
     return deg1pol(gen_1, Fq_neg(pp1, T, p), 0);
   S = Zq_Weierstrass(a4, a6, b4, b6, d + 1, T, p, pp, e);
@@ -747,7 +859,7 @@ find_kernel(GEN a4, GEN a6, long l, GEN b4, GEN b6, GEN pp1, GEN T, GEN p, GEN p
   Ge = T && lgefint(pp)==3 ? ZlXQXn_expint(Ge, d, T, p, pp[2])
                            : FqXn_expint(Ge, d, T, p);
   Ge = RgX_recip(FqX_red(Ge, T, pp));
-  if (degpol(Ge)==(l-1)/2) return Ge;
+  if (degpol(Ge)==(l-1)>>1) return Ge;
   return NULL;
 }
 
@@ -1106,23 +1218,11 @@ find_isogenous_from_J(GEN a4, GEN a6, ulong ell, struct meqn *MEQN, GEN g, GEN T
   return gerepilecopy(ltop, mkvec3(a4t, a6t, h));
 }
 
-static long
-newtonlogint(ulong n, ulong pp)
-{
-  long s = 0;
-  while (n > pp)
-  {
-    s += ulogint(n, pp);
-    n = (n+1)>>1;
-  }
-  return s;
-}
-
 static GEN
 find_isogenous(GEN a4,GEN a6, ulong ell, struct meqn *MEQN, GEN g, GEN T,GEN p)
 {
   ulong pp = itou_or_0(p);
-  long e = pp ? newtonlogint(1+(ell>>1), pp) + ulogint(2*ell+4, pp) + 1: 1;
+  long e = pp ? ulogint(((ell+1)>>1)+1, pp) + ulogint(2*ell+4, pp) + 1: 1;
   if (signe(a4)==0 || signe(a6)==0)
   {
     if (DEBUGLEVEL>0) err_printf("[%c: j=%ld]",MEQN->type,signe(a4)==0 ?0: 1728);
@@ -1205,7 +1305,7 @@ find_kernel_power(GEN Eba4, GEN Eba6, GEN Eca4, GEN Eca6, ulong ell, struct meqn
 /****************************************************************************/
 /*                                  TRACE                                   */
 /****************************************************************************/
-enum mod_type {MTpathological, MTAtkin, MTElkies, MTone_root, MTroots};
+enum mod_type {MTcm, MTpathological, MTAtkin, MTElkies, MTone_root, MTroots};
 
 static GEN
 Flxq_study_eqn(GEN mpoly, GEN T, ulong p, long *pt_dG, long *pt_r)
@@ -1258,7 +1358,7 @@ study_modular_eqn(long ell, GEN mpoly, GEN T, GEN p, enum mod_type *mt, long *pt
   pari_sp ltop = avma;
   GEN g = gen_0;
   *ptr_r = 0; /*gcc -Wall*/
-  if (!FqX_is_squarefree(mpoly, T, p)) *mt = MTpathological;
+  if (!FqX_is_squarefree(mpoly, T, p)) *mt = MTcm;
   else
   {
     long dG;
@@ -1278,6 +1378,7 @@ study_modular_eqn(long ell, GEN mpoly, GEN T, GEN p, enum mod_type *mt, long *pt
     case MTroots: err_printf("l+1 roots\t"); break;
     case MTAtkin: err_printf("Atkin\t"); break;
     case MTpathological: err_printf("Pathological\n"); break;
+    case MTcm: err_printf("CM\t"); break;
   }
   return g ? gerepilecopy(ltop, g): NULL;
 }
@@ -1389,7 +1490,7 @@ find_trace_lp1_roots(long ell, GEN q)
   return mkvecsmall2(pa, ell2 - pa);
 }
 
-/*trace modulo ell^k: [], [t] or [t1,...,td] */
+/*ell odd prime; trace modulo ell^k: [], [t] or [t1,...,td] */
 static GEN
 find_trace(GEN a4, GEN a6, GEN j, ulong ell, GEN q, GEN T, GEN p, long *ptr_kt,
   long smallfact, long vx, long vy)
@@ -1437,6 +1538,13 @@ find_trace(GEN a4, GEN a6, GEN j, ulong ell, GEN q, GEN T, GEN p, long *ptr_kt,
     if (lg(tr)==1) pari_err_PRIME("ellap",p);
     kt = 1;
     break;
+  case MTcm:
+    {
+      long D = find_CM(ell, j, T, p);
+      GEN C = Fq_ellcard_CM(D, a4, a6, T, p);
+      if (DEBUGLEVEL>1) err_printf(" D=%ld [%ld ms]\n", D, timer_delay(&ti));
+      return gc_const(ltop, C);
+    }
   default: /* case MTpathological: */
     return gc_NULL(ltop);
   }
@@ -1855,21 +1963,6 @@ get_bound_bsgs(long lp)
   return mulru(B, 1000000);
 }
 
-/*FIXME: the name of the function does not quite match what it does*/
-static const struct bb_group *
-get_FqE_group(void ** pt_E, GEN a4, GEN a6, GEN T, GEN p)
-{
-  if (!T) return get_FpE_group(pt_E,a4,a6,p);
-  else if (lgefint(p)==3)
-  {
-    ulong pp = uel(p,2);
-    GEN Tp = ZXT_to_FlxT(T,pp);
-    return get_FlxqE_group(pt_E, Fq_to_Flx(a4, Tp, pp), Fq_to_Flx(a6, Tp, pp),
-                           Tp, pp);
-  }
-  return get_FpXQE_group(pt_E,a4,a6,T,p);
-}
-
 /* E is an elliptic curve defined over Z or over Fp in ellinit format, defined
  * by the equation E: y^2 + a1*x*y + a2*y = x^3 + a2*x^2 + a4*x + a6
  * p is a prime number
@@ -1893,6 +1986,8 @@ Fq_ellcard_SEA(GEN a4, GEN a6, GEN q, GEN T, GEN p, long smallfact)
   if (signe(j) == 0 || signe(Fq_sub(j, utoi(1728), T, p)) == 0)
     return T ? FpXQ_ellcard(Fq_to_FpXQ(a4, T, p), Fq_to_FpXQ(a6, T, p), T, p)
              : Fp_ellcard(a4, a6, p);
+  if (Fq_elljissupersingular(j, T, p))
+    return Fq_ellcard_supersingular(a4, a6, T, p);
   /*First compute the trace modulo 2 */
   switch(FqX_nbroots(rhs(a4, a6, 0), T, p))
   {
@@ -1929,7 +2024,11 @@ Fq_ellcard_SEA(GEN a4, GEN a6, GEN q, GEN T, GEN p, long smallfact)
     if (absequalui(ell, p)) continue;
     trace_mod = find_trace(a4, a6, j, ell, q, T, p, &kt, smallfact, vx,vy);
     if (!trace_mod) continue;
-
+    if (typ(trace_mod)==t_INT)
+    {
+      delete_var(); delete_var();
+      return gerepileuptoint(ltop, trace_mod);
+    }
     nbtrace = lg(trace_mod) - 1;
     ellkt = (long)upowuu(ell, kt);
     if (nbtrace == 1)
@@ -1947,9 +2046,8 @@ Fq_ellcard_SEA(GEN a4, GEN a6, GEN q, GEN T, GEN p, long smallfact)
           if (DEBUGLEVEL)
             err_printf("\nAborting: #E%s(Fq) divisible by %ld\n",
                        tcard_mod_ell ? "" : "_twist", ell);
-          delete_var();
-          delete_var();
-          set_avma(ltop); return gen_0;
+          delete_var(); delete_var();
+          return gc_const(ltop, gen_0);
         }
       }
       (void)Z_incremental_CRT(&TR, t_mod_ellkt, &TR_mod, ellkt);
@@ -1966,8 +2064,7 @@ Fq_ellcard_SEA(GEN a4, GEN a6, GEN q, GEN T, GEN p, long smallfact)
       GEN bound_tr;
       if (!nb_atkin)
       {
-        delete_var();
-        delete_var();
+        delete_var(); delete_var();
         return gerepileuptoint(ltop, subii(addiu(q, 1), TR));
       }
       bound_tr = mulrr(bound_bsgs, dbltor(bound_gr));
