@@ -232,11 +232,21 @@ rfrac_denom_mul_scal(GEN d, GEN y)
   return D;
 }
 
+static int
+Leading_is_neg(GEN x)
+{
+  while (typ(x) == t_POL) x = leading_coeff(x);
+  return is_real_t(typ(x))? gsigne(x) < 0: 0;
+}
+
+static int
+transtype(GEN x) { return x != gen_1 && typ(x) != t_PADIC; }
+
 /* d a t_POL, n a coprime t_POL of same var or "scalar". Not memory clean */
 GEN
 gred_rfrac_simple(GEN n, GEN d)
 {
-  GEN c, cn, cd, z;
+  GEN _1n, _1d, c, cn, cd, z;
   long dd = degpol(d);
 
   if (dd <= 0)
@@ -246,7 +256,11 @@ gred_rfrac_simple(GEN n, GEN d)
     if (typ(n) != t_POL || varn(n) != varn(d)) n = scalarpol(n, varn(d));
     return n;
   }
-
+  if (Leading_is_neg(d)) { d = gneg(d); n = gneg(n); }
+  _1n = Rg_get_1(n);
+  _1d = Rg_get_1(d);
+  if (transtype(_1n) && !gidentical(_1n, _1d)) d = gmul(d, _1n);
+  if (transtype(_1d) && !gidentical(_1n, _1d)) n = gmul(n, _1d);
   cd = content(d);
   while (typ(n) == t_POL && !degpol(n)) n = gel(n,2);
   cn = (typ(n) == t_POL && varn(n) == varn(d))? content(n): n;
@@ -326,7 +340,7 @@ fix_rfrac(GEN x, long d)
 static GEN
 gred_rfrac2(GEN n, GEN d)
 {
-  GEN y, z;
+  GEN y, z, _1n, _1d;
   long v, vd, vn;
 
   n = simplify_shallow(n);
@@ -343,6 +357,10 @@ gred_rfrac2(GEN n, GEN d)
   vn = varn(n);
   if (varncmp(vd, vn) < 0) return gred_rfrac_simple(n,d);
   if (varncmp(vd, vn) > 0) return RgX_Rg_div(n,d);
+  _1n = Rg_get_1(n);
+  _1d = Rg_get_1(d);
+  if (transtype(_1n) && !gidentical(_1n, _1d)) d = gmul(d, _1n);
+  if (transtype(_1d) && !gidentical(_1n, _1d)) n = gmul(n, _1d);
 
   /* now n and d are t_POLs in the same variable */
   v = RgX_valrem(n, &n) - RgX_valrem(d, &d);
@@ -753,7 +771,7 @@ add_ser_scal(GEN y, GEN x)
 
   if (isrationalzero(x)) return gcopy(y);
   ly = lg(y);
-  v = valp(y);
+  v = valser(y);
   if (v < 3-ly) return gcopy(y);
   /* v + ly >= 3 */
   if (v < 0)
@@ -783,7 +801,7 @@ add_ser_scal(GEN y, GEN x)
   }
   for (; i<ly; i++) gel(z,i) = gcopy(gel(y,i));
   gel(z,2) = x;
-  z[1] = evalsigne(1) | _evalvalp(0) | evalvarn(vy);
+  z[1] = evalsigne(1) | _evalvalser(0) | evalvarn(vy);
   return gequal0(x)? normalizeser(z): z;
 }
 static long
@@ -792,15 +810,15 @@ _serprec(GEN x) { return ser_isexactzero(x)? 2: lg(x); }
 static GEN
 ser_add(GEN x, GEN y)
 {
-  long i, lx,ly, n = valp(y) - valp(x);
+  long i, lx,ly, n = valser(y) - valser(x);
   GEN z;
   if (n < 0) { n = -n; swap(x,y); }
-  /* valp(x) <= valp(y) */
+  /* valser(x) <= valser(y) */
   lx = _serprec(x);
   if (lx == 2) /* don't lose type information */
   {
     z = scalarser(gadd(Rg_get_0(x), Rg_get_0(y)), varn(x), 1);
-    setvalp(z, valp(x)); return z;
+    setvalser(z, valser(x)); return z;
   }
   ly = _serprec(y) + n; if (lx < ly) ly = lx;
   if (n)
@@ -1077,7 +1095,7 @@ gadd(GEN x, GEN y)
           av=avma; z=addir(gel(y,1),mulir(gel(y,2),x)); tetpil=avma;
           return gerepile(av,tetpil,divri(z,gel(y,2)));
         case t_COMPLEX: return addRc(x, y);
-        case t_QUAD: return gequal0(y)? rcopy(x): addqf(y, x, lg(x));
+        case t_QUAD: return gequal0(y)? rcopy(x): addqf(y, x, realprec(x));
 
         default: pari_err_TYPE2("+",x,y);
       }
@@ -1171,7 +1189,7 @@ gadd(GEN x, GEN y)
         case t_SER:
           if (lg(x) == 2) return gcopy(y);
           i = RgX_val(x); if (i == LONG_MAX) i = 0; /* e.g. x = Mod(0,3)*x^0 */
-          i = lg(y) + valp(y) - i;
+          i = lg(y) + valser(y) - i;
           if (i < 3) return gcopy(y);
           p1 = RgX_to_ser(x,i); y = ser_add(p1,y);
           settyp(p1, t_VECSMALL); /* p1 left on stack */
@@ -1190,7 +1208,7 @@ gadd(GEN x, GEN y)
         vd = RgX_valrem_inexact(gel(y,2), NULL);
         if (vd == LONG_MAX) pari_err_INV("gadd", gel(y,2));
 
-        l = lg(x) + valp(x) - (vn - vd);
+        l = lg(x) + valser(x) - (vn - vd);
         if (l < 3) { set_avma(av); return gcopy(x); }
         return gerepileupto(av, gadd(x, rfrac_to_ser_i(y, l)));
       }
@@ -1342,7 +1360,7 @@ mul_ser_scal(GEN y, GEN x)
   if (ser_isexactzero(y))
   {
     z = scalarser(lg(y) == 2? Rg_get_0(x): gmul(gel(y,2), x), varn(y), 1);
-    setvalp(z, valp(y)); return z;
+    setvalser(z, valser(y)); return z;
   }
   z = cgetg_copy(y, &l); z[1] = y[1];
   for (i = 2; i < l; i++) gel(z,i) = gmul(gel(y,i), x);
@@ -1793,24 +1811,27 @@ mulcxpowIs(GEN x, long k)
   return x;
 }
 
+/* caller will assume l > 2 later */
 static GEN
 init_ser(long l, long v, long e)
 {
   GEN z = cgetg(l, t_SER);
-  z[1] = evalvalp(e) | evalvarn(v) | evalsigne(1); return z;
+  z[1] = evalvalser(e) | evalvarn(v) | evalsigne(1); return z;
 }
 
 /* fill in coefficients of t_SER z from coeffs of t_POL y */
 static GEN
 fill_ser(GEN z, GEN y)
 {
-  long i, lx = lg(z), ly = lg(y);
+  long i, lx = lg(z), ly = lg(y); /* lx > 2 */
   if (ly >= lx) {
     for (i = 2; i < lx; i++) gel(z,i) = gel(y,i);
   } else {
     for (i = 2; i < ly; i++) gel(z,i) = gel(y,i);
     for (     ; i < lx; i++) gel(z,i) = gen_0;
   }
+  /* dangerous case (already normalized), don't use normalizeser */
+  if (ly == 3 && !signe(y)) { setsigne(z, 0); return z; }
   return normalizeser(z);
 }
 /* assume typ(x) = t_VEC */
@@ -1880,8 +1901,8 @@ gmul(GEN x, GEN y)
         return mul_ser_scal(y, x);
       }
       lx = minss(lg(x), lg(y));
-      if (lx == 2) return zeroser(vx, valp(x)+valp(y));
-      av = avma; z = init_ser(lx, vx, valp(x)+valp(y));
+      if (lx == 2) return zeroser(vx, valser(x)+valser(y));
+      av = avma; z = init_ser(lx, vx, valser(x)+valser(y));
       x = ser2pol_i(x, lx);
       y = ser2pol_i(y, lx);
       y = RgXn_mul(x, y, lx-2);
@@ -2072,7 +2093,7 @@ gmul(GEN x, GEN y)
           long v;
           av = avma; v = RgX_valrem(x, &x);
           if (v == LONG_MAX) return gerepileupto(av, Rg_get_0(x));
-          v += valp(y); ly = lg(y);
+          v += valser(y); ly = lg(y);
           if (ly == 2) { set_avma(av); return zeroser(vx, v); }
           if (degpol(x))
           {
@@ -2082,7 +2103,7 @@ gmul(GEN x, GEN y)
           }
           /* take advantage of x = c*t^v */
           set_avma(av); y = mul_ser_scal(y, gel(x,2));
-          setvalp(y, v); return y;
+          setvalser(y, v); return y;
         }
 
         case t_RFRAC: return mul_rfrac_scal(gel(y,1),gel(y,2), x);
@@ -2110,10 +2131,10 @@ sqr_ser_part(GEN x, long l1, long l2)
   pari_sp av;
   GEN Z, z, p1, p2;
   long mi;
-  if (l2 < l1) return zeroser(varn(x), 2*valp(x));
+  if (l2 < l1) return zeroser(varn(x), 2*valser(x));
   p2 = cgetg(l2+2, t_VECSMALL)+1; /* left on stack on exit */
   Z = cgetg(l2-l1+3, t_SER);
-  Z[1] = evalvalp(2*valp(x)) | evalvarn(varn(x));
+  Z[1] = evalvalser(2*valser(x)) | evalvarn(varn(x));
   z = Z + 2-l1;
   x += 2; mi = 0;
   for (i=0; i<l1; i++)
@@ -2209,7 +2230,7 @@ gsqr(GEN x)
       lx = lg(x);
       if (ser_isexactzero(x)) {
         GEN z = gcopy(x);
-        setvalp(z, 2*valp(x));
+        setvalser(z, 2*valser(x));
         return z;
       }
       if (lx < 40)
@@ -2217,7 +2238,7 @@ gsqr(GEN x)
       else
       {
         pari_sp av = avma;
-        GEN z = init_ser(lx, varn(x), 2*valp(x));
+        GEN z = init_ser(lx, varn(x), 2*valser(x));
         x = ser2pol_i(x, lx);
         x = RgXn_sqr(x, lx-2);
         return gerepilecopy(av, fill_ser(z,x));
@@ -2261,10 +2282,15 @@ static GEN
 div_scal_rfrac(GEN x, GEN y)
 {
   GEN y1 = gel(y,1), y2 = gel(y,2);
-  pari_sp av = avma;
   if (typ(y1) == t_POL && varn(y2) == varn(y1))
   {
-    if (degpol(y1)) return gerepileupto(av, gred_rfrac_simple(gmul(x, y2), y1));
+    if (degpol(y1))
+    {
+      pari_sp av = avma;
+      GEN _1 = Rg_get_1(x);
+      if (transtype(_1)) y1 = gmul(y1, _1);
+      return gerepileupto(av, gred_rfrac_simple(gmul(x, y2), y1));
+    }
     y1 = gel(y1,2);
   }
   return RgX_Rg_mul(y2, gdiv(x,y1));
@@ -2282,7 +2308,7 @@ div_ser_scal(GEN y, GEN x)
   if (ser_isexactzero(y))
   {
     z = scalarser(lg(y) == 2? Rg_get_0(x): gdiv(gel(y,2), x), varn(y), 1);
-    setvalp(z, valp(y)); return z;
+    setvalser(z, valser(y)); return z;
   }
   z = cgetg_copy(y, &l); z[1] = y[1];
   for (i = 2; i < l; i++) gel(z,i) = gdiv(gel(y,i), x);
@@ -2317,14 +2343,21 @@ static GEN
 div_scal_pol(GEN x, GEN y) {
   long ly = lg(y);
   pari_sp av;
+  GEN _1;
   if (ly == 3) return scalarpol(gdiv(x,gel(y,2)), varn(y));
   if (isrationalzero(x)) return zeropol(varn(y));
   av = avma;
+  _1 = Rg_get_1(x); if (transtype(_1)) y = gmul(y, _1);
   return gerepileupto(av, gred_rfrac_simple(x,y));
 }
 static GEN
 div_scal_ser(GEN x, GEN y)
-{ pari_sp av = avma; return gerepileupto(av, gmul(x, ser_inv(y))); }
+{
+  pari_sp av = avma;
+  GEN _1 = Rg_get_1(x);
+  if (transtype(_1)) y = gmul(y, _1);
+  return gerepileupto(av, gmul(x, ser_inv(y)));
+}
 static GEN
 div_scal_T(GEN x, GEN y, long ty) {
   switch(ty)
@@ -2341,7 +2374,7 @@ div_scal_T(GEN x, GEN y, long ty) {
 static GEN
 div_ser(GEN x, GEN y, long vx)
 {
-  long e, v = valp(x) - valp(y), lx = lg(x), ly = lg(y);
+  long e, v = valser(x) - valser(y), lx = lg(x), ly = lg(y);
   GEN y0 = y, z;
   pari_sp av = avma;
 
@@ -2350,7 +2383,7 @@ div_ser(GEN x, GEN y, long vx)
   {
     if (lx == 2) return zeroser(vx, v);
     z = scalarser(gmul(gel(x,2),Rg_get_0(y)), varn(x), 1);
-    setvalp(z, v); return z;
+    setvalser(z, v); return z;
   }
   if (lx < ly) ly = lx;
   y = ser2pol_i_normalize(y, ly, &e);
@@ -2524,9 +2557,9 @@ gdiv(GEN x, GEN y)
     case t_QFB: av = avma; return gerepileupto(av, qfbcomp(x, ginv(y)));
 
     case t_MAT:
-      av = avma; p1 = RgM_inv(y);
+      p1 = RgM_div(x,y);
       if (!p1) pari_err_INV("gdiv",y);
-      return gerepileupto(av, RgM_mul(x, p1));
+      return p1;
 
     default: pari_err_TYPE2("/",x,y);
   }
@@ -2791,7 +2824,7 @@ gdiv(GEN x, GEN y)
           long v;
           av = avma; v = RgX_valrem(x, &x);
           if (v == LONG_MAX) return gerepileupto(av, Rg_get_0(x));
-          v -= valp(y); ly = lg(y); /* > 2 */
+          v -= valser(y); ly = lg(y); /* > 2 */
           y = ser2pol_i_normalize(y, ly, &i);
           if (i)
           {
@@ -2818,7 +2851,7 @@ gdiv(GEN x, GEN y)
       {
         case t_POL:
         {
-          long v = valp(x);
+          long v = valser(x);
           lx = lg(x);
           if (lx == 2) return zeroser(vx, v - RgX_val(y));
           av = avma;
