@@ -29,7 +29,7 @@ static int
 isnegint2(GEN a, long *pa)
 {
   GEN b;
-  if (!gequal0(imag_i(a))) return 0;
+  if (!is_scalar_t(typ(a)) || !gequal0(imag_i(a))) return 0;
   a = real_i(a); if (gsigne(a) > 0) return 0;
   b = ground(a); if (!gequal(a, b)) return 0;
   if (pa) *pa = -itos(b);
@@ -79,7 +79,7 @@ airy_i(GEN x, long prec)
     A = invr(mulrr(s4, ggamma(uutoQ(2,3), prec)));
     B = mulrr(A, s3); return mkvec2(A, B);
   }
-  prec2 = prec + EXTRAPRECWORD;
+  prec2 = prec + EXTRAPREC64;
   x = gprec_wensure(x, prec2);
   z = gsqrt(gpowgs(x,3), prec2); z2 = gdivgu(gmul2n(z,1),3);
   if (is_real_t(tx) && gsigne(x) > 0)
@@ -103,10 +103,10 @@ GEN
 airy(GEN z, long prec)
 { pari_sp av = avma; return gerepilecopy(av, airy_i(z, prec)); }
 
-/* Gamma(a)*Gamma(b) */
+/* Gamma(a)*Gamma(b); allow a = NULL [omit] */
 static GEN
 mulgamma2(GEN a, GEN b, long prec)
-{ return gmul(ggamma(a, prec), ggamma(b, prec)); }
+{ GEN gb = ggamma(b, prec); return a? gmul(ggamma(a, prec), gb): gb; }
 /* Gamma(a)/Gamma(b) */
 static GEN
 divgamma2(GEN a, GEN b, long prec)
@@ -168,7 +168,7 @@ hyperu_i(GEN a, GEN b, GEN x, long prec)
   {
     if (typ(b) == t_INT)
     {
-      bit = prec2nbits(l); l += l-2;
+      bit = prec2nbits(l); l = precdbl(l);
       b = gadd(b, real2n(-bit, l));
       a = gprec_wensure(a, l);
       x = gprec_wensure(x, l);
@@ -176,7 +176,7 @@ hyperu_i(GEN a, GEN b, GEN x, long prec)
     return hyperu_F11(a, b, x, l, l);
   }
   bit = prec2nbits(l)-1;
-  l += EXTRAPRECWORD;
+  l += EXTRAPREC64;
   T = gadd(gadd(P, gmulsg(n-1, S)), sqru(n-1));
   x = gtofp(x, l);
   if (!bigx)
@@ -357,14 +357,6 @@ vpoch(GEN a, long n)
   for (j = 1; j < n; j++) gel(v,j+1) = gmul(gel(v,j), gaddsg(j, a));
   return v;
 }
-static GEN
-RgV_vpoch(GEN N, long n)
-{
-  long i, l;
-  GEN v = cgetg_copy(N, &l);
-  for (i = 1; i < l; i++) gel(v,i) = vpoch(gel(N,i), n);
-  return v;
-}
 
 static GEN
 Npoch(GEN a, long n) { return gnorm(poch(a, n, LOWDEFAULTPREC)); }
@@ -399,37 +391,37 @@ vnormpol2(GEN v)
   return P;
 }
 
+/* return an 'extraprec' */
 static long
 precFtaylor(GEN N, GEN D, GEN z, long *pmi)
 {
   GEN v, ma, P = vnormpol2(D), Q = vnormpol2(N), Nz = gnorm(z);
   double wma, logNz = (gexpo(Nz) < -27)? -27: dbllog2(Nz) / 2;
   long pr = LOWDEFAULTPREC, prec = precision(z);
-  long lN = lg(N), lD = lg(D), mi, j, i, lv;
+  long lN = lg(N), lD = lg(D), j, i, lv;
 
   P = RgX_shift_shallow(P, 2);
   /* avoid almost cancellation of leading coeff if |z| ~ 1 */
   if (!prec || fabs(logNz) > 1e-38) Q = RgX_Rg_mul(Q,Nz);
-  for (j = 1, ma = NULL; j < lN; ++j)
+  for (j = 1, ma = NULL; j < lN; j++)
   {
     GEN Nj = gel(N,j);
     if (isint(Nj,&Nj) && signe(Nj) <= 0
         && (!ma || abscmpii(ma, Nj) < 0)) ma = Nj;
   }
   /* use less sharp fujiwara_bound_real(,1) ? */
-  v = ground(realroots(gsub(P,Q), mkvec2(gen_0,mkoo()), pr));
-  v = ZV_to_zv(v); lv = lg(v);
+  v = realroots(gsub(P,Q), mkvec2(gen_0,mkoo()), pr);
+  v = ZV_to_zv(ground(v)); lv = lg(v);
   if (ma)
   {
     long sma = is_bigint(ma)? LONG_MAX: maxss(labs(itos(ma))-1, 1);
-    for (i = 1; i < lv; ++i) v[i] = maxss(minss(sma, v[i]), 1);
+    for (i = 1; i < lv; i++) v[i] = maxss(minss(sma, v[i]), 1);
   }
-  for (i = 1, wma = 0., mi = 0; i < lv; ++i)
+  for (i = 1, wma = 0.; i < lv; i++)
   {
     GEN t1 = gen_1, t2 = gen_1;
     long u = v[i];
     double t;
-    mi = maxss(mi, u);
     for (j = 1; j < lN; j++) t1 = gmul(t1, Npoch(gel(N,j), u));
     for (j = 1; j < lD; j++) t2 = gmul(t2, Npochden(gel(D,j), u));
     t = dbllog2(gdiv(t1,t2)) / 2 + u * logNz - dbllog2(mpfactr(u,pr));
@@ -437,7 +429,7 @@ precFtaylor(GEN N, GEN D, GEN z, long *pmi)
   }
   /* make up for exponential decrease in exp() */
   if (gsigne(real_i(z)) < 0) wma -= gtodouble(real_i(z)) / M_LN2;
-  *pmi = mi; return ceil(wma/BITS_IN_LONG) + 1;
+  *pmi = maxss(v[lv-1],1); return nbits2extraprec(wma+BITS_IN_LONG);
 }
 
 static GEN
@@ -465,7 +457,7 @@ Ftaylor(GEN N, GEN D, GEN z, long prec)
     C = gmul(C, gmul(gdiv(a, gmulsg(j+1, b)), z));
     if (gequal0(C)) break;
     if (j > mi) tol = gequal0(S)? 0: gexpo(C) - gexpo(S);
-    S = gadd(S, C); ++j;
+    S = gadd(S, C); j++;
     if (j > mi)
     { if (tol > bitmin) ct = 0; else if (++ct >= lN+lD-2) break; }
     if (gc_needed(av, 1)) gerepileall(av, 2, &S, &C);
@@ -480,7 +472,7 @@ bind(GEN a, GEN b, GEN c, long ind)
   {
     case 1: case 2: return gsub(c, b);
     case 5: case 6: return gsub(gaddsg(1, a), c);
-    default: return b;
+    default: return b; /* 3,4 */
   }
 }
 static GEN
@@ -490,7 +482,7 @@ cind(GEN a, GEN b, GEN c, long ind)
   {
     case 1: case 6: return gaddsg(1, gsub(a,b));
     case 4: case 5: return gsub(gaddsg(1, gadd(a,b)), c);
-    default: return c;
+    default: return c; /* 2,3 */
   }
 }
 static GEN
@@ -503,72 +495,60 @@ zind(GEN z, long ind)
     case 4: return gsubsg(1, z);
     case 5: return gsubsg(1, ginv(z));
     case 6: return ginv(z);
+    default: return z; /* 3, LCOV_EXCL_LINE*/
   }
-  return z;
 }
 
-/* z not 0 or 1, c not a nonpositive integer */
+/* z not 0 or 1, c not a nonpositive integer. Among the generic 6
+ * Kummer transforms 1/(1-z),z/(z-1),z,1-z,1-1/z,1/z, check in this order
+ * which ones are allowed and return the index for the one with smallest
+ * resulting |z|. If that modulus is "far enough" from 1 [we can sum the Taylor
+ * series], return it negated. Otherwise, we'll use integral representations. */
 static long
 F21ind(GEN a, GEN b, GEN c, GEN z, long bit)
 {
   GEN v = const_vec(6, mkoo());
   long ind = 0, B = bit - 5;
   const long LD = LOWDEFAULTPREC;
-  if (!isnegint_approx(cind(a,b,c, 1),B)) gel(v,1) = gabs(zind(z,1), LD);
+  if (!isnegint_approx(cind(a,b,c, 1),B))
+  { /* 1 and 6 are allowed */
+    gel(v,1) = gabs(zind(z,1), LD);
+    gel(v,6) = gabs(zind(z,6), LD);
+  }
   gel(v,2) = gabs(zind(z,2), LD);
   gel(v,3) = gabs(z, LD);
-  if (!isnegint_approx(cind(a,b,c, 4),B)) gel(v,4) = gabs(zind(z,4), LD);
-  if (!isnegint_approx(cind(a,b,c, 5),B)) gel(v,5) = gabs(zind(z,5), LD);
-  if (!isnegint_approx(cind(a,b,c, 6),B)) gel(v,6) = gabs(zind(z,6), LD);
+  if (!isnegint_approx(cind(a,b,c, 4),B))
+  { /* 4 and 5 are allowed */
+    gel(v,4) = gabs(zind(z,4), LD);
+    gel(v,5) = gabs(zind(z,5), LD);
+  }
   ind = vecindexmin(v); /* |znew| <= 1; close to 1 ? */
   return (gexpo(gsubgs(gel(v,ind),1)) > -maxss(bit / 4, 32))? -ind: ind;
 }
 static GEN
-mul4(GEN a, GEN b, GEN c, GEN d) { return gmul(a,gmul(b, gmul(c, d))); }
-static GEN
 mul3(GEN a, GEN b, GEN c) { return gmul(a,gmul(b, c)); }
 
-/* (1 - zt)^a t^b (1-t)^c */
-static GEN
-fF212(void *E, GEN t)
-{
-  GEN z = gel(E,1), a = gel(E,2), b = gel(E,3), c = gel(E,4);
-  GEN u = gsubsg(1, gmul(z, t));
-  long prec = precision(t);
-  return mul3(gpow(u, a, prec), gpow(t, b, prec), gpow(gsubsg(1,t), c, prec));
-}
-
-/* (1 - zt)^a T(1-zt) t^b (1-t)^c */
-static GEN
-fF21neg2(void *E, GEN t)
-{
-  GEN z = gel(E,1), a = gel(E,2), b = gel(E,3), c = gel(E,4), T = gel(E,5);
-  GEN u = gsubsg(1, gmul(z, t));
-  long prec = precision(t);
-  return mul4(gsubst(T, 0, u), gpow(u, a, prec), gpow(t, b, prec),
-              gpow(gsubsg(1,t), c, prec));
-}
-
-/* N >= 1 */
+/* return the polynomial of degree N >= 1
+ *   \sum_{i=0} binomial(N,i) (c-a)_i (a)_{N-i} X^i */
 static GEN
 F21lam(long N, GEN a, GEN c)
 {
-  long i;
   GEN C = vecbinomial(N), S = cgetg(N+2, t_VEC);
   GEN vb = vpoch(gsub(c,a), N), va = vpoch(a, N);
+  long i;
   gel(S,1) = gel(va,N);
   for (i = 1; i < N; i++) gel(S,i+1) = mul3(gel(C,i+1), gel(vb,i), gel(va,N-i));
   gel(S,i+1) = gel(vb,N); return RgV_to_RgX(S,0);
 }
 
-/* F(-m,b; c; z), m >= 0 */
+/* F(-m,b; c; z), m >= 0; c != 0, -1, -2, ... */
 static GEN
 F21finitetaylor(long m, GEN b, GEN c, GEN z, long prec)
 {
   pari_sp av;
-  GEN C, S;
-  long j, ct, pradd, mi, bitmin, mb;
-  if (isnegint2(b, &mb) && mb < m) { b = stoi(-m); m = mb; }
+  GEN C, P, S;
+  long j, ct, pradd, mi, bitmin;
+  if (isnegint2(b, &j) && j < m) { b = stoi(-m); m = j; }
   pradd = precFtaylor(mkvec2(stoi(-m), b), mkvec(c), z, &mi);
   if (pradd > 0)
   {
@@ -578,54 +558,53 @@ F21finitetaylor(long m, GEN b, GEN c, GEN z, long prec)
     z = gprec_wensure(z, prec);
   }
   bitmin = -(prec2nbits(prec) + 10);
-  C = real_1(prec); S = C; ct = 0;
+  C = vecbinomial(m);
+  P = real_1(prec); S = P; ct = 0;
   av = avma;
-  for(j = 0; j < m; ++j)
+  for(j = 0; j < m; j++) /* 15.2.4 */
   {
-    C = gmul(C, gdiv(gmulsg(j-m, gaddsg(j, b)), gmulsg(j+1, gaddsg(j, c))));
-    C = gmul(C, z);
+    GEN p;
+    P = gmul(P, gmul(z, gdiv(gaddsg(j,b), gaddsg(j,c)))); /*(b)_j z^j / (c)_j*/
+    p = gmul(P, gel(C, j+1)); /* times binomial(m,j) */
     if (j > mi && !gequal0(S))
-    { if (gexpo(C) - gexpo(S) > bitmin) ct = 0; else if (++ct == 3) break; }
-    S = gadd(S, C);
-    if (gc_needed(av, 1)) gerepileall(av, 2, &S, &C);
+    { if (gexpo(p) - gexpo(S) > bitmin) ct = 0; else if (++ct == 3) break; }
+    S = odd(j)? gsub(S, p): gadd(S, p);
+    if (gc_needed(av, 1)) gerepileall(av, 2, &S, &P);
   }
   return S;
 }
 
-/* c not a nonpositive integer */
 static GEN
-F21finite_i(long m, GEN b, GEN c, GEN z, GEN B, GEN C, GEN coe, long prec)
-{
-  return mul3(poch(B, m, prec), gdiv(gpowgs(coe, m), poch(C, m, prec)),
-              F21finitetaylor(m, b, c, z, prec));
-}
+F21finite_i(GEN F, GEN B, GEN C, long m, long prec)
+{ return gdiv(gmul(F, poch(B, m, prec)), poch(C, m, prec)); }
 
 /* F(-m,b; c; z), m >= 0; c not a nonpositive integer */
 static GEN
 F21finite(long m, GEN b, GEN c, GEN z, long prec)
 {
-  GEN a = stoi(-m), b1 = b, c1 = c, z1;
-  long ind = F21ind(a, b, c, z, prec2nbits(prec)), inda = labs(ind);
-  z1 = zind(z, inda);
-  if (ind < 0)
+  GEN a = stoi(-m), b1, c1, z1, F;
+  long ind = F21ind(a, b, c, z, prec2nbits(prec));
+
+  if (ind > 0 || ind == -3) return F21finitetaylor(m, b, c, z, prec);
+  ind = labs(ind);
+  z1 = zind(z, ind);
+  b1 = bind(a, b, c, ind);
+  c1 = cind(a, b, c, ind); /* not a nonpositive integer */
+  F = F21finitetaylor(m, b1, c1, z1, prec);
+  switch (ind)
   {
-    b1 = bind(a, b, c, inda);
-    c1 = cind(a, b, c, inda); /* not a nonpositive integer */
-  }
-  switch (inda)
-  {
-    case 1: return F21finite_i(m, b1, c1, z1, b, c, gsubsg(1,z), prec);
-    case 2: return gmul(gpowgs(gsubsg(1,z), m),
-                        F21finitetaylor(m, b1, c, z1, prec));
-    case 4: return F21finite_i(m, b1, c1, z1, gsub(c,b), c, gen_1, prec);
-    case 5: return F21finite_i(m, b1, c1, z1, gsub(c,b), c, z, prec);
-    case 6: return F21finite_i(m, b1, c1, z1, b, c, gneg(z), prec);
-    default:return F21finitetaylor(m, b1, c1, z, prec);
+    case 1: return F21finite_i(gmul(F, gpowgs(gsubsg(1,z), m)), b, c, m, prec);
+    case 2: return gmul(F, gpowgs(gsubsg(1,z), m));
+    case 4: return F21finite_i(F, gsub(c,b), c, m, prec);
+    case 5: return F21finite_i(gmul(F, gpowgs(z,m)), gsub(c,b), c, m, prec);
+    case 6: return F21finite_i(gmul(F, gpowgs(gneg(z),m)), b, c, m, prec);
   }
+  return NULL; /*LCOV_EXCL_LINE*/
 }
 
 /**********************************************************/
 
+/* allow a = NULL [omit] */
 static GEN
 multgam(GEN a, GEN b, GEN c, GEN d, long prec)
 {
@@ -640,66 +619,43 @@ intnumsplit(void *E, GEN (*f)(void*, GEN), GEN a, GEN b, GEN z, long prec)
   return gadd(intnum(E, f, a, z, NULL, prec),
               intnum(E, f, z, b, NULL, prec));
 }
+
+/* (1 - zt)^a T(1-zt) t^b (1-t)^c */
+static GEN
+fF21(void *E, GEN t)
+{
+  GEN z = gel(E,1), a = gel(E,2), b = gel(E,3), c = gel(E,4), T = gel(E,5);
+  GEN u = gsubsg(1, gmul(z, t));
+  long prec = precision(t);
+  GEN x = mul3(gpow(u, a, prec), gpow(t, b, prec), gpow(gsubsg(1,t), c, prec));
+  return T? gmul(x, poleval(T, u)): x;
+}
+
 /* z != 1 */
 static GEN
-myint21(void *E, GEN (*f)(void*, GEN), long prec)
+int21(GEN a, GEN b, GEN c, GEN z, GEN T, long prec)
 {
-  GEN z = gel(E,1), a = real_i(gel(E,2)), b = gel(E,3), c = gel(E,4);
-  GEN pz = NULL, p0 = mkendpt(gen_0, b), p1 = mkendpt(gen_1, c);
-  if (gcmpgs(a, 1) <= 0 && is0(imag_i(z), 10))
+  GEN A = gneg(a), B = gsubgs(b,1), C = gsubgs(gsub(c,b), 1), rA = real_i(A);
+  GEN pz = NULL, p0 = mkendpt(gen_0, B), p1 = mkendpt(gen_1, C);
+  GEN E = mkvec5(z, A, B, C, T);
+
+  if (gcmpgs(rA, 1) <= 0 && is0(imag_i(z), 10))
   {
     GEN r;
     pz = ginv(z); r = real_i(pz);
     if (gsigne(r) <= 0 || gcmp(r, gen_1) >= 0) pz = NULL;
   }
-  if (pz) pz = mkendpt(pz,a);
-  else if (gcmpgs(a,-1) <= 0) prec += ((gexpo(a)+1)>>1) * EXTRAPREC64;
-  return intnumsplit(E, f, p0, p1, pz, prec);
+  if (pz) pz = mkendpt(pz,rA);
+  else if (gcmpgs(rA,-1) <= 0) prec += ((gexpo(rA)+1)>>1) * EXTRAPREC64;
+  return intnumsplit((void*)E, fF21, p0, p1, pz, prec);
 }
 
-/* Algorithm used for F21(a,b;c;z)
-Basic transforms:
-  1: (c-b,1+a-b,1/(1-z))
-  2: (c-b,c,z/(z-1))
-  3: (b,c,z)
-  4: (b,b-c+a+1,1-z)
-  5: (1+a-c,b-c+a+1,1-1/z)
-  6: (1+a-c,1+a-b,1/z)
-
-F21: calls F21_i and increase prec if too much cancellation
-F21_i: c is not a non-positive integer
-- z ~ 0 or 1: return special value
-- if a, b, c-b or c-a a non-positive integer: use F21finite
-- compute index, value of z
-   if |z| < 1-epsilon return F21taylorind
-   if Re(b)<=0, swap and/or recurse
-   so may assume Re(b)>0 and Re(a)>=Re(b) and integrate.
-
-F21finite:
-- compute index, value of z
-- call F21finitetaylor
-
-F21ind: find best index (1 to 6, -1 to -6 if |z| < 1-epsilon)
-F21finitetaylor: a or b in Z_{<=0}; calls precFtaylor
-
-F21taylorind: in case 2, may lose accuracy, possible bug.
-- calls F21taylor[1456] or F21taylor
-
-F21taylor: calls Ftaylor / gamma: may lose accuracy
-
-FBaux1: F21taylor twice
-FBaux2: F21finitelim + F21taylorlim
-F21taylor[45]: if c-(a+b) integer, calls FBaux1, else calls FBaux2
-F21taylor[16]: if b-a integer, calls FBaux1, else calls FBaux2
-
-F21taylorlim: calls precFtaylor then compute
-F21finitelim: direct */
 static GEN F21taylorind(GEN a, GEN b, GEN c, GEN z, long ind, long prec);
 /* c not a nonpositive integer */
 static GEN
 F21_i(GEN a, GEN b, GEN c, GEN z, long prec)
 {
-  GEN res;
+  GEN res, p = NULL, T = NULL;
   long m, ind, prec2, bitprec = prec2nbits(prec);
   if (is0(imag_i(z), bitprec)) z = real_i(z);
   if (is0(z, bitprec)) return real_1(prec);
@@ -708,6 +664,7 @@ F21_i(GEN a, GEN b, GEN c, GEN z, long prec)
     GEN x = gsub(c, gadd(a, b)); check_hyp1(x);
     return multgam(c, x, gsub(c,a), gsub(c,b), prec);
   }
+  /* z != 0,1 */
   if (isnegint2(b, &m)) return F21finite(m, a, c, z, prec);
   if (isnegint2(a, &m)) return F21finite(m, b, c, z, prec);
   if (isnegint(gsub(c, b))) swap(a, b);
@@ -716,21 +673,25 @@ F21_i(GEN a, GEN b, GEN c, GEN z, long prec)
     GEN x = gpow(gsubsg(1, z), gneg(gaddsg(m, b)), prec);
     return gmul(x, F21finite(m, gsub(c, b), c, z, prec));
   }
-  /* Here a, b, c, c-a, c-b are not nonpositive integers */
+  /* None of a, b, c, c-a, c-b is a nonpositive integer.
+   * Try Kummer transforms */
   ind = F21ind(a, b, c, z, bitprec);
   prec2 = prec + EXTRAPREC64;
   a = gprec_wensure(a,prec2);
   b = gprec_wensure(b,prec2);
   c = gprec_wensure(c,prec2);
   z = gprec_wensure(z,prec2);
-  if (ind < 0) return gmul(ggamma(c, prec), F21taylorind(a,b,c, z, ind, prec));
+  /* can ensure |z| << 1: sum the Taylor series */
+  if (ind < 0) return gprec_wtrunc(F21taylorind(a,b,c, z, ind, prec), prec);
+  /* can ensure |z| <= 1, but too close to 1: use integral 15.6.1 for
+   * F21 / Gamma(c) */
   if (gsigne(real_i(b)) <= 0)
   {
     if (gsigne(real_i(a)) <= 0)
-    {
+    { /* 15.5.16 (a and b swapped) divided by c */
       GEN p1,p2;
       if (gcmp(real_i(b), real_i(a)) < 0) swap(a,b);
-      /* FIXME: solve recursion as below with F21auxpol */
+      /* FIXME: solve recursion as below with F21lam */
       p1 = gmul(gsubsg(1, z), F21_i(a, gaddsg(1,b), c, z, prec));
       p2 = gmul(gmul(gsubsg(1, gdiv(a,c)), z),
                 F21_i(a, gaddsg(1,b), gaddsg(1,c), z, prec));
@@ -738,23 +699,23 @@ F21_i(GEN a, GEN b, GEN c, GEN z, long prec)
     }
     swap(a,b);
   }
+  /* Here real(b) > 0 */
   if (gcmp(real_i(a), real_i(b)) < 0 && gsigne(real_i(a)) > 0) swap(a,b);
-  /* Here real(b) > 0 and either real(a) <= 0 or real(a) > real(b) */
+  /* further, either real(a) <= 0 or real(a) >= real(b) */
   if (gcmp(real_i(c), real_i(b)) <= 0)
-  {
+  { /* T(x) = sum_i binomial(N,i) (c-a)_i (a)_{N-i} x^i
+     * (c)_N F(a,b,c;z) = sum_{i = 0}^N Ti F(a+N-i,b,c+N;z) for N >= 0,
+     * where F = 2F1 / Gamma(c) */
     long N = 1 + itos(gfloor(gsub(real_i(b),real_i(c)))); /* >= 1 */
-    GEN T = F21lam(N, a, c), c0 = c;
-    void *E;
-    c = gaddsg(N,c);
-    E = (void*)mkvec5(z, gsubsg(-N,a), gsubgs(b,1), gsubgs(gsub(c,b),1), T);
-    res = gdiv(myint21(E, fF21neg2, prec2), poch(c0, N, prec));
+    T = F21lam(N, a, c);
+    p = poch(c, N, prec);
+    c = gaddsg(N,c); /* now real(b) < real(c) */
+    a = gaddsg(N,a);
   }
-  else
-  {
-    void *E = (void*)mkvec4(z, gneg(a), gsubgs(b,1), gsubgs(gsub(c,b),1));
-    res = myint21(E, fF212, prec2);
-  }
-  return gmul(multgam(gen_1, c, b, gsub(c,b), prec), res);
+  /* real(b) < real(c) */
+  res = int21(a, b, c, z, T, prec2); if (p) res = gdiv(res, p);
+  /* res = 2F1(a0,b0,c0; z)*Gamma(b)Gamma(c-b)/Gamma(c) by 15.6.1 */
+  return gdiv(gmul(ggamma(c,prec),res), mulgamma2(b, gsub(c,b), prec));
 }
 
 /* c not a non-positive integer */
@@ -820,20 +781,20 @@ F21taylorlim(GEN N, long m, GEN z, GEN Z, long ind, long prec)
   return gdiv(S, mpfact(m));
 }
 
-/* N = [a,b]; (m-1)! sum_{1 <= k < m} (-z)^k (a)_k (b)_k / k! (m-k)!*/
+/* N = [a,b]; (m-1)! sum_{0 <= k < m} (-z)^k (a)_k (b)_k / k!(m-k)...(m-1) */
 static GEN
 F21finitelim(GEN N, long m, GEN z, long prec)
 {
   GEN C, S, a, b;
-  long j;
+  long k;
   if (!m) return gen_0;
   a = gel(N,1);
   b = gel(N,2);
-  S = C = real_1(prec + EXTRAPRECWORD);
-  for (j = 1; j < m; j++)
+  S = C = real_1(prec + EXTRAPREC64);
+  for (k = 1; k < m; k++)
   {
-    GEN v1 = gaddsg(j-1, a), v2 = gaddsg(j-1, b);
-    C = gdivgs(gmul(C, gmul(gmul(v1, v2), z)), j*(j-m));
+    GEN v1 = gaddsg(k-1, a), v2 = gaddsg(k-1, b);
+    C = gdivgs(gmul(C, gmul(gmul(v1, v2), z)), k*(k-m));
     S = gadd(S, C);
   }
   return gmul(S, mpfact(m-1));
@@ -916,8 +877,9 @@ F21taylor1(GEN a, GEN b, GEN c, GEN z, long prec)
 static GEN
 F21taylor4(GEN a, GEN b, GEN c, GEN z, long prec)
 {
-  GEN bma = gsub(c, gadd(a, b)), coe2, z1, z2, g1, g2, v1, v2;
+  GEN bma = gsub(c, gadd(a, b)), coe2, z1, z2, g1, g2, v1, v2, F = NULL;
   long m;
+  z1 = gsubsg(1, z);
   if (!islong(bma,&m, prec))
   {
     GEN c1, a2, b2, c2;
@@ -925,27 +887,25 @@ F21taylor4(GEN a, GEN b, GEN c, GEN z, long prec)
     a2 = gsub(c, a);
     b2 = gsub(c, b);
     c2 = gaddsg(1, bma);
-    z1 = gsubsg(1, z); coe2 = gneg(gpow(z1, bma, prec));
+    coe2 = gneg(gpow(z1, bma, prec));
     return FBaux1(mkvec3(a,b,c1), mkvec2(a2,b2), gen_1, mkvec3(a2,b2,c2),
                   mkvec2(a,b), coe2, z1, bma, prec, prec);
   }
-  if (m < 0)
-  {
-    GEN F = F21taylor4(gaddgs(a,m), gaddgs(b,m), gaddgs(gadd(a,b), m), z, prec);
-    return gmul(gpowgs(gsubsg(1,z), m), F);
-  }
+  /* c - (a+b) ~ m; if m < 0: 15.8.12 */
+  if (m < 0) { m = -m; a = gsubgs(a, m); b = gsubgs(b, m); F = gpowgs(z1, -m); }
   v2 = g1 = mkvec2(gaddgs(a,m), gaddgs(b,m));
   v1 = g2 = mkvec2(a, b);
-  z1 = gsubgs(z, 1);
-  z2 = gneg(z1); coe2 = gpowgs(z1, m); /* 15.8.10 */
-  return FBaux2(v1, g1, gen_1, m, z1, coe2, g2, v2, z2, z2, 4, prec);
+  z2 = gneg(z1); coe2 = gpowgs(z2, m); /* 15.8.10 */
+  z1 = FBaux2(v1, g1, gen_1, m, z1, coe2, g2, v2, z1, z1, 4, prec);
+  if (F) z1 = gmul(z1, F);
+  return z1;
 }
 
 /* 1 - 1/z */
 static GEN
 F21taylor5(GEN a, GEN b, GEN c, GEN z, long prec)
 {
-  GEN bma = gsub(c, gadd(a,b)), tmp, coe1, coe2, z1, g1, g2, v1, v2, z2;
+  GEN bma = gsub(c, gadd(a,b)), coe1, coe2, z1, g1, g2, v1, v2, z2, F = NULL;
   long m;
   if (!islong(bma,&m, prec))
   {
@@ -962,12 +922,9 @@ F21taylor5(GEN a, GEN b, GEN c, GEN z, long prec)
     return FBaux1(mkvec3(a,b1,c1), mkvec2(d1,e1), coe1,
                   mkvec3(d1,b2,c2), mkvec2(a, b), coe2, z1, bma, prec, prec);
   }
-  /* c - (a + b) ~ m */
+  /* c - (a+b) ~ m; if m < 0: 15.8.12 */
   if (m < 0)
-  {
-    tmp = F21taylor5(gaddgs(a,m), gaddgs(b,m), c, z, prec);
-    return gmul(gpowgs(gsubsg(1, z), m), tmp);
-  }
+  { m = -m; a = gsubgs(a,m); b = gsubgs(b,m); F = gpowgs(gsubsg(1, z), -m); }
   g1 = mkvec2(gaddgs(a,m), gaddgs(b,m));
   v1 = mkvec2(a, gsubsg(1-m, b));
   v2 = mkvec2(gaddgs(a,m), gsubsg(1,b));
@@ -976,7 +933,9 @@ F21taylor5(GEN a, GEN b, GEN c, GEN z, long prec)
   g2 = mkvec2(a, b);
   coe1 = gpow(z, gneg(a), prec);
   coe2 = gmul(coe1, gpowgs(z2, m)); /* 15.8.11 */
-  return FBaux2(v1, g1, coe1, m, z1, coe2, g2, v2, z2, z1, 5, prec);
+  z1 = FBaux2(v1, g1, coe1, m, z1, coe2, g2, v2, z2, z1, 5, prec);
+  if (F) z1 = gmul(z1, F);
+  return z1;
 }
 
 /* 1 / z */
@@ -1016,12 +975,12 @@ F21taylor6(GEN a, GEN b, GEN c, GEN z, long prec)
 }
 
 /* (new b, new c, new z): given by bind, cind, zind
- * case 1: (c-b,1+a-b,1/(1-z))
- * case 2: (c-b,c,z/(z-1))
+ * case 1: (c-b,1+a-b,1/(1-z))   15.8.3  15.8.6
+ * case 2: (c-b,c,z/(z-1))       15.8.1
  * case 3: (b,c,z)
- * case 4: (b,b-c+a+1,1-z)
- * case 5: (1+a-c,b-c+a+1,1-1/z)
- * case 6: (1+a-c,1+a-b,1/z) */
+ * case 4: (b,b-c+a+1,1-z)       15.8.4  15.8.7  15.8.10
+ * case 5: (1+a-c,b-c+a+1,1-1/z) 15.8.5  15.8.7  15.8.11
+ * case 6: (1+a-c,1+a-b,1/z)     15.8.2  15.8.6 */
 static GEN
 F21taylorind(GEN a, GEN b, GEN c, GEN z, long ind, long prec)
 {
@@ -1029,14 +988,17 @@ F21taylorind(GEN a, GEN b, GEN c, GEN z, long ind, long prec)
   switch (labs(ind))
   {
     case 1: res = F21taylor1(a, b, c, z, prec); break;
-    case 2: res = F21taylor(a, gsub(c, b), c, gdiv(z, gsubgs(z, 1)), prec);
-            res = gmul(res, gpow(gsubsg(1,z), gneg(a), prec)); break;
-    case 3: res = F21taylor(a, b, c, z, prec); break;
+    case 2:
+      return gmul(Ftaylor(mkvec2(a, gsub(c, b)), mkvec(c),
+                          gdiv(z, gsubgs(z,1)), prec),
+                  gpow(gsubsg(1,z), gneg(a), prec));
+    case 3:
+      return Ftaylor(mkvec2(a,b), mkvec(c), z, prec);
     case 4: res = F21taylor4(a, b, c, z, prec); break;
     case 5: res = F21taylor5(a, b, c, z, prec); break;
     default:res = F21taylor6(a, b, c, z, prec); break;
   }
-  return gprec_wtrunc(res, prec);
+ return gmul(ggamma(c, prec), res);
 }
 
 static long
@@ -1100,11 +1062,51 @@ hypergeom_arg(GEN x)
   return (typ(x) == t_VEC)? x: mkvec(x);
 }
 
+/* [x[i]+k, i=1..#v] */
+static GEN
+RgV_z_add(GEN x, long k)
+{
+  if (!k) return x;
+  pari_APPLY_same(gaddgs(gel(x,i), k));
+}
+static GEN
+vp(GEN a, GEN p, GEN dft)
+{
+  long v = gvaluation(a, p);
+  return v < 0? stoi(v): dft;
+}
+static GEN
+Qp_hypergeom(GEN N, GEN D, GEN z)
+{
+  pari_sp av = avma;
+  GEN r, S = gen_1, R = gen_1, p = gel(z, 2), dft = ginv(subis(p, 1));
+  long l, i, prec = precp(z) + valp(z) + 1;
+
+  r = gsub(stoi(valp(z)), dft);
+  l = lg(N); for (i = 1; i < l; i++) r = gadd(r, vp(gel(N,i), p, dft));
+  l = lg(D); for (i = 1; i < l; i++) r = gsub(r, vp(gel(D,i), p, dft));
+  if (gsigne(r) <= 0) pari_err(e_MISC, "divergent p-adic hypergeometric sum");
+  l = itou(gceil(gdivsg(prec, r)));
+  for (i = 1; i <= l; i++)
+  {
+    GEN u = vecprod(RgV_z_add(N, i-1)), v = vecprod(RgV_z_add(D, i-1));
+    R = gmul(R, gmul(z, gdiv(u, gmulsg(i, v))));
+    S = gadd(S, R);
+    if (gc_needed(av,1))
+    {
+      if (DEBUGMEM>1) pari_warn(warnmem,"hypergeom, i = %ld / %ld", i,l);
+      gerepileall(av, 2, &R, &S);
+    }
+  }
+  return S;
+}
+
+/* assume is_scalar_t(typ(z)) */
 static GEN
 hypergeom_i(GEN N, GEN D, GEN z, long prec)
 {
   long nN, nD;
-  if (!is_scalar_t(typ(z))) pari_err_TYPE("hypergeom",z);
+  if (typ(z) == t_PADIC) return Qp_hypergeom(N, D, z);
   if (gequal0(z)) return gen_1;
   nN = lg(N) - 1;
   nD = lg(D) - 1;
@@ -1138,56 +1140,35 @@ hypergeom_i(GEN N, GEN D, GEN z, long prec)
   return NULL; /*LCOV_EXCL_LINE*/
 }
 
-/* [v[i]+k, i=1..#v]; no need to optimize for k = 0 */
-static GEN
-RgV_z_add(GEN v, long k)
-{
-  long i, l;
-  GEN w = cgetg_copy(v, &l);
-  for (i = 1; i < l; i++) gel(w,i) = gaddgs(gel(v,i), k);
-  return w;
-}
-static GEN
-vpoch_mul(GEN v, long k)
-{
-  long i, l = lg(v);
-  GEN p;
-  if (l == 1) return gen_1;
-  p = gmael(v, 1, k);
-  for (i = 2; i < l; i++) p = gmul(p, gmael(v, i, k));
-  return p;
-}
-
 static GEN
 serhypergeom(GEN N, GEN D, GEN y, long prec)
 {
-  pari_sp av;
-  GEN S, pn, vN, vD, y0 = NULL;
+  GEN Di, Ni, S = gen_1, R = gen_1, y0 = NULL;
   long v, l, i;
+  pari_sp av;
   if (!signe(y)) return gadd(gen_1, y);
-  v = valp(y);
+  v = valser(y); l = lg(y);
   if (v < 0) pari_err_DOMAIN("hypergeom","valuation", "<", gen_0, y);
-  l = lg(y);
-  if (v) S = gen_1;
-  else
+  if (!v)
   {
     y0 = gel(y, 2);
-    y = serchop0(y);
-    l = 3 + (l - 3) / valp(y);
+    if (!is_scalar_t(typ(y0))) pari_err_TYPE("hypergeom",y);
+    y = serchop0(y); l = 3 + (l - 3) / valser(y);
     S = hypergeom(N, D, y0, prec);
   }
-  vN = RgV_vpoch(N, l-1);
-  vD = RgV_vpoch(D, l-1); av = avma;
-  pn = y;
+  av = avma; Ni = N; Di = D;
   for (i = 1; i < l; i++)
   {
-    GEN H = gdiv(vpoch_mul(vN, i), vpoch_mul(vD, i));
-    if (y0) H = gmul(H, hypergeom_i(RgV_z_add(N,i), RgV_z_add(D,i), y0,prec));
-    S = gadd(S, gmul(pn, H)); if (i + 1 < l) pn = gdivgu(gmul(pn, y), i + 1);
+    R = gmul(R, gmul(y, gdiv(vecprod(Ni), gmulsg(i, vecprod(Di)))));
+    /* have to offset by 1 when y0 != NULL; keep Ni,Di to avoid recomputing
+     * in next loop */
+    Ni = RgV_z_add(N, i);
+    Di = RgV_z_add(D, i);
+    S = gadd(S, y0? gmul(R, hypergeom_i(Ni, Di, y0, prec)): R);
     if (gc_needed(av,1))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"hypergeom, i = %ld / %ld", i,l-1);
-      gerepileall(av, 2, &S, &pn);
+      gerepileall(av, 4, &S, &R, &Ni, &Di);
     }
   }
   return S;
