@@ -30,7 +30,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 static GEN
 mysercoeff(GEN x, long n)
 {
-  long N = n - valp(x);
+  long N = n - valser(x);
   return (N < 0)? gen_0: gel(x, N+2);
 }
 
@@ -149,11 +149,9 @@ sumVgaimpos(GEN v)
 static long
 vgaell(GEN Vga)
 {
-  long d = lg(Vga)-1;
-  GEN c;
-  if (d != 2) return 0;
-  c = gsub(gel(Vga,1), gel(Vga,2));
-  return gequal1(c) || gequalm1(c);
+  if (lg(Vga) == 3)
+  { GEN c = gsub(gel(Vga,1), gel(Vga,2)); return gequal1(c) || gequalm1(c); }
+  return 0;
 }
 int
 Vgaeasytheta(GEN Vga) { return lg(Vga)-1 == 1 || vgaell(Vga); }
@@ -240,7 +238,7 @@ static int
 serisscalar(GEN x)
 {
   long i;
-  if (valp(x)) return 0;
+  if (valser(x)) return 0;
   for (i = lg(x)-1; i > 3; i--) if (!gequal0(gel(x,i))) return 0;
   return 1;
 }
@@ -274,7 +272,7 @@ static GEN
 serpole(GEN r)
 {
   GEN s = cgetg(3, t_SER);
-  s[1] = evalsigne(1)|evalvalp(-1)|evalvarn(0);
+  s[1] = evalsigne(1)|evalvalser(-1)|evalvarn(0);
   gel(s,2) = r; return s;
 }
 /* a0 +  a1 x + O(x^e), e >= 0 */
@@ -374,7 +372,7 @@ polgammaeval(GEN F, GEN s)
       if (!gequal0(r)) break;
     }
     if (e > 1) r = gdiv(r, mpfact(e));
-    r = serpole(r); setvalp(r, e);
+    r = serpole(r); setvalser(r, e);
   }
   return r;
 }
@@ -593,7 +591,7 @@ normalizepoles(GEN r, GEN k)
     GEN rj = gel(r,j), a = gel(rj,1), ra = gel(rj,2);
     if (!is_scalar_t(typ(a)) || typ(ra) != t_SER)
       pari_err_TYPE("lfunrootres [poles]",r);
-    if (valp(ra) >= 0) continue;
+    if (valser(ra) >= 0) continue;
     gel(v,iv++) = rj;
   }
   setlg(v, iv); return v;
@@ -630,7 +628,7 @@ lfunrtoR_i(GEN ldata, GEN r, GEN eno, long prec)
   r = normalizepoles(r, k);
   if (typ(r) == t_COL) return gerepilecopy(av, r);
   if (typ(ldata_get_dual(ldata)) != t_INT)
-    pari_err(e_MISC,"please give the Taylor development of Lambda");
+    pari_err(e_MISC,"please give the Taylor expansion of Lambda");
   vr = lfunrtopoles(r); lr = lg(vr);
   FVga = gammafactor(Vga);
   R = cgetg(2*lr, t_COL);
@@ -639,9 +637,9 @@ lfunrtoR_i(GEN ldata, GEN r, GEN eno, long prec)
     GEN rj = gel(r,j), a = gel(rj,1), ra = gel(rj,2);
     GEN Ra = rtoR(a, ra, FVga, N, prec);
     GEN b = gsub(k, conj_i(a));
-    if (lg(Ra)-2 < -valp(Ra))
+    if (lg(Ra)-2 < -valser(Ra))
       pari_err(e_MISC,
-        "please give more terms in L function's Taylor development at %Ps", a);
+        "please give more terms in L function's Taylor expansion at %Ps", a);
     gel(R,jR++) = mkvec2(a, Ra);
     if (!tablesearch(vr, b, (int (*)(GEN,GEN))&cmp_universal))
     {
@@ -664,7 +662,7 @@ prec_fix(long prec)
 {
 #ifndef LONG_IS_64BIT
   /* make sure that default accuracy is the same on 32/64bit */
-  if (odd(prec)) prec += EXTRAPRECWORD;
+  if (odd(prec)) prec += EXTRAPREC64;
 #endif
   return prec;
 }
@@ -685,7 +683,7 @@ lfunthetainit0(GEN ldata, GEN tdom, GEN an2, long m,
     get_cone_fuzz(tdom, &r, &a);
     tdom = mkvec2(dbltor(r), a? dbltor(a): gen_0);
   }
-  prec += maxss(EXTRAPRECWORD, nbits2extraprec(extrabit));
+  prec += maxss(EXTRAPREC64, nbits2extraprec(extrabit));
   tech = mkvecn(7, an2,K,R, stoi(bitprec), stoi(m), tdom,
                    gsqrt(ginv(N), prec_fix(prec)));
   return mkvec3(mkvecsmall(t_LDESC_THETA), ldata, tech);
@@ -1229,6 +1227,12 @@ parse_dom(double k, GEN dom, struct lfunp *S)
 {
   long l = lg(dom);
   if (typ(dom)!=t_VEC) pari_err_TYPE("lfuninit [domain]", dom);
+  if (l == 1)
+  {
+    S->dc = 0;
+    S->dw = -1;
+    S->dh = -1; return;
+  }
   if (l == 2)
   {
     S->dc = k/2.;
@@ -1260,20 +1264,22 @@ int
 sdomain_isincl(double k, GEN dom, GEN dom0)
 {
   struct lfunp S0, S;
-  parse_dom(k, dom, &S);
-  parse_dom(k, dom0, &S0);
+  parse_dom(k, dom, &S); if (S.dw < 0) return 1;
+  parse_dom(k, dom0, &S0); if (S0.dw < 0) return 0;
   return S0.dc - S0.dw <= S.dc - S.dw
       && S0.dc + S0.dw >= S.dc + S.dw && S0.dh >= S.dh;
 }
 
 static int
-checklfuninit(GEN linit, GEN dom, long der, long bitprec)
+checklfuninit(GEN linit, GEN DOM, long der, long bitprec)
 {
   GEN ldata = linit_get_ldata(linit);
   GEN domain = lfun_get_domain(linit_get_tech(linit));
+  GEN dom = domain_get_dom(domain);
+  if (lg(dom) == 1) return 1;
   return domain_get_der(domain) >= der
     && domain_get_bitprec(domain) >= bitprec
-    && sdomain_isincl(gtodouble(ldata_get_k(ldata)), dom, domain_get_dom(domain));
+    && sdomain_isincl(gtodouble(ldata_get_k(ldata)), DOM, dom);
 }
 
 static GEN
@@ -1300,6 +1306,21 @@ lfuninit_make(long t, GEN ldata, GEN tech, GEN domain)
   tech = mkvec3(domain, tech, mkvec4(k2, w2, expot, gammafactor(Vga)));
   return mkvec3(mkvecsmall(t), ldata, tech);
 }
+static GEN
+lfunnoinit(GEN ldata, long bitprec)
+{
+  GEN tech, domain = mkvec2(cgetg(1,t_VEC), mkvecsmall2(0, bitprec));
+  GEN R = gen_0, r = ldata_get_residue(ldata), v = lfunrootres(ldata, bitprec);
+  ldata = shallowcopy(ldata);
+  gel(ldata,6) = gel(v,3);
+  if (r)
+  {
+    if (isintzero(r)) setlg(ldata,7); else gel(ldata,7) = r;
+    R = gel(v,2);
+  }
+  tech = mkvec3(domain, gen_0, R);
+  return lfuninit_make(t_LDESC_INIT, ldata, tech, domain);
+}
 
 static void
 lfunparams2(struct lfunp *S)
@@ -1392,7 +1413,7 @@ lfuncost(GEN L, GEN dom, long der, long bit)
   GEN w, k = ldata_get_k(ldata);
   struct lfunp S;
 
-  parse_dom(gtodouble(k), dom, &S);
+  parse_dom(gtodouble(k), dom, &S); if (S.dw < 0) return mkvecsmall2(0, 0);
   lfunp_set(ldata, der, bit, &S);
   w = ldata_get_rootno(ldata);
   if (isintzero(w)) /* for lfunrootres */
@@ -1427,17 +1448,45 @@ lfuncost0(GEN L, GEN dom, long der, long bitprec)
   return gerepileupto(av, zv_to_ZV(C));
 }
 
+static int
+is_dirichlet(GEN ldata)
+{
+  switch(ldata_get_type(ldata))
+  {
+    case t_LFUN_ZETA:
+    case t_LFUN_KRONECKER:
+    case t_LFUN_CHIZ: return 1;
+    case t_LFUN_CHIGEN: return ldata_get_degree(ldata)==1;
+    default: return 0;
+  }
+}
+
+static ulong
+lfuninit_cutoff(GEN ldata)
+{
+  GEN gN = ldata_get_conductor(ldata);
+  ulong L, N;
+  if (ldata_get_type(ldata) == t_LFUN_NF) /* N ~ f^(d-1), exact for d prime */
+    gN = sqrtnint(gN, ldata_get_degree(ldata) - 1);
+  N = itou_or_0(gN);
+  if (N > 1000) L = 7000;
+  else if (N > 100) L = 5000;
+  else if (N > 15) L = 3000;
+  else L = 2500;
+  return L;
+}
+
 GEN
 lfuninit(GEN lmisc, GEN dom, long der, long bitprec)
 {
-  pari_sp ltop = avma;
+  pari_sp av = avma;
   GEN poqk, AB, R, h, theta, ldata, eno, r, domain, tech, k;
   struct lfunp S;
 
   if (is_linit(lmisc))
   {
     long t = linit_get_type(lmisc);
-    if (t==t_LDESC_INIT || t==t_LDESC_PRODUCT)
+    if (t == t_LDESC_INIT || t == t_LDESC_PRODUCT)
     {
       if (checklfuninit(lmisc, dom, der, bitprec)) return lmisc;
       pari_warn(warner,"lfuninit: insufficient initialization");
@@ -1445,13 +1494,34 @@ lfuninit(GEN lmisc, GEN dom, long der, long bitprec)
   }
   ldata = lfunmisc_to_ldata_shallow(lmisc);
 
-  if (ldata_get_type(ldata)==t_LFUN_NF)
+  switch (ldata_get_type(ldata))
   {
-    GEN T = gel(ldata_get_an(ldata), 2);
-    return lfunzetakinit(T, dom, der, bitprec);
+  case t_LFUN_NF:
+    {
+      GEN T = gel(ldata_get_an(ldata), 2);
+      return gerepilecopy(av, lfunzetakinit(T, dom, der, bitprec));
+    }
+  case t_LFUN_ABELREL:
+    {
+      GEN T = gel(ldata_get_an(ldata), 2);
+      return gerepilecopy(av, lfunabelianrelinit(gel(T,1), gel(T,2), dom, der, bitprec));
+    }
   }
   k = ldata_get_k(ldata);
   parse_dom(gtodouble(k), dom, &S);
+  /* Reduce domain for Dirichlet characters. NOT for Abelian t_LFUN_NF,
+   * handled above. */
+  if (S.dw >= 0 && (!der && is_dirichlet(ldata)))
+    S.dh = mindd(S.dh, lfuninit_cutoff(ldata));
+  if (S.dw < 0)
+  {
+    if (der)
+      pari_err_IMPL("domain = [] for derivatives in lfuninit");
+    if (!is_dirichlet(ldata))
+      pari_err_IMPL("domain = [] for L functions of degree > 1");
+    return gerepilecopy(av, lfunnoinit(ldata, bitprec));
+  }
+
   lfunp_set(ldata, der, bitprec, &S);
   ldata = ldata_newprec(ldata, nbits2prec(S.Dmax));
   r = ldata_get_residue(ldata);
@@ -1495,7 +1565,7 @@ lfuninit(GEN lmisc, GEN dom, long der, long bitprec)
     AB = lfuninit_pol(AB, poqk, S.precmax);
   tech = mkvec3(h, AB, R);
   domain = mkvec2(dom, mkvecsmall2(der, bitprec));
-  return gerepilecopy(ltop, lfuninit_make(t_LDESC_INIT, ldata, tech, domain));
+  return gerepilecopy(av, lfuninit_make(t_LDESC_INIT, ldata, tech, domain));
 }
 
 GEN
@@ -1551,7 +1621,7 @@ lfunsumcoth(GEN R, GEN s, GEN h, long prec)
   for (j = 1; j < lg(R); ++j)
   {
     GEN r = gel(R,j), be = gel(r,1), Rj = gel(r, 2);
-    long e = valp(Rj);
+    long e = valser(Rj);
     GEN z1 = gexpm1(gmul(h, gsub(s,be)), prec); /* exp(h(s-beta))-1 */
     GEN c1 = gaddgs(gdivsg(2, z1), 1); /* coth((h/2)(s-beta)) */
     GEN C1 = veccothderivn(c1, 1-e);
@@ -1566,31 +1636,35 @@ lfunsumcoth(GEN R, GEN s, GEN h, long prec)
 }
 
 static GEN lfunlambda_OK(GEN linit, GEN s, GEN sdom, long bitprec);
-/* L is a t_LDESC_PRODUCT Linit */
+/* L is a t_LDESC_PRODUCT or t_LDESC_INIT Linit */
 static GEN
-lfunlambda_product(GEN L, GEN s, GEN sdom, long bitprec)
+_product(GEN (*fun)(GEN,GEN,long), GEN L, GEN s, long bitprec)
 {
-  GEN ldata = linit_get_ldata(L), v = lfunprod_get_fact(linit_get_tech(L));
-  GEN r = gen_1, F = gel(v,1), E = gel(v,2), C = gel(v,3), cs = conj_i(s);
-  long i, l = lg(F), isreal = gequal(imag_i(s), imag_i(cs));
-  for (i = 1; i < l; ++i)
+  GEN ldata = linit_get_ldata(L), v, r, F, E, C, cs;
+  long i, l;
+  int isreal;
+  if (linit_get_type(L) == t_LDESC_INIT) return fun(ldata, s, bitprec);
+  v = lfunprod_get_fact(linit_get_tech(L));
+  F = gel(v,1); E = gel(v,2); C = gel(v,3); l = lg(F);
+  cs = conj_i(s); isreal = gequal(imag_i(s), imag_i(cs));
+  for (i = 1, r = gen_1; i < l; ++i)
   {
-    GEN f = lfunlambda_OK(gel(F, i), s, sdom, bitprec);
-    if (E[i])
-      r = gmul(r, gpowgs(f, E[i]));
+    GEN f = fun(gel(F, i), s, bitprec);
+    if (typ(f)==t_VEC) f = RgV_prod(f);
+    if (E[i]) r = gmul(r, gpowgs(f, E[i]));
     if (C[i])
     {
-      GEN fc = isreal? f: lfunlambda_OK(gel(F, i), cs, sdom, bitprec);
-      r = gmul(r, gpowgs(conj_i(fc), C[i]));
+      GEN fc = isreal? f: conj_i(fun(gel(F, i), cs, bitprec));
+      r = gmul(r, gpowgs(fc, C[i]));
     }
   }
   return (ldata_isreal(ldata) && gequal0(imag_i(s)))? real_i(r): r;
 }
 
-/* s a t_SER */
+/* s a t_SER; # terms - 1 */
 static long
 der_level(GEN s)
-{ return signe(s)? lg(s)-3: valp(s)-1; }
+{ return signe(s)? lg(s)-3: valser(s)-1; }
 
 /* s a t_SER; return coeff(s, X^0) */
 static GEN
@@ -1616,20 +1690,21 @@ get_domain(GEN s, GEN *dom, long *der)
 static GEN
 lfunlambda_OK(GEN linit, GEN s, GEN sdom, long bitprec)
 {
-  GEN eno, ldata, tech, h, pol;
-  GEN S, S0 = NULL, k2, cost;
+  GEN dom, eno, ldata, tech, h, pol, k2, cost, S, S0 = NULL;
   long prec, prec0;
   struct lfunp D, D0;
 
   if (linit_get_type(linit) == t_LDESC_PRODUCT)
-    return lfunlambda_product(linit, s, sdom, bitprec);
+    return _product(&lfunlambda, linit, s, bitprec);
   ldata = linit_get_ldata(linit);
   eno = ldata_get_rootno(ldata);
   tech = linit_get_tech(linit);
+  dom = lfun_get_dom(tech);
+  if (lg(dom) == 1) return lfunlambda(linit, s, bitprec); /* FIXME:not OK! */
   h = lfun_get_step(tech); prec = realprec(h);
   /* try to reduce accuracy */
   parse_dom(0, sdom, &D0);
-  parse_dom(0, domain_get_dom(lfun_get_domain(tech)), &D);
+  parse_dom(0, dom, &D);
   if (0.8 * D.dh > D0.dh)
   {
     cost = lfuncost(linit, sdom, typ(s)==t_SER? der_level(s): 0, bitprec);
@@ -1664,13 +1739,85 @@ lfunlambda_OK(GEN linit, GEN s, GEN sdom, long bitprec)
   if (S0) S = gadd(S,S0);
   return gprec_w(gmul(S,h), nbits2prec(bitprec));
 }
+
+static long
+lfunspec_OK(GEN lmisc, GEN s, GEN *pldata)
+{
+  long t, large = 0;
+  GEN ldata;
+  *pldata = ldata = lfunmisc_to_ldata_shallow(lmisc);
+  if (!is_linit(lmisc)) lmisc = ldata;
+  else switch(linit_get_type(lmisc))
+  {
+    case t_LDESC_INIT: case t_LDESC_PRODUCT:
+      if (lg(lfun_get_dom(linit_get_tech(lmisc))) == 1) large = 1;
+      break;
+    default: return 0;
+  }
+  switch(typ(s))
+  {
+    case t_INT: case t_REAL: case t_FRAC: case t_COMPLEX: break;
+    default: return 0;
+  }
+  t = ldata_get_type(ldata);
+  switch(t)
+  {
+    case t_LFUN_KRONECKER: case t_LFUN_ZETA:
+      if (typ(s) == t_INT && !is_bigint(s)) return 1;
+      /* fall through */
+    case t_LFUN_NF: case t_LFUN_CHIZ:
+      if (!large)
+        large = (fabs(gtodouble(imag_i(s))) >= lfuninit_cutoff(ldata));
+      break;
+    case t_LFUN_CHIGEN:
+      if (ldata_get_degree(ldata) != 1) return 0;
+      if (!large)
+        large = (fabs(gtodouble(imag_i(s))) >= lfuninit_cutoff(ldata));
+      break;
+  }
+  if (large)
+  {
+    if (t == t_LFUN_NF)
+    {
+      GEN an = ldata_get_an(ldata), nf = gel(an,2), G = galoisinit(nf, NULL);
+      if (isintzero(G) || !group_isabelian(galois_group(G))) return 0;
+    }
+    return 2;
+  }
+  return 0;
+}
+
 GEN
 lfunlambda(GEN lmisc, GEN s, long bitprec)
 {
   pari_sp av = avma;
-  GEN linit, dom, z;
+  GEN linit = NULL, dom, z;
   long der;
   s = get_domain(s, &dom, &der);
+  if (!der)
+  {
+    GEN ldata;
+    long t = lfunspec_OK(lmisc, s, &ldata);
+    if (t == 1)
+    { /* special value ? */
+      GEN z = lfun(ldata, s, bitprec), gv = ldata_get_gammavec(ldata);
+      long e = itou(gel(gv, 1));
+      if (!isintzero(z) && (e || gsigne(s) > 0)) /* TODO */
+      {
+        GEN q = ldata_get_conductor(ldata);
+        long prec = nbits2prec(bitprec);
+        GEN se, r, Q = divir(q, mppi(prec));
+        se = gmul2n(gaddgs(s, e), -1);
+        r = gmul(gpow(Q, se, prec), ggamma(se, prec));
+        if (e && !equali1(q)) r = gdiv(r, gsqrt(q, prec));
+        return gerepileupto(av, gmul(r, z));
+      }
+    }
+    if (is_linit(lmisc)) linit = lmisc; else lmisc = ldata;
+    if (t == 2)
+      return gerepilecopy(av, linit? _product(&lfunlambda, linit, s, bitprec)
+                                   : lfunlambdalarge(ldata, s, bitprec));
+  }
   linit = lfuninit(lmisc, dom, der, bitprec);
   z = lfunlambda_OK(linit,s, dom, bitprec);
   return gerepilecopy(av, z);
@@ -1689,7 +1836,7 @@ is_ser(GEN x)
 static GEN
 lfunser(GEN L)
 {
-  long v = valp(L);
+  long v = valser(L);
   if (v > 0) return gen_0;
   if (v == 0) L = gel(L, 2);
   else
@@ -1706,7 +1853,7 @@ lfunservec(GEN x)
 static GEN
 lfununext(GEN L)
 {
-  setlg(L, maxss(lg(L)-1, valp(L)? 2: 3));
+  setlg(L, maxss(lg(L)-1, valser(L)? 2: 3));
   return normalizeser(L);
 }
 static GEN
@@ -1747,21 +1894,24 @@ GEN
 lfun(GEN lmisc, GEN s, long bitprec)
 {
   pari_sp av = avma;
-  GEN linit, dom, z;
+  GEN linit = NULL, ldata, dom, z;
   long der;
   s = get_domain(s, &dom, &der);
-  if (!der && typ(s) == t_INT && !is_bigint(s))
-  { /* special value ? */
-    GEN ldata;
-    long t, ss = itos(s);
-    if (is_linit(lmisc))
-      ldata = linit_get_ldata(lmisc);
-    else
-      lmisc = ldata = lfunmisc_to_ldata_shallow(lmisc);
-    t = ldata_get_type(ldata);
-    if (t == t_LFUN_KRONECKER || t == t_LFUN_ZETA)
+  if (der && typ(s) != t_SER)
+  {
+    if (lfunspec_OK(lmisc, s, &ldata))
     {
-      long D = itos_or_0(gel(ldata_get_an(ldata), 2));
+      linit = lfuninit(lmisc, cgetg(1,t_VEC), 0, bitprec);
+      return derivnumk((void*)linit, (GEN(*)(void*,GEN,long))&lfun,
+                       s, stoi(der), nbits2prec(bitprec));
+    }
+  }
+  else
+  {
+    long t = lfunspec_OK(lmisc, s, &ldata);
+    if (t == 1)
+    { /* special value ? */
+      long D = itos_or_0(gel(ldata_get_an(ldata), 2)), ss = itos(s);
       if (D)
       {
         if (ss <= 0) return lfunquadneg(D, ss);
@@ -1770,7 +1920,7 @@ lfun(GEN lmisc, GEN s, long bitprec)
         {
           long prec = nbits2prec(bitprec), q = labs(D);
           ss = 1 - ss; /* <= 0 */
-          z = powrs(divrs(mppi(prec + EXTRAPRECWORD), q), 1-ss);
+          z = powrs(divrs(mppi(prec + EXTRAPREC64), q), 1-ss);
           z = mulrr(shiftr(z, -ss), sqrtr_abs(utor(q, prec)));
           z = gdiv(z, mpfactr(-ss, prec));
           if (smodss(ss, 4) > 1) togglesign(z);
@@ -1778,6 +1928,10 @@ lfun(GEN lmisc, GEN s, long bitprec)
         }
       }
     }
+    if (is_linit(lmisc)) linit = lmisc; else lmisc = ldata;
+    if (t == 2)
+      return gerepilecopy(av, linit? _product(&lfun, linit, s, bitprec)
+                                   : lfunlarge(ldata, s, bitprec));
   }
   linit = lfuninit(lmisc, dom, der, bitprec);
   z = lfun_OK(linit, s, dom, bitprec);
@@ -1791,9 +1945,9 @@ sersplit1(GEN s, GEN *head)
   long i, l = lg(s);
   GEN y;
   *head = simplify_shallow(mysercoeff(s, 0));
-  if (valp(s) > 0) return s;
+  if (valser(s) > 0) return s;
   y = cgetg(l-1, t_SER); y[1] = s[1];
-  setvalp(y, 1);
+  setvalser(y, 1);
   for (i=3; i < l; i++) gel(y,i-1) = gel(s,i);
   return normalizeser(y);
 }
@@ -1823,7 +1977,7 @@ lfunlambdaord(GEN linit, GEN s)
 static GEN
 derser(GEN res, long m)
 {
-  long v = valp(res);
+  long v = valser(res);
   if (v > m) return gen_0;
   if (v >= 0)
     return gmul(mysercoeff(res, m), mpfact(m));
@@ -1839,26 +1993,32 @@ static GEN
 lfunderiv(GEN lmisc, long m, GEN s, long flag, long bitprec)
 {
   pari_sp ltop = avma;
-  GEN res, S = NULL, linit, dom;
+  GEN res, S = NULL, linit, ldata, dom;
   long der, prec = nbits2prec(bitprec);
   if (m <= 0) pari_err_DOMAIN("lfun", "D", "<=", gen_0, stoi(m));
   s = get_domain(s, &dom, &der);
+  if (typ(s) != t_SER && lfunspec_OK(lmisc, s, &ldata) == 2)
+  {
+    linit = lfuninit(lmisc, cgetg(1,t_VEC), 0, bitprec);
+    return derivnumk((void*)linit, (GEN(*)(void*,GEN,long))&lfun,
+                     s, stoi(der + m), prec);
+  }
   linit = lfuninit(lmisc, dom, der + m, bitprec);
+  if (lg(lfun_get_dom(linit_get_tech(linit))) == 1)
+    pari_err_IMPL("domain = [] for derivatives in lfuninit");
   if (typ(s) == t_SER)
   {
-    long v, l = lg(s)-1;
-    GEN sh;
-    if (valp(s) < 0) pari_err_DOMAIN("lfun","valuation", "<", gen_0, s);
-    S = sersplit1(s, &sh);
-    v = valp(S);
-    s = deg1ser_shallow(gen_1, sh, varn(S), m + (l+v-1)/v);
+    GEN a;
+    if (valser(s) < 0) pari_err_DOMAIN("lfun","valuation", "<", gen_0, s);
+    S = sersplit1(s, &a);
+    s = deg1ser_shallow(gen_1, a, varn(S), m + ceildivuu(lg(s)-2, valser(S)));
   }
   else
   {
-    long ex = lfunlambdaord(linit, s);
+    long e = lfunlambdaord(linit, s) + m + 1;
     /* HACK: pretend lfuninit was done to right accuracy */
-    if (gequal0(s)) s = gen_0;
-    s = deg1ser_shallow(gen_1, s, 0, m+1+ex);
+    if (gequal0(s)) { s = gen_0; e--; }
+    s = deg1ser_shallow(gen_1, s, 0, e);
   }
   res = flag ? lfunlambda_OK(linit, s, dom, bitprec):
                lfun_OK(linit, s, dom, bitprec);
@@ -1866,7 +2026,7 @@ lfunderiv(GEN lmisc, long m, GEN s, long flag, long bitprec)
     res = gsubst(derivn(res, m, -1), varn(S), S);
   else if (typ(res)==t_SER)
   {
-    long v = valp(res);
+    long v = valser(res);
     if (v > m) { set_avma(ltop); return gen_0; }
     if (v >= 0)
       res = gmul(mysercoeff(res, m), mpfact(m));
@@ -1896,38 +2056,62 @@ GEN
 lfunhardy(GEN lmisc, GEN t, long bitprec)
 {
   pari_sp ltop = avma;
-  long prec = nbits2prec(bitprec), d;
-  GEN argz, z, linit, ldata, tech, dom, w2, k2, E, h, a, k;
+  long prec = nbits2prec(bitprec), d, isbig = 0;
+  GEN linit, h, ldata, tech, w2, k2, E, a, argz, z;
 
   switch(typ(t))
   {
     case t_INT: case t_FRAC: case t_REAL: break;
     default: pari_err_TYPE("lfunhardy",t);
   }
-
-  ldata = lfunmisc_to_ldata_shallow(lmisc);
-  if (!is_linit(lmisc)) lmisc = ldata;
-  k = ldata_get_k(ldata);
-  d = ldata_get_degree(ldata);
-  dom = mkvec3(gmul2n(k, -1), gen_0, gabs(t,LOWDEFAULTPREC));
-  linit = lfuninit(lmisc, dom, 0, bitprec);
-  tech = linit_get_tech(linit);
+  if (lfunspec_OK(lmisc, mkcomplex(gen_0, t), &ldata) == 2)
+  {
+    long B = bitprec + maxss(gexpo(t), 0);
+    GEN L = NULL;
+    isbig = 1;
+    k2 = ghalf;
+    z = mkcomplex(k2, t);
+    if (is_linit(lmisc))
+    {
+      linit = lmisc;
+      if (linit_get_type(linit) == t_LDESC_PRODUCT)
+        L = mkvec(linit);/*HACK*/
+    }
+    else
+    {
+      linit = lfunnoinit(ldata, B);
+      ldata = linit_get_ldata(linit); /* make sure eno is included */
+    }
+    h = lfunloglambdalarge(L? L: ldata, gprec_w(z, nbits2prec(B)), B);
+    tech = linit_get_tech(linit);
+  }
+  else
+  {
+    GEN k = ldata_get_k(ldata);
+    GEN dom = mkvec3(gmul2n(k, -1), gen_0, gabs(t,LOWDEFAULTPREC));
+    if (!is_linit(lmisc)) lmisc = ldata;
+    linit = lfuninit(lmisc, dom, 0, bitprec);
+    tech = linit_get_tech(linit);
+    k2 = lfun_get_k2(tech);
+    z = mkcomplex(k2, t);
+    h = lfunlambda_OK(linit, z, dom, bitprec);
+  }
   w2 = lfun_get_w2(tech);
-  k2 = lfun_get_k2(tech);
   E = lfun_get_expot(tech); /* 4E = d(k2 - 1) + real(vecsum(Vga)) */
-  z = mkcomplex(k2, t);
+  d = ldata_get_degree(ldata);
   /* more accurate than garg: k/2 in Q */
   argz = gequal0(k2)? Pi2n(-1, prec): gatan(gdiv(t, k2), prec);
   prec = precision(argz);
   /* prec may have increased: don't lose accuracy if |z|^2 is exact */
   a = gsub(gmulsg(d, gmul(t, gmul2n(argz,-1))),
            gmul(E, glog(gnorm(z),prec)));
-  h = lfunlambda_OK(linit, z, dom, bitprec);
   if (!isint1(w2) && typ(ldata_get_dual(ldata))==t_INT)
-    h = mulrealvec(h, w2);
+    h = isbig ? gadd(h, glog(w2, prec)) : mulrealvec(h, w2);
   if (typ(h) == t_COMPLEX && gexpo(imag_i(h)) < -(bitprec >> 1))
     h = real_i(h);
-  return gerepileupto(ltop, gmul(h, gexp(a, prec)));
+  if (isbig) h = greal(gexp(gadd(h, a), prec));
+  else h = gmul(h, gexp(a, prec));
+  return gerepileupto(ltop, h);
 }
 
 /* L = log(t); return  \sum_{i = 0}^{v-1}  R[-i-1] L^i/i! */
@@ -1951,7 +2135,7 @@ theta_add_polar_part(GEN S, GEN R, GEN t, long prec)
   for (j = 1; j < l; j++)
   {
     GEN Rj = gel(R,j), b = gel(Rj,1), Rb = gel(Rj,2);
-    long v = -valp(Rb);
+    long v = -valser(Rb);
     if (v > 1 && !logt) logt = glog(t, prec);
     S = gsub(S, gmul(theta_pole_contrib(Rb,v,logt), gpow(t,b,prec)));
   }
@@ -1977,11 +2161,11 @@ lfuncheckfeq_i(GEN theta, GEN thetad, GEN t0, GEN t0i, long bitprec)
     if (gequal0(R))
     {
       GEN v, r;
-      if (ldata_get_type(ldata) == t_LFUN_NF)
+      long t = ldata_get_type(ldata);
+      if (t == t_LFUN_NF || t == t_LFUN_ABELREL)
       { /* inefficient since theta not needed; no need to optimize for this
            (artificial) query [e.g. lfuncheckfeq(t_POL)] */
-        GEN T = gel(ldata_get_an(ldata), 2);
-        GEN L = lfunzetakinit(T,zerovec(3),0,bitprec);
+        GEN L = lfuninit(ldata,zerovec(3),0,bitprec);
         return lfuncheckfeq(L,t0,bitprec);
       }
       v = lfunrootres(theta, bitprec);
@@ -2043,7 +2227,7 @@ ropm1(GEN w, long prec)
   GEN r;
   if (typ(w) == t_INT) return w;
   r = grndtoi(w, &e);
-  return (e < -prec2nbits(prec)/2)? r: w;
+  return (e < -prec/2)? r: w;
 }
 
 /* theta for t=1/sqrt(2) and t2==2t simultaneously, saving 25% of the work.
@@ -2130,7 +2314,7 @@ get_eno(GEN R, GEN k, GEN t, GEN v, GEN vi, long vx, long bitprec, long force)
 
 }
 /* Return w using theta(1/t) - w t^k \bar{theta}(t) = polar_part(t,w).
- * The full Taylor development of L must be known */
+ * The full Taylor expansion of L must be known */
 GEN
 lfunrootno(GEN linit, long bitprec)
 {
@@ -2254,7 +2438,8 @@ static GEN
 lfunhardyzeros(void *E, GEN t)
 {
   struct lhardyz_t *S = (struct lhardyz_t*)E;
-  return gprec_wensure(lfunhardy(S->linit, t, S->bitprec), S->prec);
+  GEN z = gprec_wensure(lfunhardy(S->linit, t, S->bitprec), S->prec);
+  return typ(z) == t_VEC ? RgV_prod(z): z;
 }
 
 /* initialize for computation on critical line up to height h, zero
@@ -2262,15 +2447,17 @@ lfunhardyzeros(void *E, GEN t)
 static GEN
 lfuncenterinit(GEN lmisc, double h, long m, long bitprec)
 {
+  GEN ldata = lfunmisc_to_ldata_shallow(lmisc);
   if (m < 0)
   { /* choose a sensible default */
-    if (!is_linit(lmisc) || linit_get_type(lmisc) != t_LDESC_INIT) m = 4;
-    else
+    m = 4;
+    if (is_linit(lmisc) && linit_get_type(lmisc) == t_LDESC_INIT)
     {
       GEN domain = lfun_get_domain(linit_get_tech(lmisc));
       m = domain_get_der(domain);
     }
   }
+  if (is_dirichlet(ldata)) m = 0;
   return lfuninit(lmisc, mkvec(dbltor(h)), m, bitprec);
 }
 
@@ -2291,13 +2478,33 @@ lfunorderzero(GEN lmisc, long m, long bitprec)
   linit = lfuncenterinit(lmisc, 0, m, bitprec);
   ldata = linit_get_ldata(linit);
   eno = ldata_get_rootno(ldata);
-  if (typ(eno) == t_VEC) pari_err_TYPE("lfunorderzero [vector-valued]", lmisc);
   k2 = gmul2n(ldata_get_k(ldata), -1);
   G = -bitprec/2;
   c0 = 0; st = 1;
-  if (ldata_isreal(ldata)) { st = 2; if (!gequal1(eno)) c0 = 1; }
-  for (c = c0;; c += st)
-    if (gexpo(lfun0(linit, k2, c, bitprec)) > G) return gc_long(ltop, c);
+  if (typ(eno) == t_VEC)
+  {
+    long i, l = lg(eno), cnt = l-1, s = 0;
+    GEN v = zero_zv(l-1);
+    if (ldata_isreal(ldata)) st = 2;
+    for (c = c0; cnt; c += st)
+    {
+      GEN L = lfun0(linit, k2, c, bitprec);
+      for (i = 1; i < l; i++)
+      {
+        if (v[i]==0 && gexpo(gel(L,i)) > G)
+        {
+          v[i] = c; cnt--; s += c;
+        }
+      }
+    }
+    return gc_long(ltop,s);
+  }
+  else
+  {
+    if (ldata_isreal(ldata)) { st = 2; if (!gequal1(eno)) c0 = 1; }
+    for (c = c0;; c += st)
+      if (gexpo(lfun0(linit, k2, c, bitprec)) > G) return gc_long(ltop, c);
+  }
 }
 
 /* assume T1 * T2 > 0, T1 <= T2 */
@@ -2313,7 +2520,7 @@ lfunzeros_i(struct lhardyz_t *S, GEN *pw, long *ct, GEN T1, GEN T2, long d,
     GEN D, T0, z;
     D = gcmp(T, pi2) < 0? cN
                         : gadd(cN, gmulsg(d, glog(gdiv(T, pi2), prec)));
-    D = gdiv(pi2div, D);
+    D = gdiv(pi2div, gmulsg(d, D));
     for(;;)
     {
       long s0;
@@ -2350,15 +2557,22 @@ lfunzeros(GEN ldata, GEN lim, long divz, long bitprec)
   }
   if (typ(lim) == t_VEC)
   {
-    if (lg(lim) != 3 || gcmp(gel(lim,1),gel(lim,2)) >= 0)
+    double H1, H2;
+    if (lg(lim) != 3 || gcmp(gel(lim, 1), gel(lim, 2)) >= 0)
       pari_err_TYPE("lfunzeros",lim);
-    h1 = gel(lim,1);
-    h2 = gel(lim,2);
-    maxt = maxdd(fabs(gtodouble(h1)), fabs(gtodouble(h2)));
+    h1 = gel(lim, 1); H1 = gtodouble(h1);
+    h2 = gel(lim, 2); H2 = gtodouble(h2);
+    maxt = maxdd(fabs(H1), fabs(H2));
+    if (H1 * H2 > 0)
+    {
+      GEN LDATA = lfunmisc_to_ldata_shallow(ldata);
+      double m = mindd(fabs(H1), fabs(H2));
+      if (is_dirichlet(LDATA) && m > lfuninit_cutoff(LDATA)) maxt = 0;
+    }
   }
   else
   {
-    if (gcmp(lim,gen_0) <= 0) pari_err_TYPE("lfunzeros",lim);
+    if (gcmp(lim, gen_0) <= 0) pari_err_TYPE("lfunzeros",lim);
     h1 = gen_0;
     h2 = lim;
     maxt = gtodouble(h2);
@@ -2385,7 +2599,7 @@ lfunzeros(GEN ldata, GEN lim, long divz, long bitprec)
     if (!r || gequal0(r))
     {
       ct = lfunorderzero(linit, -1, bitprec);
-      if (ct) T = real2n(-prec2nbits(prec) / (2*ct), prec);
+      if (ct) T = real2n(-prec / (2*ct), prec);
     }
   }
   if (s1 <= 0)
@@ -2555,7 +2769,7 @@ lfunconductor(GEN data, GEN maxcond, long flag, long bitprec)
   if (eval) bitprec += bitprec/2;
   prec = nbits2prec(bitprec);
   ld = shallowcopy(ldata);
-  gel(ld, 5) = eval? M: gel(M,lg(M)-1);
+  gel(ld, 5) = eval? M: veclast(M);
   theta = lfunthetainit_i(ld, tdom, 0, bitprec);
   thetad = theta_dual(theta, ldata_get_dual(ldata));
   gel(theta,3) = shallowcopy(linit_get_tech(theta));
