@@ -46,17 +46,6 @@ vecRCpol(long r, long d)
   }
   return v;
 }
-/* D a t_INT */
-static GEN
-RgXV_rescale(GEN v, GEN D)
-{
-  long j, l;
-  GEN w;
-  if (equali1(D)) return v;
-  w = cgetg_copy(v, &l);
-  for (j = 1; j < l; j++) gel(w,j) = RgX_rescale(gel(v,j), D);
-  return w;
-}
 static GEN
 euler_sumdiv(GEN q, long v)
 {
@@ -603,7 +592,7 @@ Lfeq(long D, long k)
     if (odd(kd) && !(k % d) && uisprime(Da)) den = kd * Da;
   }
   else if (Da == 4) den = 2;
-  z = Linv(D, k, den); prec = lg(z);
+  z = Linv(D, k, den); prec = realprec(z);
   z = mulrr(z, powrs(divru(Pi2n(1, prec), Da), k));
   if (Da != 4) { z = mulrr(z, sqrtr_abs(utor(Da,prec))); shiftr_inplace(z,-1); }
   res = divrr(mpfactr(k-1, prec), z);
