@@ -304,7 +304,7 @@ rtopoles(GEN r)
 }
 /* re = polar part; overestimate when re = gen_0 (unknown) */
 static long
-orderpole(GEN re) { return typ(re) == t_SER? -valp(re): 1; }
+orderpole(GEN re) { return typ(re) == t_SER? -valser(re): 1; }
 static GEN
 lfunmulpoles(GEN ldata1, GEN ldata2, long bitprec)
 {
@@ -401,7 +401,7 @@ lfundivpoles(GEN ldata1, GEN ldata2, long bitprec)
       if (orderpole(s1) == orderpole(s2)) continue;
     }
     z = gdiv(lfun(ldata1,be,bitprec), lfun(ldata2,be,bitprec));
-    if (valp(z) < 0) gel(r,i++) = mkvec2(be, z);
+    if (valser(z) < 0) gel(r,i++) = mkvec2(be, z);
   }
   if (i == 1) return NULL;
   setlg(r, i); return r;
@@ -600,7 +600,7 @@ simple_pole(GEN r)
   GEN S;
   if (isintzero(r)) return gen_0;
   S = deg1ser_shallow(gen_0, r, 1);
-  setvalp(S, -1); return S;
+  setvalser(S, -1); return S;
 }
 
 GEN
@@ -729,9 +729,6 @@ lfunzeta(void)
   gel(zet,3) = mkvec(gen_0);
   return zet;
 }
-static GEN
-lfunzetainit(GEN dom, long der, long bitprec)
-{ return lfuninit(lfunzeta(), dom, der, bitprec); }
 
 static GEN
 vecan_Kronecker(GEN D, long n)
@@ -1010,13 +1007,15 @@ lfunzetak_i(GEN T)
   long r1, r2;
   if (typ(T) == t_POL)
   {
-    T = nfinit(T, DEFAULTPREC);
+    T = nfinit0(T, nf_NOLLL, DEFAULTPREC);
     if (lg(T) == 3) T = gel(T,1); /* [nf,change of var] */
   }
+  if (nf_get_degree(T) == 1) return lfunzeta();
   nf_get_sign(T,&r1,&r2); Vga = vec01(r1+r2,r2);
   N = absi_shallow(nf_get_disc(T));
   return mkvecn(7, tag(T,t_LFUN_NF), gen_0, Vga, gen_1, N, gen_1, gen_0);
 }
+/* truen nf or t_POL */
 static GEN
 lfunzetak(GEN T)
 { pari_sp av = avma; return gerepilecopy(av, lfunzetak_i(T)); }
@@ -1161,56 +1160,134 @@ lfunchigen(GEN bnr, GEN CHI)
 }
 
 /* Find all characters of clgp whose kernel contain group given by HNF H.
- * Set *pcnj[i] if chi[i] is not real */
+ * Set *pcnj[i] to the conductor */
 static GEN
 chigenkerfind(GEN bnr, GEN H, GEN *pcnj)
 {
-  GEN res, cnj, L = bnrchar(bnr, H, NULL), cyc = bnr_get_cyc(bnr);
+  GEN res, cnj, L = bnrchar(bnr, H, NULL);
   long i, k, l = lg(L);
 
   res = cgetg(l, t_VEC);
-  *pcnj = cnj = cgetg(l, t_VECSMALL);
+  *pcnj = cnj = cgetg(l, t_VEC);
   for (i = k = 1; i < l; i++)
   {
-    GEN chi = gel(L,i), c = charconj(cyc, chi);
-    long fl = ZV_cmp(c, chi);
-    if (fl < 0) continue; /* keep one char in pair of conjugates */
+    GEN chi = gel(L,i);
     gel(res, k) = chi;
-    cnj[k] = fl; k++;
+    gel(cnj, k) = ZV_equal0(chi)? gen_0: bnrconductor_raw(bnr, chi);
+    k++;
   }
   setlg(cnj, k);
   setlg(res, k); return res;
 }
 
-/* bnf = NULL: base field = Q */
-GEN
-lfunabelianrelinit(GEN nfabs, GEN bnf, GEN polrel, GEN dom, long der, long bitprec)
+static GEN
+vec_classes(GEN A, GEN F)
 {
-  pari_sp ltop = avma;
-  GEN cond, chi, cnj, res, bnr, M, domain;
-  long l, i;
-  long v = -1;
+  GEN w = vec_equiv(F);
+  long i, l = lg(w);
+  GEN V = cgetg(l, t_VEC);
+  for (i = 1; i < l; i++) gel(V,i) = vecpermute(A,gel(w,i));
+  return V;
+}
 
-  if (bnf) bnf = checkbnf(bnf);
-  else
+static GEN
+abelrel_pfactor(GEN bnr, GEN pr, GEN U, GEN D, GEN h)
+{
+  GEN v = bnrisprincipalmod(bnr, pr, h, 0);
+  GEN E = ZV_ZV_mod(ZM_ZC_mul(U, v), D);
+  ulong o = itou(charorder(D, E)), f = pr_get_f(pr);
+  return gpowgs(gsub(gen_1, monomial(gen_1, f * o, 0)), itou(h) / o);
+}
+
+static GEN
+abelrel_factor(GEN bnr, GEN C, GEN p, GEN mod, GEN U, GEN D, GEN h)
+{
+  GEN nf = bnr_get_nf(bnr), F = pol_1(0), prid = idealprimedec(nf,p);
+  GEN mod2 = shallowcopy(mod);
+  long i, l = lg(prid);
+  for (i = 1; i < l; i++)
   {
-    v = fetch_var();
-    bnf = Buchall(pol_x(v), 0, nbits2prec(bitprec));
+    GEN pr = gel(prid, i), Fpr;
+    long v = idealval(nf,mod,pr);
+    if (v > 0)
+    {
+      GEN bnr2, C2, U2, D2, h2;
+      gel(mod2, 1) = idealdivpowprime(nf, gel(mod, 1), pr, utoi(v));
+      bnr2 = bnrinitmod(bnr, mod2, 0, h);
+      C2 = bnrmap(bnrmap(bnr, bnr2), C);
+      D2 = ZM_snfall_i(C2, &U2, NULL, 1);
+      h2 = ZV_prod(D2);
+      Fpr = abelrel_pfactor(bnr2, pr, U2, D2, h2);
+    }
+    else
+      Fpr = abelrel_pfactor(bnr, pr, U, D, h);
+    F = ZX_mul(F, Fpr);
   }
-  if (typ(polrel) != t_POL) pari_err_TYPE("lfunabelianrelinit", polrel);
-  cond = rnfconductor0(bnf, polrel, 1);
-  bnr = gel(cond,2);
-  chi = chigenkerfind(bnr, gel(cond,3), &cnj);
-  l = lg(chi); res = cgetg(l, t_VEC);
+  return gcopy(mkrfrac(gen_1, F));
+}
+
+static GEN
+eulerf_abelrel(GEN an, GEN p)
+{
+  GEN bnr = gel(an,1), C = gel(an,2), mod = gel(an,3);
+  GEN U, D = ZM_snfall_i(C, &U, NULL, 1), h = ZV_prod(D);
+  return abelrel_factor(bnr, C, p, mod, U, D, h);
+}
+
+struct direuler_abelrel
+{
+  GEN bnr, C, mod, U, D, h;
+};
+
+static GEN
+_direuler_abelrel(void *E, GEN p)
+{
+  struct direuler_abelrel *s = (struct direuler_abelrel*) E;
+  return abelrel_factor(s->bnr, s->C, p, s->mod, s->U, s->D, s->h);
+}
+
+static GEN
+vecan_abelrel(GEN an, long N)
+{
+  struct direuler_abelrel s;
+  s.bnr = gel(an,1);
+  s.C   = gel(an,2);
+  s.mod = gel(an,3);
+  s.D = ZM_snfall_i(s.C, &s.U, NULL, 1);
+  s.h = ZV_prod(s.D);
+  return direuler((void*)&s, _direuler_abelrel, gen_1, stoi(N), NULL);
+}
+
+static GEN
+lfunabelrel_i(GEN bnr, GEN H, GEN mod)
+{
+  GEN NrD = bnrdisc(bnr, H, 0), N = absi_shallow(gel(NrD,3));
+  long n = itos(gel(NrD,1)), r1 = itos(gel(NrD,2)), r2 = (n-r1)>>1;
+  if (!mod) mod = bnrconductor(bnr, H, 0);
+  return mkvecn(7, tag(mkvec3(bnr,H,mod),t_LFUN_ABELREL),
+                   gen_0, vec01(r1+r2, r2), gen_1, N, gen_1, gen_0);
+}
+static GEN
+lfunabelrel(GEN bnr, GEN H, GEN mod)
+{ pari_sp av = avma; return gerepilecopy(av, lfunabelrel_i(bnr, H, mod)); }
+
+GEN
+lfunabelianrelinit(GEN bnr, GEN H, GEN dom, long der, long bitprec)
+{
+  GEN X, cnj, M, D,C ;
+  long l, i;
+  C = chigenkerfind(bnr, H, &cnj);
+  C = vec_classes (C, cnj);
+  X = cgetg_copy(C,&l);
   for (i = 1; i < l; ++i)
   {
-    GEN L = lfunchigen(bnr, gel(chi,i));
-    gel(res, i) = lfuninit(L, dom, der, bitprec);
+    GEN chi = gel(C,i);
+    GEN L = lfunchigen(bnr, lg(chi)==2 ? gel(chi,1): chi);
+    gel(X,i) = lfuninit(L, dom, der, bitprec);
   }
-  if (v >= 0) delete_var();
-  M = mkvec3(res, const_vecsmall(l-1, 1), cnj);
-  domain = mkvec2(dom, mkvecsmall2(der, bitprec));
-  return gerepilecopy(ltop, lfuninit_make(t_LDESC_PRODUCT, lfunzetak_i(nfabs), M, domain));
+  M = mkvec3(X, const_vecsmall(l-1, 1), const_vecsmall(l-1, 0));
+  D = mkvec2(dom, mkvecsmall2(der, bitprec));
+  return lfuninit_make(t_LDESC_PRODUCT, lfunabelrel_i(bnr, H, NULL), M, D);
 }
 
 /*****************************************************************/
@@ -1227,8 +1304,8 @@ dirzetak0(GEN nf, ulong N)
   long court[] = {evaltyp(t_INT)|_evallg(3), evalsigne(1)|evallgefint(3),0};
   forprime_t S;
 
-  c  = cgetalloc(t_VECSMALL, N+1);
-  c2 = cgetalloc(t_VECSMALL, N+1);
+  c  = cgetalloc(N+1, t_VECSMALL);
+  c2 = cgetalloc(N+1, t_VECSMALL);
   c2[1] = c[1] = 1; for (i=2; i<=N; i++) c[i] = 0;
   u_forprime_init(&S, 2, N); av2 = avma;
   while ( (p = u_forprime_next(&S)) )
@@ -1273,16 +1350,15 @@ dirzetak0(GEN nf, ulong N)
 static GEN
 eulerf_zetak(GEN nf, GEN p)
 {
-  GEN vect, T = nf_get_pol(nf), index = nf_get_index(nf), f = pol_1(0);
+  GEN v, f = pol_1(0);
   long i, l;
-  if (dvdii(index, p)) /* p does not divide index */
-    vect = idealprimedec_degrees(nf,p);
+  if (dvdii(nf_get_index(nf), p)) /* p does not divide index */
+    v = idealprimedec_degrees(nf,p);
   else
-    vect = gel(FpX_degfact(T,p),1);
-  l = lg(vect);
-  for (i = 1; i < l; i++)
-    f = gmul(f, gsub(gen_1, monomial(gen_1, vect[i], 0)));
-  return gcopy(mkrfrac(gen_1, f));
+    v = gel(FpX_degfact(nf_get_pol(nf), p), 1);
+  l = lg(v);
+  for (i = 1; i < l; i++) f = ZX_sub(f, RgX_shift_shallow(f, v[i]));
+  retmkrfrac(gen_1, ZX_copy(f));
 }
 
 GEN
@@ -1318,11 +1394,11 @@ lfunproduct(GEN ldata, GEN linit1, GEN linit2, GEN domain)
                   vecsmall_concat(gel(M1, 3), gel(M2, 3)));
   return lfuninit_make(t_LDESC_PRODUCT, ldata, M3, domain);
 }
-
+static GEN lfunzetakinit_artin(GEN nf, GEN gal, GEN dom, long der, long bit);
+/* true nf */
 static GEN
 lfunzetakinit_quotient(GEN nf, GEN polk, GEN dom, long der, long bitprec)
 {
-  pari_sp av = avma;
   GEN ak, an, nfk, Vga, ldata, N, Lk, LKk, domain;
   long r1k, r2k, r1, r2;
 
@@ -1337,34 +1413,32 @@ lfunzetakinit_quotient(GEN nf, GEN polk, GEN dom, long der, long bitprec)
   ldata = mkvecn(6, an, gen_0, Vga, gen_1, N, gen_1);
   LKk = lfuninit(ldata, dom, der, bitprec); /* zeta_K/zeta_k */
   domain = mkvec2(dom, mkvecsmall2(der, bitprec));
-  return gerepilecopy(av, lfunproduct(lfunzetak_i(nf), Lk, LKk, domain));
+  return lfunproduct(lfunzetak_i(nf), Lk, LKk, domain);
 }
-
-static GEN
-lfunzetakinit_artin(GEN nf, GEN gal, GEN dom, long der, long bitprec);
-
-static GEN
-lfunzetakinit_Galois(GEN nf, GEN gal, GEN dom, long der, long bitprec)
-{
-  GEN grp = galois_group(gal);
-  if (group_isabelian(grp))
-    return lfunabelianrelinit(nf, NULL, gal_get_pol(gal), dom, der, bitprec);
-  else return lfunzetakinit_artin(nf, gal, dom, der, bitprec);
-}
-
 /* true nf */
 GEN
 lfunzetakinit(GEN nf, GEN dom, long der, long bitprec)
 {
-  GEN G, nfs, sbg;
-  long lf, d = nf_get_degree(nf);
-  if (d == 1) return lfunzetainit(dom, der, bitprec);
-  G = galoisinit(nf, NULL);
-  if (!isintzero(G))
-    return lfunzetakinit_Galois(nf, G, dom, der, bitprec);
-  nfs = nfsubfields(nf, 0); lf = lg(nfs)-1;
-  sbg = gmael(nfs,lf-1,1);
-  return lfunzetakinit_quotient(nf, sbg, dom, der, bitprec);
+  long n, d = nf_get_degree(nf);
+  GEN L, Q, R, T = nf_get_pol(nf);
+  if (d == 1) return lfuninit(lfunzeta(), dom, der, bitprec);
+  if (d > 2)
+  {
+    GEN G = galoisinit(nf, NULL);
+    if (isintzero(G))
+    { /* not Galois */
+      GEN S = nfsubfields(nf, 0); n = lg(S)-1;
+      return lfunzetakinit_quotient(nf, gmael(S,n-1,1), dom, der, bitprec);
+    }
+    if (!group_isabelian(galois_group(G))) /* Galois, not Abelian */
+      return lfunzetakinit_artin(nf, G, dom, der, bitprec);
+  }
+  /* Abelian over Q */
+  Q = Buchall(pol_x(1), 0, nbits2prec(bitprec));
+  T = shallowcopy(T); setvarn(T,0);
+  R = rnfconductor0(Q, T, 1);
+  L = lfunabelianrelinit(gel(R,2), gel(R,3), dom, der, bitprec);
+  delete_var(); return L;
 }
 
 /***************************************************************/
@@ -2141,7 +2215,63 @@ ellfromeqncharpoly(GEN P, GEN Q, GEN p)
 }
 
 static GEN
-genus2_eulerfact(GEN P, GEN p)
+nfellcharpoly(GEN e, GEN T, GEN p)
+{
+  GEN nf, E, t;
+  nf = nfinit(mkvec2(T, mkvec(p)), DEFAULTPREC);
+  E = ellinit(e, nf, DEFAULTPREC);
+  if (lg(E)==1) return NULL;
+  t = elleulerf(E, p);
+  obj_free(E);
+  return t;
+}
+
+static GEN
+genus2_red5(GEN P, GEN T, GEN p)
+{
+  long vx = varn(P), vy = varn(T);
+  GEN f = shallowcopy(T);
+  setvarn(f, vx);
+  while(1)
+  {
+    GEN Pr, R, r, Rs;
+    (void) ZXX_pvalrem(P, p, &Pr);
+    R = FpXQX_roots_mult(Pr, 3, T, p);
+    if (lg(R)==1) return P;
+    r = FpX_center(gel(R,1), p, shifti(p,-1));
+    r = gel(R,1);
+    P = RgX_affine(P, p, r);
+    setvarn(r, vx);
+    f = RgX_Rg_div(gsub(f, r), p);
+    Rs = RgX_rem(RgXY_swap(P, 3, vy), gsub(f, pol_x(vy)));
+    P = RgXY_swap(Rs, 3, vy);
+    (void) ZXX_pvalrem(P, sqri(p), &P);
+  }
+}
+
+static GEN
+genus2_type5(GEN P, GEN p)
+{
+  GEN E, F, T, a, a2, Q;
+  long v;
+  (void) ZX_pvalrem(P, p, &F);
+  F = FpX_red(F, p);
+  if (degpol(F) < 1) return NULL;
+  F = FpX_factor(F, p);
+  if (mael(F,2,1) != 3 || degpol(gmael(F,1,1)) != 2) return NULL;
+  T = gmael(F, 1, 1);
+  v = fetch_var_higher();
+  Q = RgV_to_RgX(ZX_digits(P, T), v);
+  Q = genus2_red5(Q, T, p);
+  a = gel(Q,5); a2 = ZX_sqr(a);
+  E = mkvec5(gen_0, gel(Q,4), gen_0, ZX_mul(gel(Q,3),a), ZX_mul(gel(Q,2),a2));
+  delete_var();
+  return nfellcharpoly(E, T, p);
+}
+
+/* Assume P has semistable reduction at p */
+static GEN
+genus2_eulerfact_semistable(GEN P, GEN p)
 {
   GEN Pp = FpX_red(P, p);
   GEN GU = genus2_redmodel(Pp, p);
@@ -2184,6 +2314,19 @@ genus2_eulerfact(GEN P, GEN p)
   return ginv( ZX_mul(abe, tor) );
 }
 
+static GEN
+genus2_eulerfact(GEN P, GEN p)
+{
+  pari_sp av = avma;
+  GEN W, R = genus2_type5(P, p), E;
+  if (R) return R;
+  W = hyperellextremalmodels(P, 2, p);
+  if (lg(W) < 3) return genus2_eulerfact_semistable(P,p);
+  E =  gmul(genus2_eulerfact_semistable(gel(W,1),p),
+            genus2_eulerfact_semistable(gel(W,2),p));
+  return gerepileupto(av, E);
+}
+
 static GEN
 F2x_genus2_find_trans(GEN P, GEN Q, GEN F)
 {
@@ -2267,7 +2410,7 @@ F2xqX_quad_nbroots(GEN b, GEN c, GEN T)
 }
 
 static GEN
-genus2_eulerfact2(GEN PQ)
+genus2_eulerfact2_semistable(GEN PQ)
 {
   GEN V = F2x_genus_red(ZX_to_F2x(gel(PQ, 1)), ZX_to_F2x(gel(PQ, 2)));
   GEN P = gel(V, 1), Q = gel(V, 2);
@@ -2309,6 +2452,19 @@ genus2_eulerfact2(GEN PQ)
   return ginv( ZX_mul(abe, tor) );
 }
 
+static GEN
+genus2_eulerfact2(GEN F, GEN PQ)
+{
+  pari_sp av = avma;
+  GEN W, R = genus2_type5(F, gen_2), E;
+  if (R) return R;
+  W = hyperellextremalmodels(PQ, 2, gen_2);
+  if (lg(W) < 3) return genus2_eulerfact2_semistable(PQ);
+  E = gmul(genus2_eulerfact2_semistable(gel(W,1)),
+           genus2_eulerfact2_semistable(gel(W,2)));
+  return gerepileupto(av, E);
+}
+
 GEN
 lfungenus2(GEN G)
 {
@@ -2322,7 +2478,7 @@ lfungenus2(GEN G)
   if (ram2 && equalis(gmael(M,2,1),-1))
     pari_warn(warner,"unknown valuation of conductor at 2");
   e = cgetg(lL+(ram2?0:1), t_VEC);
-  gel(e,1) = mkvec2(gen_2, ram2 ? genus2_eulerfact2(PQ)
+  gel(e,1) = mkvec2(gen_2, ram2 ? genus2_eulerfact2(F, PQ)
            : ginv( RgX_recip(hyperellcharpoly(gmul(PQ,gmodulss(1,2))))) );
   for(i = ram2? 2: 1; i < lL; i++)
   {
@@ -2897,14 +3053,12 @@ lfunartin(GEN nf, GEN gal, GEN ch, long o, long bitprec)
   return gerepilecopy(av, Ldata);
 }
 
+/* true nf */
 static GEN
 lfunzetakinit_artin(GEN nf, GEN gal, GEN dom, long der, long bitprec)
 {
-  pari_sp ltop = avma;
-  GEN To = galoischartable(gal), T = gel(To, 1);
-  long o = itos(gel(To, 2));
-  GEN F, E, M, domain;
-  long i, l = lg(T);
+  GEN F, E, M, domain, To = galoischartable(gal), T = gel(To, 1);
+  long i, o = itos(gel(To, 2)), l = lg(T);
   F = cgetg(l, t_VEC);
   E = cgetg(l, t_VECSMALL);
   for (i = 1; i < l; ++i)
@@ -2915,8 +3069,7 @@ lfunzetakinit_artin(GEN nf, GEN gal, GEN dom, long der, long bitprec)
   }
   domain = mkvec2(dom, mkvecsmall2(der, bitprec));
   M = mkvec3(F, E, zero_zv(l-1));
-  return gerepilecopy(ltop, lfuninit_make(t_LDESC_PRODUCT, lfunzetak_i(nf),
-                                          M, domain));
+  return lfuninit_make(t_LDESC_PRODUCT, lfunzetak_i(nf), M, domain);
 }
 
 /********************************************************************/
@@ -2924,7 +3077,7 @@ lfunzetakinit_artin(GEN nf, GEN gal, GEN dom, long der, long bitprec)
 /********************************************************************/
 enum { t_LFUNMISC_POL, t_LFUNMISC_CHIQUAD, t_LFUNMISC_CHICONREY,
        t_LFUNMISC_CHIGEN, t_LFUNMISC_ELLINIT, t_LFUNMISC_ETAQUO,
-       t_LFUNMISC_GCHAR };
+       t_LFUNMISC_GCHAR, t_LFUNMISC_ABELREL };
 static long
 lfundatatype(GEN data)
 {
@@ -2934,15 +3087,17 @@ lfundatatype(GEN data)
     case t_INTMOD: return t_LFUNMISC_CHICONREY;
     case t_POL: return t_LFUNMISC_POL;
     case t_VEC:
-    {
-      long l = lg(data);
-      if (checknf_i(data)) return t_LFUNMISC_POL;
-      if (l == 17) return t_LFUNMISC_ELLINIT;
-      if (l == 3 && typ(gel(data,1)) == t_VEC)
-        return is_gchar_group(gel(data,1))? t_LFUNMISC_GCHAR
-                                          : t_LFUNMISC_CHIGEN;
+      switch(lg(data))
+      {
+        case 17: return t_LFUNMISC_ELLINIT;
+        case 10: return t_LFUNMISC_POL;
+        case 3:
+          if (typ(gel(data,1)) != t_VEC) break;
+          return is_gchar_group(gel(data,1))  ? t_LFUNMISC_GCHAR
+                    : typ(gel(data,2))==t_MAT ? t_LFUNMISC_ABELREL
+                                              : t_LFUNMISC_CHIGEN;
+      }
       break;
-    }
   }
   return -1;
 }
@@ -2977,6 +3132,9 @@ lfunmisc_to_ldata_i(GEN ldata, long shallow)
     }
     break;
     case t_LFUNMISC_GCHAR: return lfungchar(gel(ldata,1), gel(ldata,2));
+    case t_LFUNMISC_ABELREL:
+      return lfunabelrel(gel(ldata,1), gel(ldata,2),
+                         lg(ldata)==3? NULL: gel(ldata,3));
     case t_LFUNMISC_ELLINIT: return lfunell(ldata);
   }
   if (shallow != 2) pari_err_TYPE("lfunmisc_to_ldata",ldata);
@@ -3008,6 +3166,7 @@ ldata_vecan(GEN van, long L, long prec)
   if (DEBUGLEVEL >= 1)
     err_printf("Lfun: computing %ld coeffs, prec %ld, type %ld\n", L, prec, t);
   if (DEBUGLEVEL >= 2) timer_start(&ti);
+  if (L < 0) L = 0;
   switch (t)
   {
     long n;
@@ -3028,6 +3187,7 @@ ldata_vecan(GEN van, long L, long prec)
       an = (ell_get_type(an) == t_ELL_Q) ? ellanQ_zv(an, L): ellan(an, L);
       break;
     case t_LFUN_KRONECKER: an = vecan_Kronecker(an, L); break;
+    case t_LFUN_ABELREL: an = vecan_abelrel(an, L); break;
     case t_LFUN_CHIZ: an = vecan_chiZ(an, L, prec); break;
     case t_LFUN_CHIGEN: an = vecan_chigen(an, L, prec); break;
     case t_LFUN_HECKE: an = vecan_gchar(an, L, prec); break;
@@ -3045,7 +3205,7 @@ ldata_vecan(GEN van, long L, long prec)
     {
       GEN F = gel(an,1), E = gel(an,2), c = gel(an,3);
       an = mfcoefs(F,L,1) + 1; /* skip a_0 */
-      an[0] = evaltyp(t_VEC)|evallg(L+1);
+      an[0] = evaltyp(t_VEC)|_evallg(L+1);
       an = mfvecembed(E, an);
       if (!isint1(c)) an = RgV_Rg_mul(an,c);
       break;
@@ -3099,6 +3259,7 @@ ldata_eulerf(GEN van, GEN p, long prec)
     case t_LFUN_ELL: f = elleulerf(an, p); break;
     case t_LFUN_KRONECKER:
       f = mkrfrac(gen_1, deg1pol_shallow(stoi(-kronecker(an, p)), gen_1, 0)); break;
+    case t_LFUN_ABELREL: f = eulerf_abelrel(an, p); break;
     case t_LFUN_CHIZ: f = eulerf_chiZ(an, p, prec); break;
     case t_LFUN_CHIGEN: f = eulerf_chigen(an, p, prec); break;
     case t_LFUN_HECKE: f = eulerf_gchar(an, p, prec); break;
