@@ -149,7 +149,7 @@ FpM_ratlift(GEN M, GEN mod, GEN amax, GEN bmax, GEN den)
 static GEN
 lead_simplify(GEN P)
 {
-  GEN x = gel(P, lg(P)-1); /* x a nonzero ZX or t_INT */
+  GEN x = leading_coeff(P); /* x a nonzero ZX or t_INT */
   if (typ(x) == t_POL)
   {
     if (degpol(x)) return x;
@@ -355,7 +355,7 @@ fix_nf(GEN *pnf, GEN *pT, GEN *pA)
   long i, l, lim;
 
   if (*pnf) return gen_1;
-  lim = GP_DATA->primelimit + 1;
+  lim = GP_DATA->factorlimit + 1;
   nfmaxord(&S, mkvec2(T, utoipos(lim)), nf_PARTIALFACT);
   NF = nfinit_complete(&S, 0, DEFAULTPREC);
   *pnf = nf = proper_nf(NF);
@@ -437,12 +437,12 @@ nfroots(GEN nf,GEN pol)
   A = RgX_nffix("nfroots", T,pol,1);
   d = degpol(A);
   if (d < 0) pari_err_ROOTS0("nfroots");
-  if (d == 0) return cgetg(1,t_VEC);
+  if (d == 0) return cgetg(1,t_COL);
   if (d == 1)
   {
     A = QXQX_normalize(A,T);
     A = mkpolmod(gneg_i(gel(A,2)), T);
-    return gerepilecopy(av, mkvec(A));
+    return gerepilecopy(av, mkcol(A));
   }
   dT = degpol(T);
   if (dT == 1) return gerepileupto(av, nfrootsQ(simplify_shallow(A)));
@@ -465,7 +465,7 @@ nfroots(GEN nf,GEN pol)
     z = nfsqff(nf,B, ROOTS, den);
   z = gerepileupto(av, QXQV_to_mod(z, T));
   gen_sort_inplace(z, (void*)&cmp_RgX, &cmp_nodata, NULL);
-  return z;
+  settyp(z, t_COL); return z;
 }
 
 static GEN
@@ -565,7 +565,7 @@ fact_from_sqff(GEN rep, GEN A, GEN B, GEN y, GEN T, GEN bad)
     }
     else
     { /* compute valuations mod a prime of degree 1 (avoid coeff explosion) */
-      GEN quo, p, r, Bp, lb = leading_coeff(B), E = cgetalloc(t_VECSMALL,n+1);
+      GEN quo, p, r, Bp, lb = leading_coeff(B), E = cgetalloc(n+1,t_VECSMALL);
       pari_sp av1 = avma;
       ulong pp;
       long j;
@@ -1643,7 +1643,7 @@ nf_DDF_roots(GEN pol, GEN polred, GEN nfpol, long fl, nflift_t *L)
     else if (fl == ROOTS_SPLIT) return cgetg(1, t_VEC);
   }
   if (DEBUGLEVEL > 3) err_printf(" done\n");
-  z[0] = evaltyp(t_VEC) | evallg(m);
+  z[0] = evaltyp(t_VEC) | _evallg(m);
   return z;
 }
 
@@ -1788,7 +1788,7 @@ nf_pick_prime(GEN nf, GEN pol, long fl, GEN *lt, GEN *Tp, ulong *pp)
 
 /* Assume lt(T) is a t_INT and T square free. Return t_VEC of irred. factors */
 static GEN
-nfsqff_trager(GEN u, GEN T, GEN dent)
+nfsqff_trager(GEN u, GEN T, GEN dent, long fl)
 {
   long k = 0, i, lx;
   GEN U, P, x0, mx0, fa, n = ZX_ZXY_rnfequation(T, u, &k);
@@ -1796,7 +1796,7 @@ nfsqff_trager(GEN u, GEN T, GEN dent)
   if (DEBUGLEVEL>4) err_printf("nfsqff_trager: choosing k = %ld\n",k);
 
   /* n guaranteed to be squarefree */
-  fa = ZX_DDF(Q_primpart(n)); lx = lg(fa);
+  fa = ZX_DDF_max(Q_primpart(n),fl==ROOTS || fl==ROOTS_SPLIT ? degpol(T): 0); lx = lg(fa);
   if (lx == 2) return mkvec(u);
 
   tmonic = is_pm1(leading_coeff(T));
@@ -1843,7 +1843,7 @@ polfnf(GEN a, GEN T)
   (void)nfgcd_all(A,RgX_deriv(A), T, dent, &B);
   if (degpol(B) != dA) B = Q_primpart( QXQX_normalize(B, T) );
   ensure_lt_INT(B);
-  y = nfsqff_trager(B, T, dent);
+  y = nfsqff_trager(B, T, dent, FACTORS);
   fact_from_sqff(rep, A, B, y, T, bad);
   return sort_factor_pol(rep, cmp_RgX);
 }
@@ -1885,7 +1885,7 @@ nfsqff(GEN nf, GEN pol, long fl, GEN den)
     GEN z;
     if (DEBUGLEVEL>2) err_printf("Using Trager's method\n");
     if (typ(nf) != t_POL) den =  mulii(den, nf_get_index(nf));
-    z = nfsqff_trager(Q_primpart(pol), nfpol, den);
+    z = nfsqff_trager(Q_primpart(pol), nfpol, den, fl);
     if (fl != FACTORS) {
       long i, l = lg(z);
       for (i = 1; i < l; i++)
