@@ -2001,7 +2001,7 @@ QXQ_charpoly(GEN A, GEN T, long v)
 {
   pari_sp av = avma;
   GEN den, B = Q_remove_denom(A, &den);
-  GEN P = ZXQ_charpoly(B, T, v);
+  GEN P = ZXQ_charpoly(B, Q_primpart(T), v);
   return gerepilecopy(av, den ? RgX_rescale(P, ginv(den)): P);
 }
 
@@ -3323,8 +3323,8 @@ ffmap_i(GEN m, GEN x)
     case t_POL: case t_RFRAC: case t_SER:
     case t_VEC: case t_COL: case t_MAT:
       y = cgetg_copy(x, &lx);
-      for (i=1; i<lontyp[tx]; i++) y[i] = x[1];
-      for (i=lontyp[tx]; i<lx; i++)
+      for (i = 1; i < lontyp[tx]; i++) y[i] = x[i];
+      for (; i < lx; i++)
       {
         GEN yi = ffmap_i(m, gel(x,i));
         if (!yi) return NULL;
