@@ -1962,11 +1962,6 @@ quadnegclassnou(long D, long *pD0, GEN *pP, GEN *pE)
 {
   ulong d = (ulong)-D, d0 = coredisc2u_fact(factoru(d), -1, pP, pE);
   ulong h = uquadclassnoF_fact(d0, -1, *pP, *pE) * quadclassnos(-(long)d0);
-  if (d0 != d) switch(d0)
-  {
-    case 4: h >>= 1; break;
-    case 3: h /= 3; break;
-  }
   *pD0 = -(long)d0; return h;
 }
 
@@ -2100,7 +2095,7 @@ polclass0(long D, long inv, long vx, GEN *db)
     GEN v;
     if (!p) { del++; continue; }
     v = Flv_roots_to_pol(gel(H,i), p, vx); l = lg(v);
-    *++v = evaltyp(t_VECSMALL) | evallg(l-1); /* Flx_to_Flv inplace */
+    *++v = evaltyp(t_VECSMALL) | _evallg(l-1); /* Flx_to_Flv inplace */
     uel(plist, j) = p;
     gel(H, j++) = gerepileuptoleaf(av2, v);
   }
