@@ -397,55 +397,28 @@ _uisprime(ulong n)
 int
 uisprime(ulong n)
 {
-  if (n < 103)
-    switch(n)
-    {
-      case 2:
-      case 3:
-      case 5:
-      case 7:
-      case 11:
-      case 13:
-      case 17:
-      case 19:
-      case 23:
-      case 29:
-      case 31:
-      case 37:
-      case 41:
-      case 43:
-      case 47:
-      case 53:
-      case 59:
-      case 61:
-      case 67:
-      case 71:
-      case 73:
-      case 79:
-      case 83:
-      case 89:
-      case 97:
-      case 101: return 1;
-      default: return 0;
-    }
+  if (!odd(n)) return n == 2;
+  if (n <= maxprimelim()) return PRIMES_search(n) > 0;
   /* gcd-extraction is much slower */
-  return odd(n) && n % 3 && n % 5 && n % 7 && n % 11 && n % 13 && n % 17
-                && n % 19 && n % 23 && n % 29 && n % 31 && n % 37 && n % 41
-                && (n < 1849 || _uisprime(n));
+  return n % 3 && n % 5 && n % 7 && n % 11 && n % 13 && n % 17
+      && n % 19 && n % 23 && n % 29 && n % 31 && n % 37 && n % 41
+      && _uisprime(n);
 }
 
-/* assume no prime divisor <= 101 */
+/* assume n <= maxprimelim() or no prime divisor <= 101 */
 int
 uisprime_101(ulong n)
 {
+  if (n <= maxprimelim()) return PRIMES_search(n) > 0;
   if (n < 1016801) return n < 10609? 1: (uispsp(2, n) && !is_2_prp_101(n));
   return _uisprime(n);
 }
 
-/* assume no prime divisor <= 661 */
+/* assume n <= maxprimelim() or no prime divisor <= 661 */
 int
 uisprime_661(ulong n)
 {
+  if (n <= maxprimelim()) return PRIMES_search(n) > 0;
   if (n < 1016801) return n < 452929? 1: uispsp(2, n);
   return _uisprime(n);
 }
@@ -480,10 +453,10 @@ BPSW_psp(GEN N)
   return gc_long(av, is2psp(N) && islucaspsp(N));
 }
 
-/* can we write n = x^k ? Assume N has no prime divisor <= 2^14.
- * Not memory clean */
+/* can we write n = x^k ? Assume N has no prime divisor <= 2^cutoff, else may
+ * miss some powers. Not memory clean */
 long
-isanypower_nosmalldiv(GEN N, GEN *px)
+Z_isanypower_nosmalldiv(GEN N, ulong cutoff, GEN *px)
 {
   GEN x = N, y;
   ulong mask = 7;
@@ -492,12 +465,13 @@ isanypower_nosmalldiv(GEN N, GEN *px)
   while (Z_issquareall(x, &y)) { k <<= 1; x = y; }
   while ( (ex = is_357_power(x, &y, &mask)) ) { k *= ex; x = y; }
   (void)u_forprime_init(&T, 11, ULONG_MAX);
-  /* stop when x^(1/k) < 2^14 */
-  while ( (ex = is_pth_power(x, &y, &T, 15)) ) { k *= ex; x = y; }
+  /* stop when x^(1/k) < 2^cutoff */
+  while ( (ex = is_pth_power(x, &y, &T, cutoff)) ) { k *= ex; x = y; }
   *px = x; return k;
 }
 
-/* no prime divisor <= 2^14 (> 661) */
+/* assume N has no prime divisor < 661. When N > 2^512, assume further
+ * that it has no prime divisor < 2^14. */
 long
 BPSW_psp_nosmalldiv(GEN N)
 {
@@ -508,7 +482,7 @@ BPSW_psp_nosmalldiv(GEN N)
   av = avma;
   /* N large: test for pure power, rarely succeeds, but requires < 1% of
    * compositeness test times */
-  if (bit_accuracy(l) > 512 && isanypower_nosmalldiv(N,&N) != 1)
+  if (bit_accuracy(l) > 512 && Z_isanypower_nosmalldiv(N, 15, &N) != 1)
     return gc_long(av,0);
   N = absi_shallow(N);
   return gc_long(av, is2psp(N) && islucaspsp(N));
@@ -949,9 +923,10 @@ RESET:
 static GEN
 prime_table_find_n(ulong N)
 {
-  byteptr d;
-  ulong n, p, maxp = maxprime();
+  ulong n, p, maxp;
   long i;
+  if (N <= (ulong)pari_PRIMES[0]) return utoipos((ulong)pari_PRIMES[N]);
+  /* not in table */
   for (i = 1; i < prime_table_len; i++)
   {
     n = prime_table[i].n;
@@ -965,33 +940,17 @@ prime_table_find_n(ulong N)
   if (i == prime_table_len) i = prime_table_len - 1;
   p = prime_table[i].p;
   n = prime_table[i].n;
-  if (n > N && p > maxp)
+  if (n > N)
   {
     i--;
     p = prime_table[i].p;
     n = prime_table[i].n;
   }
-  /* if beyond prime table, then n <= N */
-  d = diffptr + n;
-  if (n > N)
-  {
-    n -= N;
-    do { n--; PREC_PRIME_VIADIFF(p,d); } while (n) ;
-  }
-  else if (n < N)
-  {
-    ulong maxpN = maxprimeN();
-    if (N >= maxpN)
-    {
-      if (N == maxpN) return utoipos(maxp);
-      if (p < maxp) { p = maxp; n = maxpN; }
-      return prime_successor(p, N - n);
-    }
-    /* can find prime(N) in table */
-    n = N - n;
-    do { n--; NEXT_PRIME_VIADIFF(p,d); } while (n) ;
-  }
-  return utoipos(p);
+  /* n <= N */
+  if (n == N) return utoipos(p);
+  maxp = maxprime();
+  if (p < maxp) { p = maxp; n = pari_PRIMES[0]; }
+  return prime_successor(p, N - n);
 }
 
 ulong
@@ -1015,6 +974,8 @@ prime(long N)
   set_avma(av); return icopy(p);
 }
 
+/* N encodes an interval in which to draw a random uniform prime;
+ * decoded as [a, b], d = b-a+1 */
 static void
 prime_interval(GEN N, GEN *pa, GEN *pb, GEN *pd)
 {
@@ -1062,7 +1023,7 @@ prime_interval(GEN N, GEN *pa, GEN *pb, GEN *pd)
   *pa = a; *pb = b; *pd = d;
 }
 
-/* random b-bit prime */
+/* random prime */
 GEN
 randomprime(GEN N)
 {
@@ -1099,7 +1060,13 @@ randomprime0(GEN N, GEN q)
   if (!N) N = int2n(31);
   prime_interval(N, &a, &b, &d);
   r = modii(subii(C, a), D);
-  if (signe(r)) { a = addii(a, r); d = subii(d, r); }
+  if (signe(r))
+  {
+    a = addii(a, r);
+    if (cmpii(a,b) > 0)
+      pari_err(e_MISC, "no prime satisfies congruence in interval");
+    d = subii(d, r);
+  }
   if (!equali1(gcdii(C,D)))
   {
     if (isprime(a)) return gerepilecopy(av, a);
@@ -1116,66 +1083,44 @@ randomprime0(GEN N, GEN q)
   return NULL;
 }
 
-/* set *pp = nextprime(a) = p
- *     *pd so that NEXT_PRIME_VIADIFF(d, p) = nextprime(p+1)
- *     *pn so that p = the n-th prime
- * error if nextprime(a) is out of primetable bounds */
-void
-prime_table_next_p(ulong a, byteptr *pd, ulong *pp, ulong *pn)
+/* cf gen_search; assume 0 <= x <= maxprimelim(). Return i > 0 such that
+ * x = T[i] or -i < 0 such that T[i-1] < x < T[i]; return -1 for the special
+ * case 0 <= x < 2 */
+long
+PRIMES_search(ulong x)
 {
-  byteptr d;
-  ulong p, n, maxp = maxprime();
-  long i = prime_table_closest_p(a);
-  p = prime_table[i].p;
-  if (p > a && p > maxp)
+  pari_prime *T = pari_PRIMES;
+  ulong i, u = minuu(T[0], (x + 2) >> ((x < 122UL)? 1: 2)), l = 1;
+  do
   {
-    i--;
-    p = prime_table[i].p;
-  }
-  /* if beyond prime table, then p <= a */
-  n = prime_table[i].n;
-  d = diffptr + n;
-  if (p < a)
-  {
-    if (a > maxp) pari_err_MAXPRIME(a);
-    do { n++; NEXT_PRIME_VIADIFF(p,d); } while (p < a);
-  }
-  else if (p != a)
-  {
-    do { n--; PREC_PRIME_VIADIFF(p,d); } while (p > a) ;
-    if (p < a) { NEXT_PRIME_VIADIFF(p,d); n++; }
-  }
-  *pn = n;
-  *pp = p;
-  *pd = d;
+    i = (l+u) >> 1;
+    if (x < T[i]) u = i-1; else if (x > T[i]) l = i+1; else return i;
+  } while (u > l);
+  if (u == l) { i = l; if (x == T[i]) return i; }
+  return -(long)(T[i] > x? i: i+1);
 }
 
 ulong
 uprimepi(ulong a)
 {
-  ulong p, n, maxp = maxprime();
-  if (a <= maxp)
+  ulong maxp, p, n;
+  forprime_t S;
+  long i;
+  if (a <= maxprimelim())
   {
-    byteptr d;
-    prime_table_next_p(a, &d, &p, &n);
-    return p == a? n: n-1;
-  }
-  else
-  {
-    long i = prime_table_closest_p(a);
-    forprime_t S;
-    p = prime_table[i].p;
-    if (p > a)
-    {
-      i--;
-      p = prime_table[i].p;
-    }
-    /* p = largest prime in table <= a */
-    n = prime_table[i].n;
-    (void)u_forprime_init(&S, p+1, a);
-    for (; p; n++) p = u_forprime_next(&S);
-    return n-1;
+    i = PRIMES_search(a);
+    return (ulong)(i > 0? i: -i-1);
   }
+  /* a > maxprimelim >= maxprime */
+  i = prime_table_closest_p(a);
+  p = prime_table[i].p;
+  if (p > a) p = prime_table[--i].p;
+  /* p = largest prime in prime_table <= a and a > maxprime */
+  maxp = maxprime(); /* replace p by maxprime if larger */
+  if (p > maxp) n = prime_table[i].n; else { p = maxp; n = maxprimeN(); }
+  (void)u_forprime_init(&S, p+1, a);
+  for (; p; n++) p = u_forprime_next(&S);
+  return n-1;
 }
 
 GEN
@@ -1188,7 +1133,7 @@ primepi(GEN x)
   long i;
   if (typ(N) != t_INT) pari_err_TYPE("primepi",N);
   if (signe(N) <= 0) return gen_0;
-  if (lgefint(N) == 3) { n = N[2]; set_avma(av); return utoi(uprimepi(n)); }
+  if (lgefint(N) == 3) return gc_utoi(av, uprimepi(N[2]));
   i = prime_table_len-1;
   p = prime_table[i].p;
   n = prime_table[i].n;
@@ -1272,7 +1217,7 @@ primes(long n)
   if (n <= 0) return cgetg(1, t_VEC);
   y = cgetg(n+1, t_VEC);
   (void)new_chunk(3*n); /*HACK*/
-  u_forprime_init(&S, 2, ULONG_MAX);
+  u_forprime_init(&S, 2, (ulong)n > maxprimeN()? ULONG_MAX: maxprime());
   set_avma((pari_sp)y);
   for (i = 1; i <= n; i++) gel(y, i) = utoipos( u_forprime_next(&S) );
   return y;
@@ -1285,7 +1230,8 @@ primes_zv(long n)
   GEN y;
   if (n <= 0) return cgetg(1, t_VECSMALL);
   y = cgetg(n+1,t_VECSMALL);
-  u_forprime_init(&S, 2, ULONG_MAX);
+  /* don't initialize sieve if all fits in primetable */
+  u_forprime_init(&S, 2, (ulong)n > maxprimeN()? ULONG_MAX: maxprime());
   for (i = 1; i <= n; i++) y[i] =  u_forprime_next(&S);
   set_avma((pari_sp)y); return y;
 }
@@ -1345,14 +1291,31 @@ primes_interval(GEN a, GEN b)
   setlg(y, i); return gerepileupto(av, y);
 }
 
-/* a <= b, at most d primes in [a,b]. Return them */
+/* simpler case a <= b <= maxprimelim() */
+static GEN
+PRIMES_interval(ulong a, ulong b)
+{
+  long k, i = PRIMES_search(a), j = PRIMES_search(b);
+  GEN y;
+  if (i < 0) i = -i;
+  if (j < 0) j = -j - 1;
+  /* T[i] = smallest p >= a, T[j] = largest p <= b */
+  y = cgetg(j - i + 2, t_VECSMALL);
+  if (i == 1)
+    for (; i <= j; i++) y[i] = pari_PRIMES[i];
+  else
+    for (k = 1; i <= j; i++) y[k++] = pari_PRIMES[i];
+  return y;
+}
+/* a <= b, at most d primes in [a,b]. Return them.
+ * PRIMES_interval is more efficient if b <= maxprimelim() */
 static GEN
 primes_interval_i(ulong a, ulong b, ulong d)
 {
-  ulong p, i = 1, n = d + 1;
-  forprime_t S;
+  ulong p, i = 1, n = d+1;
   GEN y = cgetg(n+1, t_VECSMALL);
   pari_sp av = avma;
+  forprime_t S;
   u_forprime_init(&S, a, b);
   while ((p = u_forprime_next(&S))) y[i++] = p;
   set_avma(av); setlg(y, i); stackdummy((pari_sp)(y + i), (pari_sp)(y + n+1));
@@ -1362,8 +1325,9 @@ GEN
 primes_interval_zv(ulong a, ulong b)
 {
   ulong d;
-  if (!a) return primes_upto_zv(b);
+  if (a < 3) return primes_upto_zv(b);
   if (b < a) return cgetg(1, t_VECSMALL);
+  if (b <= maxprimelim()) return PRIMES_interval(a, b);
   d = b - a;
   if (d > 100000UL)
   {
@@ -1377,6 +1341,7 @@ primes_upto_zv(ulong b)
 {
   ulong d;
   if (b < 2) return cgetg(1, t_VECSMALL);
+  if (b <= maxprimelim()) return PRIMES_interval(2, b);
   d = (b > 100000UL)? (ulong)primepi_upper_bound(b): b;
   return primes_interval_i(2, b, d);
 }
