@@ -33,13 +33,13 @@ trans_fix_arg(long *prec, GEN *s0, GEN *sig, GEN *tau, pari_sp *av, GEN *res)
   *res = cgetc(l); *av = avma;
   if (typ(s) == t_COMPLEX)
   { /* s = sig + i t */
-    s = cxtofp(s, l+EXTRAPRECWORD);
+    s = cxtofp(s, l+EXTRAPREC64);
     *sig = gel(s,1);
     *tau = gel(s,2);
   }
   else /* real number */
   {
-    *sig = s = gtofp(s, l+EXTRAPRECWORD);
+    *sig = s = gtofp(s, l+EXTRAPREC64);
     *tau = gen_0;
     p1 = trunc2nr(s, 0);
     if (!signe(subri(s,p1))) *s0 = p1;
@@ -68,7 +68,7 @@ mpatan(GEN x)
   if (!sx) return real_0_bit(expo(x));
   l = lp = realprec(x);
   if (absrnz_equal1(x)) { /* |x| = 1 */
-    y = Pi2n(-2, l+EXTRAPRECWORD); if (sx < 0) setsigne(y,-1);
+    y = Pi2n(-2, l+EXTRAPREC64); if (sx < 0) setsigne(y,-1);
     return y;
   }
   if (l > AGM_ATAN_LIMIT)
@@ -78,7 +78,7 @@ mpatan(GEN x)
   if (e > 0) lp += nbits2extraprec(e);
 
   y = cgetr(lp); av0 = avma;
-  p1 = rtor(x, l+EXTRAPRECWORD); setabssign(p1); /* p1 = |x| */
+  p1 = rtor(x, l+EXTRAPREC64); setabssign(p1); /* p1 = |x| */
   if (inv) p1 = invr(p1);
   e = expo(p1);
   if (e < -100)
@@ -111,7 +111,7 @@ mpatan(GEN x)
     p5 = addsr(1, sqrtr_abs(p5)); setprec(p5,l2);
     affrr(divrr(p2,p5), p2); set_avma(av);
   }
-  p3 = sqrr(p2); l1 = minss(LOWDEFAULTPREC+EXTRAPRECWORD, l2); /* l1 increases to l2 */;
+  p3 = sqrr(p2); l1 = minss(LOWDEFAULTPREC+EXTRAPREC64, l2); /* l1 increases to l2 */;
   unr = real_1(l2); setprec(unr,l1);
   p4 = cgetr(l2); setprec(p4,l1);
   affrr(divru(unr,2*n+1), p4);
@@ -119,7 +119,8 @@ mpatan(GEN x)
   for (i = n; i > 1; i--) /* n >= 1. i = 1 done outside for efficiency */
   {
     setprec(p3,l1); p5 = mulrr(p4,p3);
-    l1 += dvmdsBIL(s - e, &s); if (l1 > l2) l1 = l2;
+    l1 += nbits2extraprec(dvmdsBIL(s - e, &s)<<TWOPOTBITS_IN_LONG);
+    if (l1 > l2) l1 = l2;
     setprec(unr,l1); p5 = subrr(divru(unr,2*i-1), p5);
     setprec(p4,l1); affrr(p5,p4); set_avma(av);
   }
@@ -146,11 +147,11 @@ gatan(GEN x, long prec)
       av = avma; return gerepilecopy(av, mulcxmI(gatanh(mulcxI(x),prec)));
     default:
       av = avma; if (!(y = toser_i(x))) break;
-      if (valp(y) < 0) pari_err_DOMAIN("atan","valuation", "<", gen_0, x);
+      if (valser(y) < 0) pari_err_DOMAIN("atan","valuation", "<", gen_0, x);
       if (lg(y)==2) return gerepilecopy(av, y);
       /* lg(y) > 2 */
       a = integser(gdiv(derivser(y), gaddsg(1,gsqr(y))));
-      if (!valp(y)) a = gadd(a, gatan(gel(y,2),prec));
+      if (!valser(y)) a = gadd(a, gatan(gel(y,2),prec));
       return gerepileupto(av, a);
   }
   return trans_eval("atan",gatan,x,prec);
@@ -203,17 +204,17 @@ gasin(GEN x, long prec)
       av = avma; if (!(y = toser_i(x))) break;
       if (gequal0(y)) return gerepilecopy(av, y);
       /* lg(y) > 2*/
-      if (valp(y) < 0) pari_err_DOMAIN("asin","valuation", "<", gen_0, x);
+      if (valser(y) < 0) pari_err_DOMAIN("asin","valuation", "<", gen_0, x);
       p1 = gsubsg(1,gsqr(y));
       if (gequal0(p1))
       {
         GEN t = Pi2n(-1,prec);
         if (gsigne(gel(y,2)) < 0) setsigne(t, -1);
-        return gerepileupto(av, scalarser(t, varn(y), valp(p1)>>1));
+        return gerepileupto(av, scalarser(t, varn(y), valser(p1)>>1));
       }
       p1 = gdiv(derivser(y), gsqrt(p1,prec));
       a = integser(p1);
-      if (!valp(y)) a = gadd(a, gasin(gel(y,2),prec));
+      if (!valser(y)) a = gadd(a, gasin(gel(y,2),prec));
       return gerepileupto(av, a);
   }
   return trans_eval("asin",gasin,x,prec);
@@ -270,17 +271,17 @@ gacos(GEN x, long prec)
       return gerepilecopy(av, mulcxmI(y));
     default:
       av = avma; if (!(y = toser_i(x))) break;
-      if (valp(y) < 0) pari_err_DOMAIN("acos","valuation", "<", gen_0, x);
+      if (valser(y) < 0) pari_err_DOMAIN("acos","valuation", "<", gen_0, x);
       if (lg(y) > 2)
       {
         p1 = gsubsg(1,gsqr(y));
-        if (gequal0(p1)) { set_avma(av); return zeroser(varn(y), valp(p1)>>1); }
+        if (gequal0(p1)) { set_avma(av); return zeroser(varn(y), valser(p1)>>1); }
         p1 = integser(gdiv(gneg(derivser(y)), gsqrt(p1,prec)));
         /*y(t) = 1+O(t)*/
-        if (gequal1(gel(y,2)) && !valp(y)) return gerepileupto(av, p1);
+        if (gequal1(gel(y,2)) && !valser(y)) return gerepileupto(av, p1);
       }
       else p1 = y;
-      a = (lg(y)==2 || valp(y))? Pi2n(-1, prec): gacos(gel(y,2),prec);
+      a = (lg(y)==2 || valser(y))? Pi2n(-1, prec): gacos(gel(y,2),prec);
       return gerepileupto(av, gadd(a,p1));
   }
   return trans_eval("acos",gacos,x,prec);
@@ -394,8 +395,8 @@ gcosh(GEN x, long prec)
       return gerepileupto(av, gmul2n(p1,-1));
     default:
       av = avma; if (!(y = toser_i(x))) break;
-      if (gequal0(y) && valp(y) == 0) return gerepilecopy(av, y);
-      v = valp(y);
+      if (gequal0(y) && valser(y) == 0) return gerepilecopy(av, y);
+      v = valser(y);
       if (v > 0) y = sertoser(y, lg(y) - 2 + v);
       p1 = gexp(y,prec); p1 = gadd(p1, ginv(p1));
       return gerepileupto(av, gmul2n(p1,-1));
@@ -413,16 +414,17 @@ static GEN
 mpsinh(GEN x)
 {
   pari_sp av;
-  long ex = expo(x), lx;
+  long lx;
   GEN z, res;
 
-  if (!signe(x)) return mpsinh0(ex);
+  if (!signe(x)) return mpsinh0(expo(x));
   lx = realprec(x); res = cgetr(lx); av = avma;
-  if (ex < 1 - BITS_IN_LONG)
+  if (expo(x) + BITS_IN_LONG < 1)
   { /* y = e^x-1; e^x - e^(-x) = y(1 + 1/(y+1)) */
     GEN y = mpexpm1(x);
-    z = addrs(y,1); if (realprec(z) > lx+1) z = rtor(z,lx+1); /* e^x */
-    z = mulrr(y, addsr(1,invr(z)));
+    lx += EXTRAPRECWORD;
+    z = addrs(y, 1); if (realprec(z) > lx) z = rtor(z,lx); /* e^x */
+    z = mulrr(y, addsr(1, invr(z)));
   }
   else
   {
@@ -437,20 +439,22 @@ void
 mpsinhcosh(GEN x, GEN *s, GEN *c)
 {
   pari_sp av;
-  long lx, ex = expo(x);
+  long lx, ex;
   GEN z, zi, S, C;
   if (!signe(x))
   {
+    ex = expo(x);
     *c = mpcosh0(ex);
     *s = mpsinh0(ex); return;
   }
   lx = realprec(x);
   *c = cgetr(lx);
   *s = cgetr(lx); av = avma;
-  if (ex < 1 - BITS_IN_LONG)
+  if (expo(x) + BITS_IN_LONG < 1)
   { /* y = e^x-1; e^x - e^(-x) = y(1 + 1/(y+1)) */
     GEN y = mpexpm1(x);
-    z = addrs(y,1); if (realprec(z) > lx+1) z = rtor(z,lx+1); /* e^x */
+    lx += EXTRAPRECWORD;
+    z = addrs(y,1); if (realprec(z) > lx) z = rtor(z, lx); /* e^x */
     zi = invr(z); /* z = exp(x), zi = exp(-x) */
     S = mulrr(y, addsr(1,zi));
   }
@@ -482,7 +486,7 @@ gsinh(GEN x, long prec)
       return gerepileupto(av, gmul2n(p1,-1));
     default:
       av = avma; if (!(y = toser_i(x))) break;
-      if (gequal0(y) && valp(y) == 0) return gerepilecopy(av, y);
+      if (gequal0(y) && valser(y) == 0) return gerepilecopy(av, y);
       p1 = gexp(y, prec); p1 = gsub(p1, ginv(p1));
       return gerepileupto(av, gmul2n(p1,-1));
   }
@@ -506,9 +510,9 @@ mptanh(GEN x)
     y = real_1(lx);
   } else {
     pari_sp av = avma;
-    long ex = expo(x);
+    long e = expo(x) + BITS_IN_LONG;
     GEN t;
-    if (ex < 1 - BITS_IN_LONG) x = rtor(x, lx + nbits2extraprec(-ex)-1);
+    if (e < 0) x = rtor(x, lx + nbits2extraprec(-e));
     t = exp1r_abs(gmul2n(x,1)); /* exp(|2x|) - 1 */
     y = gerepileuptoleaf(av, divrr(t, addsr(2,t)));
   }
@@ -556,9 +560,9 @@ mpcotanh(GEN x)
     y = real_1(lx);
   } else {
     pari_sp av = avma;
-    long ex = expo(x);
+    long e = expo(x) + BITS_IN_LONG;
     GEN t;
-    if (ex < 1 - BITS_IN_LONG) x = rtor(x, lx + nbits2extraprec(-ex)-1);
+    if (e < 0) x = rtor(x, lx + nbits2extraprec(-e));
     t = exp1r_abs(gmul2n(x,1)); /* exp(|2x|) - 1 */
     y = gerepileuptoleaf(av, divrr(addsr(2,t), t));
   }
@@ -601,10 +605,10 @@ gcotanh(GEN x, long prec)
 static GEN
 mpasinh(GEN x)
 {
-  long lx = realprec(x), ex = expo(x);
+  long lx = realprec(x), e = expo(x) + BITS_IN_LONG;
   GEN z, res = cgetr(lx);
   pari_sp av = avma;
-  if (ex < 1 - BITS_IN_LONG) x = rtor(x, lx + nbits2extraprec(-ex)-1);
+  if (e < 0) x = rtor(x, lx + nbits2extraprec(-e));
   z = logr_abs( addrr_sign(x,1, sqrtr_abs( addrs(sqrr(x), 1) ), 1) );
   if (signe(x) < 0) togglesign(z);
   affrr(z, res); return gc_const(av, res);
@@ -641,17 +645,17 @@ gasinh(GEN x, long prec)
     default:
       av = avma; if (!(y = toser_i(x))) break;
       if (gequal0(y)) return gerepilecopy(av, y);
-      if (valp(y) < 0) pari_err_DOMAIN("asinh","valuation", "<", gen_0, x);
+      if (valser(y) < 0) pari_err_DOMAIN("asinh","valuation", "<", gen_0, x);
       p1 = gaddsg(1,gsqr(y));
       if (gequal0(p1))
       {
         GEN t = PiI2n(-1,prec);
         if ( gsigne(imag_i(gel(y,2))) < 0 ) setsigne(gel(t,2), -1);
-        return gerepileupto(av, scalarser(t, varn(y), valp(p1)>>1));
+        return gerepileupto(av, scalarser(t, varn(y), valser(p1)>>1));
       }
       p1 = gdiv(derivser(y), gsqrt(p1,prec));
       a = integser(p1);
-      if (!valp(y)) a = gadd(a, gasinh(gel(y,2),prec));
+      if (!valser(y)) a = gadd(a, gasinh(gel(y,2),prec));
       return gerepileupto(av, a);
   }
   return trans_eval("asinh",gasinh,x,prec);
@@ -720,7 +724,7 @@ gacosh(GEN x, long prec)
       GEN a, d;
       long v;
       av = avma; if (!(y = toser_i(x))) break;
-      v = valp(y);
+      v = valser(y);
       if (v < 0) pari_err_DOMAIN("acosh","valuation", "<", gen_0, x);
       if (gequal0(y))
       {
@@ -728,7 +732,7 @@ gacosh(GEN x, long prec)
         return gerepileupto(av, gadd(y, PiI2n(-1, prec)));
       }
       d = gsubgs(gsqr(y),1);
-      if (gequal0(d)) { set_avma(av); return zeroser(varn(y), valp(d)>>1); }
+      if (gequal0(d)) { set_avma(av); return zeroser(varn(y), valser(d)>>1); }
       d = gdiv(derivser(y), gsqrt(d,prec));
       a = integser(d);
       if (v)
@@ -759,7 +763,7 @@ mpatanh(GEN x)
   z = s > 0? subsr(1,x): addsr(1,x); e = expo(z);
   if (e < -5)
   {
-    x = rtor(x, realprec(x) + nbits2extraprec(-e)-1);
+    x = rtor(x, realprec(x) + nbits2extraprec(-e)-EXTRAPRECWORD);
     z = s > 0? subsr(1,x): addsr(1,x); e = expo(z);
   }
   z = invr(z); shiftr_inplace(z, 1); /* 2/(1-|x|) */
@@ -906,10 +910,10 @@ gatanh(GEN x, long prec)
 
     default:
       av = avma; if (!(y = toser_i(x))) break;
-      if (valp(y) < 0) pari_err_DOMAIN("atanh","valuation", "<", gen_0, x);
+      if (valser(y) < 0) pari_err_DOMAIN("atanh","valuation", "<", gen_0, x);
       z = gdiv(derivser(y), gsubsg(1,gsqr(y)));
       a = integser(z);
-      if (!valp(y)) a = gadd(a, gatanh(gel(y,2),prec));
+      if (!valser(y)) a = gadd(a, gatanh(gel(y,2),prec));
       return gerepileupto(av, a);
   }
   return trans_eval("atanh",gatanh,x,prec);
@@ -945,8 +949,8 @@ _mul(void *data, GEN x, GEN y)
 {
   long prec = (long)data;
   /* switch to t_REAL ? */
-  if (typ(x) == t_INT && lgefint(x) > prec) x = itor(x, prec);
-  if (typ(y) == t_INT && lgefint(y) > prec) y = itor(y, prec);
+  if (typ(x) == t_INT && lg2prec(lgefint(x)) > prec) x = itor(x, prec);
+  if (typ(y) == t_INT && lg2prec(lgefint(y)) > prec) y = itor(y, prec);
   return mpmul(x, y);
 }
 static GEN
@@ -1010,7 +1014,7 @@ static GEN
 lngamma1(GEN z, long prec)
 { /* sum_{i > l} |z|^(i-1) = |z|^l / (1-|z|) < 2^-B
    * for l > (B+1) / |log2(|z|)| */
-  long i, l = ceil((bit_accuracy(prec) + 1) / - dbllog2(z));
+  long i, l = ceil((prec2nbits(prec) + 1) / - dbllog2(z));
   GEN s, vz;
 
   if (l <= 1) return gmul(negeuler(prec), z);
@@ -1115,7 +1119,7 @@ gamma_use_1(double s, double t, long prec, long *plim, long *pN)
   if (d < 1e-16) return 1;
   gamma_optim(s, t, prec, plim, pN);
   if (d >= 0.5) return 0;
-  k = bit_accuracy(prec) / -log2(dnorm(a, t)); /* 2k = lngamma1 bound */
+  k = prec2nbits(prec) / -log2(dnorm(a, t)); /* 2k = lngamma1 bound */
   return (t ? k: 1.5*k) < *plim + *pN;
 }
 static GEN
@@ -1324,7 +1328,7 @@ cxgamma(GEN s0, int dolog, long prec)
 static long
 gamma2_n(long prec)
 {
-  long b = bit_accuracy(prec);
+  long b = prec2nbits(prec);
   if (b <=  64) return 1450;
   if (b <= 128) return 1930;
   if (b <= 192) return 2750;
@@ -1352,7 +1356,7 @@ gammahs(long m, long prec)
   z = sqrtr( mppi(prec) );
   if (m)
   {
-    GEN t = mulu_interval_step_prec(1, ma-1, 2, prec + EXTRAPRECWORD);
+    GEN t = mulu_interval_step_prec(1, ma-1, 2, prec + EXTRAPREC64);
     if (m >= 0) z = mpmul(z,t);
     else
     {
@@ -1407,7 +1411,7 @@ gadw(GEN x, long p)
   GEN s, t, u = cgetg(p+1, t_VEC);
   long j, k, kp, n = nboft(precp(x)+valp(x)+1, p);
 
-  t = s = gaddsg(1, zeropadic(gel(x,2), n));
+  t = s = cvtop(gen_1, gel(x,2), n);
   gel(u, 1) = s;
   gel(u, 2) = s;
   for (j = 2; j < p; ++j)
@@ -1461,18 +1465,16 @@ Qp_gamma_Dwork(GEN x, long p)
 static GEN
 Qp_gamma_Morita(long n, GEN p, long e)
 {
-  pari_sp ltop=avma;
-  GEN p2 = gaddsg((n&1)?-1:1, zeropadic(p, e));
-  long i;
-  long pp=is_bigint(p)? 0: itos(p);
+  pari_sp av = avma;
+  GEN p2 = cvtop((n&1)? gen_m1: gen_1, p, e);
+  long i, pp = is_bigint(p)? 0: itos(p);
   for (i = 2; i < n; i++)
     if (!pp || i%pp)
     {
       p2 = gmulgu(p2, i);
-      if ((i&0xFL) == 0xFL)
-        p2 = gerepileupto(ltop, p2);
+      if ((i&0xFL) == 0xFL) p2 = gerepileupto(av, p2);
     }
-  return gerepileupto(ltop, p2);
+  return gerepileupto(av, p2);
 }
 
 /* x\in\N: Gamma(-x)=(-1)^(1+x+x\p)*Gamma(1+x) */
@@ -1492,7 +1494,7 @@ GEN
 Qp_gamma(GEN x)
 {
   GEN n, m, N, p = gel(x,2);
-  long s, e = precp(x);
+  long s, e = valp(x) + precp(x);
   if (absequaliu(p, 2) && e == 2) e = 1;
   if (valp(x) < 0) pari_err_DOMAIN("gamma","v_p(x)", "<", gen_0, x);
   n = gtrunc(x);
@@ -1529,7 +1531,7 @@ static GEN
 serlngamma0(GEN y, long prec)
 {
   GEN t;
-  if (valp(y)) pari_err_DOMAIN("lngamma","valuation", "!=", gen_0, y);
+  if (valser(y)) pari_err_DOMAIN("lngamma","valuation", "!=", gen_0, y);
   t = derivser(y);
   /* don't compute psi if y'=0 */
   if (signe(t)) t = gmul(t, gpsi(y,prec));
@@ -1542,7 +1544,7 @@ sergamma(GEN y, long prec)
   GEN z, y0, Y;
   if (lg(y) == 2) pari_err_DOMAIN("gamma", "argument", "=", gen_0,y);
   /* exp(lngamma) */
-  if (valp(y) > 0) return gdiv(gexp(glngamma(gaddgs(y,1),prec),prec),y);
+  if (valser(y) > 0) return gdiv(gexp(glngamma(gaddgs(y,1),prec),prec),y);
   y0 = simplify_shallow(gel(y,2));
   z = NULL; Y = y;
   if (isint(y0, &y0))
@@ -1733,12 +1735,12 @@ gammafrac24(GEN a, GEN b, long prec)
       m = A / B;
       x = A % B; /* = A - m*B */
       if (x < 0) { x += B; m--; } /* now 0 < x < B, A/B = x/B + m */
-      bit = bit_accuracy(prec);
+      bit = prec2nbits(prec);
       /* Depending on B and prec, we must experimentally replace the 0.5
        * by 0.4 to 2.0 for optimal value. Play safe. */
       if (labs(m) > 0.5 * bit * sqrt(bit) / log(bit)) return NULL;
       z0 = cgetr(prec); av = avma;
-      prec += EXTRAPRECWORD;
+      prec += EXTRAPREC64;
       z = gammafrac24_s(x, B, prec);
       if (m)
       {
@@ -1783,7 +1785,7 @@ ggamma(GEN x, long prec)
          * Gamma(x) = Pi / (sin(Pi z) * Gamma(z)) */
         GEN z = mkfrac(negi(c), b), q = ground(z), r = gsub(z,q);
         GEN pi = mppi(prec); /* |r| <= 1/2 */
-        z = fractor(z, prec+EXTRAPRECWORD);
+        z = fractor(z, prec+EXTRAPREC64);
         y = divrr(pi, mulrr(mpsin(gmul(pi, r)), cxgamma(z, 0, prec)));
         if (mpodd(q)) togglesign(y);
         return gerepileupto(av, y);
@@ -1792,7 +1794,7 @@ ggamma(GEN x, long prec)
       { /* 0 < x < 1/2 gamma would use funeq: adding 1 is cheaper. */
         if (expi(a) - expi(b) < -3) /* close to 0 */
         {
-          if (lgefint(b) >= prec) x = fractor(x,prec);
+          if (lg2prec(lgefint(b)) >= prec) x = fractor(x,prec);
           y = mpexp(lngamma1(x, prec));
         }
         else
@@ -1802,7 +1804,7 @@ ggamma(GEN x, long prec)
       if (expi(c) - expi(b) < -3)
       { /* x = 1 + c/b is close to 1 */
         x = mkfrac(c,b);
-        if (lgefint(b) >= prec) x = fractor(x,prec);
+        if (lg2prec(lgefint(b)) >= prec) x = fractor(x,prec);
         y = mpexp(lngamma1(x, prec));
       }
       else
@@ -1822,7 +1824,7 @@ static GEN
 mpfactr_basecase(long n, long prec)
 {
   GEN v = cgetg(expu(n) + 2, t_VEC);
-  long k, prec2 = prec + EXTRAPRECWORD;
+  long k, prec2 = prec + EXTRAPREC64;
   GEN a;
   for (k = 1;; k++)
   {
@@ -1844,7 +1846,7 @@ mpfactr_basecase(long n, long prec)
 static long
 mpfactr_n(long prec)
 {
-  long b = bit_accuracy(prec);
+  long b = prec2nbits(prec);
   if (b <=  64) return 1930;
   if (b <= 128) return 2650;
   if (b <= 192) return 3300;
@@ -1884,7 +1886,7 @@ mpfactr(long n, long prec)
 static ulong
 lngamma_n(long prec)
 {
-  long b = bit_accuracy(prec);
+  long b = prec2nbits(prec);
   double N;
   if (b <=  64) return 1450UL;
   if (b <= 128) return 2010UL;
@@ -1925,7 +1927,7 @@ glngamma(GEN x, long prec)
          * lngamma(x) = log |Pi / (sin(Pi z) * Gamma(z))| + I*Pi * floor(x) */
         GEN z = mkfrac(negi(c), b), q = ground(z), r = gsub(z,q);
         GEN pi = mppi(prec); /* |r| <= 1/2 */
-        z = fractor(z, prec+EXTRAPRECWORD);
+        z = fractor(z, prec+EXTRAPREC64);
         y = subrr(logr_abs(divrr(pi, mpsin(gmul(pi,r)))), cxgamma(z, 1, prec));
         y = gadd(y, mkcomplex(gen_0, mulri(pi, gfloor(x))));
         return gerepileupto(av, y);
@@ -1934,7 +1936,7 @@ glngamma(GEN x, long prec)
       { /* 0 < x < 1/2 gamma would use funeq: adding 1 is cheaper. */
         if (expi(a) - expi(b) < -3) /* close to 0 */
         {
-          if (lgefint(b) >= prec) x = fractor(x,prec);
+          if (lg2prec(lgefint(b)) >= prec) x = fractor(x,prec);
           y = lngamma1(x, prec);
         }
         else
@@ -1944,12 +1946,13 @@ glngamma(GEN x, long prec)
       if (e > 3)
       {
         x = mkfrac(c,b);
-        if (lgefint(b) >= prec) x = fractor(x,prec + nbits2nlong(e));
+        if (lg2prec(lgefint(b)) >= prec)
+          x = fractor(x, prec + nbits2extraprec(e));
         y = lngamma1(x, prec);
       }
       else
       {
-        x = fractor(x, e > 1? prec+EXTRAPRECWORD: prec);
+        x = fractor(x, e > 1? prec+EXTRAPREC64: prec);
         y = cxgamma(x, 1, prec);
       }
       return gerepileupto(av, y);
@@ -1999,7 +2002,7 @@ dlogE(double s, double t)
   return maxdd(dnorm(rlog,ilog), 1e-6);
 }
 static GEN
-cxpsi(GEN s0, long prec)
+cxpsi(GEN s0, long der, long prec)
 {
   pari_sp av, av2;
   GEN sum, z, a, res, sig, tau, s, unr, s2, sq;
@@ -2008,6 +2011,13 @@ cxpsi(GEN s0, long prec)
   int funeq = 0;
   pari_timer T;
 
+  if (der)
+  {
+    av = avma;
+    res = zetahurwitz(stoi(der + 1), s0, 0, prec2nbits(prec));
+    if(!odd(der)) res = gneg(res);
+    return gerepileupto(av, gmul(mpfact(der), res));
+  }
   if (DEBUGLEVEL>2) timer_start(&T);
   s = trans_fix_arg(&prec,&s0,&sig,&tau,&av,&res);
   if (signe(sig) <= 0) { funeq = 1; s = gsub(gen_1, s); sig = real_i(s); }
@@ -2066,7 +2076,7 @@ psi1series(long n, long v, long prec)
   long i, l = n+3;
   GEN s = cgetg(l, t_SER), z = constzeta(n + 1, prec);
 
-  s[1] = evalsigne(1)|evalvalp(0)|evalvarn(v);
+  s[1] = evalsigne(1)|evalvalser(0)|evalvarn(v);
   for (i = 1; i <= n+1; i++)
   {
     GEN c = gel(z,i); /* zeta(i) */
@@ -2090,7 +2100,7 @@ serpsiz0(GEN z0, long L, long v, long prec)
   GEN A,A1,A2, B,B1,B2, Q;
   long n;
   n = gprecision(z0); if (n) prec = n;
-  z0 = gtofp(z0, prec + EXTRAPRECWORD);
+  z0 = gtofp(z0, prec + EXTRAPREC64);
   /* Start from n = 3; in Luke's notation, A2 := A_{n-2}, A1 := A_{n-1},
    * A := A_n. Same for B */
   av = avma;
@@ -2149,8 +2159,8 @@ Hseries(long m, long L, long v, long prec)
   long i, k, bit, l = L+3, M = m < 0? 1-m: m;
   pari_sp av = avma;
   GEN H = cgetg(l, t_SER);
-  H[1] = evalsigne(1)|evalvarn(v)|evalvalp(0);
-  prec++;
+  H[1] = evalsigne(1)|evalvarn(v)|evalvalser(0);
+  prec += EXTRAPREC64;
   bit = -prec2nbits(prec);
   for(k = 2; k < l; k++) gel(H,k) = gen_1; /* i=1 */
   for (i = 2; i < M; i++)
@@ -2176,7 +2186,7 @@ static GEN
 serpsi(GEN y, long prec)
 {
   GEN Q = NULL, z0, Y = y, Y2;
-  long L = lg(y)-2, v  = varn(y), vy = valp(y);
+  long L = lg(y)-2, v  = varn(y), vy = valser(y);
 
   if (!L) pari_err_DOMAIN("psi", "argument", "=", gen_0,y);
   if (vy < 0) pari_err_DOMAIN("psi", "series valuation", "<", gen_0,y);
@@ -2204,7 +2214,7 @@ serpsi(GEN y, long prec)
       else
       {
         Q = scalarser(gen_m1, v, 1);
-        setvalp(Q,-1);
+        setvalser(Q,-1);
       }
     }
   }
@@ -2235,6 +2245,22 @@ psi_n(ulong b)
   if (b <= 1024) return 715;
   return 0.010709 * pow((double)b, 1.631); /* 1.631 ~ log_3(6) */
 }
+
+GEN
+Qp_psi(GEN x, long der)
+{
+  pari_sp av = avma;
+  GEN p = gel(x,2), p1 = subis(p,1), z;
+  long e = valp(x) + precp(x);
+  if (valp(x) < 0) pari_err_DOMAIN("psi","v_p(x)", "<", gen_0, x);
+  if (der < 0) pari_err_DOMAIN("psi","der","<", gen_0, stoi(der));
+  x = cvtop(x, p, e + 1);
+  z = gmul(mpfact(der), Qp_zetahurwitz(cvtop(stoi(der + 1), p, e + sdivsi(e,p1)), x, -der));
+  if (!odd(der)) z = gneg(z);
+  if (!der) z = gadd(mkfrac(p1,p), z);
+  return gerepileupto(av, z);
+}
+
 GEN
 gpsi(GEN x, long prec)
 {
@@ -2248,10 +2274,54 @@ gpsi(GEN x, long prec)
       if (lgefint(x) > 3 || (n = itou(x)) > psi_n(prec2nbits(prec))) break;
       av = avma; y = mpeuler(prec);
       return gerepileuptoleaf(av, n == 1? negr(y): gsub(harmonic(n-1), y));
-    case t_REAL: case t_COMPLEX: return cxpsi(x,prec);
+    case t_REAL: case t_COMPLEX: return cxpsi(x,0,prec);
+    case t_PADIC: return Qp_psi(x, 0);
     default:
       av = avma; if (!(y = toser_i(x))) break;
       return gerepileupto(av, serpsi(y,prec));
   }
   return trans_eval("psi",gpsi,x,prec);
 }
+
+static GEN
+_gpsi_der(void *E, GEN x, long prec)
+{
+  return gpsi_der(x, (long) E, prec);
+}
+
+GEN
+gpsi_der(GEN x, long der, long prec)
+{
+  pari_sp av;
+  ulong n;
+  GEN y;
+  if (der < 0) pari_err_DOMAIN("gpsi", "der", "<", gen_0, stoi(der));
+  switch(typ(x))
+  {
+    case t_INT:
+      if (signe(x) <= 0) err_psi(x);
+      if (lgefint(x) > 3 || (n = itou(x)) > psi_n(prec2nbits(prec))) break;
+      av = avma;
+      y = der ? szeta(der + 1, prec): mpeuler(prec);
+      if (n > 1)
+      {
+        y = gsub(y, harmonic0(n - 1, stoi(der + 1)));
+        if (!odd(der)) y = gneg(y);
+        y = gmul(mpfact(der), y);
+        return gerepileuptoleaf(av, y);
+      }
+    case t_REAL: case t_COMPLEX: return cxpsi(x, der, prec);
+    case t_PADIC: return Qp_psi(x, der);
+    default:
+      av = avma; if (!(y = toser_i(x))) break;
+      if (!der) y = serpsi(y,prec);
+      else
+      {
+        y = zetahurwitz(stoi(der + 1), x, 0, prec2nbits(prec));
+        if(!odd(der)) y = gneg(y);
+        y = gmul(mpfact(der), y);
+      }
+      return gerepileupto(av, y);
+  }
+  return trans_evalgen("psi",(void*)der,_gpsi_der,x,prec);
+}
