@@ -223,7 +223,7 @@ zetamult_Zagier(GEN avec, long bit, long prec)
   ze= cgetg(s + 1, t_VEC);
   b = cgetg(l + 1, t_VEC);
   for (r = 1; r <= s; r++) gel(ze,r) = cgetr(prec);
-  for (r = 1; r <= l; r++) { gel(b,r) = cgetr(prec); affur(0,gel(b,r)); }
+  for (r = 1; r <= l; r++) gel(b,r) = utor(0, prec);
   affur(1, gel(b,1)); av = avma;
   for (r = 1, h = -1; r <= t; r++)
   {
@@ -301,7 +301,7 @@ zetamult_interpolate2_i(GEN avec, GEN t, long prec)
       gcoeff(a, i+1, j) = gerepileupto(av, gdivgu(S, j));
     }
   }
-  _1 = real_1(prec + EXTRAPRECWORD); av = avma;
+  _1 = real_1(prec + EXTRAPREC64); av = avma;
   ze = cgetg(n+1, t_VEC);
   for (i = 1; i <= n; i++)
   {
@@ -690,10 +690,10 @@ zetamult_i(GEN a, long prec)
   if (r == 1) return szeta(a[1], prec);
   bit = prec2nbits(prec);
   if (bit <= 128)
-    return zetamult_Zagier(a, bit, prec + EXTRAPRECWORD);
+    return zetamult_Zagier(a, bit, prec + EXTRAPREC64);
   k = zv_sum(a);
   if (((double)r) / (k*k) * bit / log((double)10*bit) < 0.5)
-    return zetamult_Zagier(a, bit, prec + EXTRAPRECWORD);
+    return zetamult_Zagier(a, bit, prec + EXTRAPREC64);
   bit += maxss(log2zeta_bound(a), 64);
   return zetamult_Akhilesh(atoe(a), bit, prec);
 }
@@ -724,8 +724,7 @@ zetamult_interpolate(GEN s, GEN t, long prec)
     return zetamult_interpolate2_i(vecsmall_reverse(avec), t, prec);
   v = allstar(avec); l = lg(v); la = lg(avec);
   V = cgetg(la, t_VEC);
-  for (i = 1; i < la; i++)
-  { gel(V,i) = cgetr(prec + EXTRAPRECWORD); affur(0, gel(V,i)); }
+  for (i = 1; i < la; i++) gel(V,i) = utor(0, prec + EXTRAPREC64);
   av2 = avma;
   for (i = 1; i < l; i++, set_avma(av2))
   {
