@@ -5,7 +5,8 @@ This file is part of the PARI/GP package.
 
 PARI/GP is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
-Foundation. It is distributed in the hope that it will be useful, but WITHOUT
+Foundation; either version 2 of the License, or (at your option) any later
+version. It is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
@@ -55,12 +56,12 @@ gcd_plus_minus(GEN x, GEN y, GEN res)
     shift_right(res,t, 2,lt, t[1],m);
   }
   res[1] = evalsigne(1)|evallgefint(lt);
-  avma = av;
+  set_avma(av);
 }
 
 /* uses modified right-shift binary algorithm now --GN 1998Jul23 */
-GEN
-gcdii(GEN a, GEN b)
+static GEN
+gcdii_basecase(GEN a, GEN b)
 {
   long v, w;
   pari_sp av;
@@ -82,10 +83,10 @@ gcdii(GEN a, GEN b)
     return igcduu((ulong)b[2], u);
   }
 
-  /* larger than gcd: "avma=av" gerepile (erasing t) is valid */
+  /* larger than gcd: "set_avma(av)" gerepile (erasing t) is valid */
   av = avma; (void)new_chunk(lgefint(b)); /* HACK */
   t = remii(a,b);
-  if (!signe(t)) { avma = av; return absi(b); }
+  if (!signe(t)) { set_avma(av); return absi(b); }
 
   a = b; b = t;
   v = vali(a); a = shifti(a,-v); setabssign(a);
@@ -93,10 +94,10 @@ gcdii(GEN a, GEN b)
   if (w < v) v = w;
   switch(abscmpii(a,b))
   {
-    case  0: avma = av; a = shifti(a,v); return a;
+    case  0: set_avma(av); a = shifti(a,v); return a;
     case -1: swap(a,b);
   }
-  if (is_pm1(b)) { avma = av; return int2n(v); }
+  if (is_pm1(b)) { set_avma(av); return int2n(v); }
 
   /* we have three consecutive memory locations: a,b,t.
    * All computations are done in place */
@@ -107,17 +108,35 @@ gcdii(GEN a, GEN b)
     /* if a=b mod 4 set t=a-b, otherwise t=a+b, then strip powers of 2 */
     /* so that t <= (a+b)/4 < a/2 */
     gcd_plus_minus(a,b, t);
-    if (is_pm1(t)) { avma = av; return int2n(v); }
+    if (is_pm1(t)) { set_avma(av); return int2n(v); }
     switch(abscmpii(t,b))
     {
       case -1: p1 = a; a = b; b = t; t = p1; break;
       case  1: swap(a,t); break;
-      case  0: avma = av; b = shifti(b,v); return b;
+      case  0: set_avma(av); b = shifti(b,v); return b;
     }
   }
   {
     long r[] = {evaltyp(t_INT)|_evallg(3), evalsigne(1)|evallgefint(3), 0};
     r[2] = (long) gcduodd((ulong)b[2], (ulong)a[2]);
-    avma = av; return shifti(r,v);
+    set_avma(av); return shifti(r,v);
   }
 }
+
+GEN
+gcdii(GEN x, GEN y)
+{
+  pari_sp av=avma;
+  while (lgefint(y)-2 >= GCD_HALFGCD_LIMIT)
+  {
+    GEN M = HGCD0(x,y);
+    x = gel(M,2); y = gel(M,3);
+    if (signe(y) && expi(y)<magic_threshold(x))
+    {
+      swap(x,y);
+      y = remii(y,x);
+    }
+    if (gc_needed(av, 1)) gerepileall(av,2,&x,&y);
+  }
+  return gerepileuptoint(av, gcdii_basecase(x,y));
+}
