@@ -46,30 +46,46 @@ get_sep(const char *t)
 
 /* "atoul" + optional [kmg] suffix */
 static ulong
-my_int(char *s)
+my_int(char *s, int size)
 {
   ulong n = 0;
   char *p = s;
 
-  while (isdigit((int)*p)) {
+  while (isdigit((unsigned char)*p)) {
     ulong m;
     if (n > (~0UL / 10)) pari_err(e_SYNTAX,"integer too large",s,s);
     n *= 10; m = n;
     n += *p++ - '0';
     if (n < m) pari_err(e_SYNTAX,"integer too large",s,s);
   }
-  if (n)
+  if (n && *p)
   {
+    long i = 0;
+    ulong pow[] = {0, 1000UL, 1000000UL, 1000000000UL
+#ifdef LONG_IS_64BIT
+     , 1000000000000UL
+#endif
+    };
     switch(*p)
     {
-      case 'k': case 'K': n = umuluu_or_0(n,1000UL);       p++; break;
-      case 'm': case 'M': n = umuluu_or_0(n,1000000UL);    p++; break;
-      case 'g': case 'G': n = umuluu_or_0(n,1000000000UL); p++; break;
+      case 'k': case 'K': p++; i = 1; break;
+      case 'm': case 'M': p++; i = 2; break;
+      case 'g': case 'G': p++; i = 3; break;
 #ifdef LONG_IS_64BIT
-      case 't': case 'T': n = umuluu_or_0(n,1000000000000UL); p++; break;
+      case 't': case 'T': p++; i = 4; break;
 #endif
     }
-    if (!n) pari_err(e_SYNTAX,"integer too large",s,s);
+    if (i)
+    {
+      if (*p == 'B' && p[-1] != 'm' && p[-1] != 'g' && size)
+      {
+        p++;
+        n = umuluu_or_0(n, 1UL << (10*i));
+      }
+      else
+        n = umuluu_or_0(n, pow[i]);
+      if (!n) pari_err(e_SYNTAX,"integer too large",s,s);
+    }
   }
   if (*p) pari_err(e_SYNTAX,"I was expecting an integer here", s, s);
   return n;
@@ -84,20 +100,20 @@ get_int(const char *s, long dflt)
   int minus = 0;
 
   if (*p == '-') { minus = 1; p++; }
-  if (!isdigit((int)*p)) return gc_long(av, dflt);
+  if (!isdigit((unsigned char)*p)) return gc_long(av, dflt);
 
-  n = (long)my_int(p);
+  n = (long)my_int(p, 0);
   if (n < 0) pari_err(e_SYNTAX,"integer too large",s,s);
   return gc_long(av, minus? -n: n);
 }
 
-ulong
-get_uint(const char *s)
+static ulong
+get_uint(const char *s, int size)
 {
   pari_sp av = avma;
   char *p = get_sep(s);
   if (*p == '-') pari_err(e_SYNTAX,"arguments must be positive integers",s,s);
-  return gc_ulong(av, my_int(p));
+  return gc_ulong(av, my_int(p, size));
 }
 
 #if defined(__EMX__) || defined(_WIN32) || defined(__CYGWIN32__)
@@ -198,8 +214,7 @@ long
 setrealprecision(long n, long *prec)
 {
   GP_DATA->fmt->sigd = n;
-  *prec = ndec2prec(n);
-  precreal = prec2nbits(*prec);
+  *prec = precreal = ndec2prec(n);
   return n;
 }
 
@@ -231,11 +246,12 @@ sd_toggle(const char *v, long flag, const char *s, int *ptn)
 }
 
 static void
-sd_ulong_init(const char *v, const char *s, ulong *ptn, ulong Min, ulong Max)
+sd_ulong_init(const char *v, const char *s, ulong *ptn, ulong Min, ulong Max,
+              int size)
 {
   if (v)
   {
-    ulong n = get_uint(v);
+    ulong n = get_uint(v, size);
     if (n > Max || n < Min)
     {
       char *buf = stack_malloc(strlen(s) + 2 * 20 + 40);
@@ -246,30 +262,44 @@ sd_ulong_init(const char *v, const char *s, ulong *ptn, ulong Min, ulong Max)
     *ptn = n;
   }
 }
-
+static GEN
+sd_res(const char *v, long flag, const char *s, ulong n, ulong oldn,
+       const char **msg)
+{
+  switch(flag)
+  {
+    case d_RETURN:
+      return utoi(n);
+    case d_ACKNOWLEDGE:
+      if (!v || n != oldn) {
+        if (!msg)         /* no specific message */
+          pari_printf("   %s = %lu\n", s, n);
+        else if (!msg[1]) /* single message, always printed */
+          pari_printf("   %s = %lu %s\n", s, n, msg[0]);
+        else              /* print (new)-n-th message */
+          pari_printf("   %s = %lu %s\n", s, n, msg[n]);
+      }
+      break;
+  }
+  return gnil;
+}
 /* msg is NULL or NULL-terminated array with msg[0] != NULL. */
 GEN
 sd_ulong(const char *v, long flag, const char *s, ulong *ptn, ulong Min, ulong Max,
          const char **msg)
 {
   ulong n = *ptn;
-  sd_ulong_init(v, s, ptn, Min, Max);
-  switch(flag)
-  {
-    case d_RETURN:
-      return utoi(*ptn);
-    case d_ACKNOWLEDGE:
-      if (!v || *ptn != n) {
-        if (!msg)         /* no specific message */
-          pari_printf("   %s = %lu\n", s, *ptn);
-        else if (!msg[1]) /* single message, always printed */
-          pari_printf("   %s = %lu %s\n", s, *ptn, msg[0]);
-        else              /* print (new)-n-th message */
-          pari_printf("   %s = %lu %s\n", s, *ptn, msg[*ptn]);
-      }
-      break;
-  }
-  return gnil;
+  sd_ulong_init(v, s, ptn, Min, Max, 0);
+  return sd_res(v, flag, s, *ptn, n, msg);
+}
+
+static GEN
+sd_size(const char *v, long flag, const char *s, ulong *ptn, ulong Min, ulong Max,
+         const char **msg)
+{
+  ulong n = *ptn;
+  sd_ulong_init(v, s, ptn, Min, Max, 1);
+  return sd_res(v, flag, s, *ptn, n, msg);
 }
 
 static void
@@ -287,12 +317,12 @@ parse_intarray(const char *v, const char *s)
   long i, l;
   GEN w;
   if (*t != '[') err_intarray(t, t, s);
-  if (t[1] == ']') return gc_const(av, cgetalloc(t_VECSMALL, 1));
+  if (t[1] == ']') return gc_const(av, cgetalloc(1, t_VECSMALL));
   for (p = t+1, l=2; *p; p++)
     if (*p == ',') l++;
     else if (*p < '0' || *p > '9') break;
   if (*p != ']') err_intarray(t, p, s);
-  w = cgetalloc(t_VECSMALL, l);
+  w = cgetalloc(l, t_VECSMALL);
   for (p = t+1, i=0; *p; p++)
   {
     long n = 0;
@@ -321,7 +351,7 @@ sd_realprecision(const char *v, long flag)
   {
     ulong newnb = fmt->sigd;
     long prec;
-    sd_ulong_init(v, "realprecision", &newnb, 1, prec2ndec(LGBITS));
+    sd_ulong_init(v, "realprecision", &newnb, 1, prec2ndec(LGBITS), 0);
     if (fmt->sigd == (long)newnb) return gnil;
     if (fmt->sigd >= 0) fmt->sigd = newnb;
     prec = ndec2nbits(newnb);
@@ -350,7 +380,7 @@ sd_realbitprecision(const char *v, long flag)
   {
     ulong newnb = precreal;
     long n;
-    sd_ulong_init(v, "realbitprecision", &newnb, 1, prec2nbits(LGBITS));
+    sd_ulong_init(v, "realbitprecision", &newnb, 1, LGBITS, 0);
     if ((long)newnb == precreal) return gnil;
     n = nbits2ndec(newnb);
     if (!n) n = 1;
@@ -383,7 +413,7 @@ gp_get_color(char **st)
   char *s, *v = *st;
   int trans;
   long c;
-  if (isdigit((int)*v))
+  if (isdigit((unsigned)*v))
     { c = atol(v); trans = 1; } /* color on transparent background */
   else
   {
@@ -475,13 +505,13 @@ sd_format(const char *v, long flag)
       pari_err(e_SYNTAX,"default: inexistent format",v,v);
     fmt->format = c; v++;
 
-    if (isdigit((int)*v))
-      { while (isdigit((int)*v)) v++; } /* FIXME: skip obsolete field width */
+    if (isdigit((unsigned char)*v))
+      { while (isdigit((unsigned char)*v)) v++; } /* FIXME: skip obsolete field width */
     if (*v++ == '.')
     {
       if (*v == '-') fmt->sigd = -1;
       else
-        if (isdigit((int)*v)) fmt->sigd=atol(v);
+        if (isdigit((unsigned char)*v)) fmt->sigd=atol(v);
     }
   }
   if (flag == d_RETURN)
@@ -546,8 +576,7 @@ sd_histsize(const char *s, long flag)
 {
   gp_hist *H = GP_DATA->hist;
   ulong n = H->size;
-  GEN r = sd_ulong(s,flag,"histsize",&n, 1,
-                     (LONG_MAX / sizeof(long)) - 1,NULL);
+  GEN r = sd_ulong(s,flag,"histsize",&n, 1, (LONG_MAX / sizeof(long)) - 1,NULL);
   if (n != H->size)
   {
     const ulong total = H->total;
@@ -672,7 +701,7 @@ GEN
 sd_parisizemax(const char *v, long flag)
 {
   ulong size = pari_mainstack->vsize, n = size;
-  GEN r = sd_ulong(v,flag,"parisizemax",&n, 0,LONG_MAX,NULL);
+  GEN r = sd_size(v,flag,"parisizemax",&n, 0,LONG_MAX,NULL);
   if (n != size) {
     if (flag == d_INITRC)
       paristack_setsize(pari_mainstack->rsize, n);
@@ -686,7 +715,7 @@ GEN
 sd_parisize(const char *v, long flag)
 {
   ulong rsize = pari_mainstack->rsize, n = rsize;
-  GEN r = sd_ulong(v,flag,"parisize",&n, 10000,LONG_MAX,NULL);
+  GEN r = sd_size(v,flag,"parisize",&n, 10000,LONG_MAX,NULL);
   if (n != rsize) {
     if (flag == d_INITRC)
       paristack_setsize(n, pari_mainstack->vsize);
@@ -700,7 +729,7 @@ GEN
 sd_threadsizemax(const char *v, long flag)
 {
   ulong size = GP_DATA->threadsizemax, n = size;
-  GEN r = sd_ulong(v,flag,"threadsizemax",&n, 0,LONG_MAX,NULL);
+  GEN r = sd_size(v,flag,"threadsizemax",&n, 0,LONG_MAX,NULL);
   if (n != size)
     GP_DATA->threadsizemax = n;
   return r;
@@ -710,7 +739,7 @@ GEN
 sd_threadsize(const char *v, long flag)
 {
   ulong size = GP_DATA->threadsize, n = size;
-  GEN r = sd_ulong(v,flag,"threadsize",&n, 0,LONG_MAX,NULL);
+  GEN r = sd_size(v,flag,"threadsize",&n, 0,LONG_MAX,NULL);
   if (n != size)
     GP_DATA->threadsize = n;
   return r;
@@ -721,6 +750,19 @@ sd_primelimit(const char *v, long flag)
 { return sd_ulong(v,flag,"primelimit",&(GP_DATA->primelimit),
                   0,2*(ulong)(LONG_MAX-1024) + 1,NULL); }
 
+GEN
+sd_factorlimit(const char *v, long flag)
+{
+  GEN z = sd_ulong(v,flag,"factorlimit",&(GP_DATA->factorlimit),
+                   0,2*(ulong)(LONG_MAX-1024) + 1,NULL);
+  if (v && flag != d_INITRC)
+      mt_broadcast(snm_closure(is_entry("default"),
+                   mkvec2(strtoGENstr("factorlimit"), strtoGENstr(v))));
+  if (GP_DATA->primelimit < GP_DATA->factorlimit)
+    GP_DATA->primelimit = GP_DATA->factorlimit;
+  return z;
+}
+
 GEN
 sd_simplify(const char *v, long flag)
 { return sd_toggle(v,flag,"simplify", &(GP_DATA->simplify)); }
@@ -789,8 +831,9 @@ sd_datadir(const char *v, long flag)
   const char *str;
   if (v)
   {
-    mt_broadcast(snm_closure(is_entry("default"),
-                 mkvec2(strtoGENstr("datadir"), strtoGENstr(v))));
+    if (flag != d_INITRC)
+      mt_broadcast(snm_closure(is_entry("default"),
+                   mkvec2(strtoGENstr("datadir"), strtoGENstr(v))));
     if (pari_datadir) pari_free(pari_datadir);
     pari_datadir = path_expand(v);
   }
@@ -806,7 +849,8 @@ sd_PATH(const char *v, long flag, const char* s, gp_path *p)
 {
   if (v)
   {
-    mt_broadcast(snm_closure(is_entry("default"),
+    if (flag != d_INITRC)
+      mt_broadcast(snm_closure(is_entry("default"),
                  mkvec2(strtoGENstr(s), strtoGENstr(v))));
     pari_free((void*)p->PATH);
     p->PATH = pari_strdup(v);
@@ -928,11 +972,7 @@ init_path(gp_path *path, const char *v)
 static void
 init_fmt(gp_data *D)
 {
-#ifdef LONG_IS_64BIT
   static pariout_t DFLT_OUTPUT = { 'g', 38, 1, f_PRETTYMAT, 0 };
-#else
-  static pariout_t DFLT_OUTPUT = { 'g', 28, 1, f_PRETTYMAT, 0 };
-#endif
   D->fmt = &DFLT_OUTPUT;
 }
 
@@ -964,17 +1004,17 @@ init_graphs(gp_data *D)
     "white","black","blue","violetred","red","green","grey","gainsboro"
   };
   const long N = 8;
-  GEN c = cgetalloc(t_VECSMALL, 3), s;
+  GEN c = cgetalloc(3, t_VECSMALL), s;
   long i;
   c[1] = 4;
   c[2] = 5;
   D->graphcolors = c;
   c = (GEN)pari_malloc((N+1 + 4*N)*sizeof(long));
-  c[0] = evaltyp(t_VEC)|evallg(N+1);
+  c[0] = evaltyp(t_VEC)|_evallg(N+1);
   for (i = 1, s = c+N+1; i <= N; i++, s += 4)
   {
     GEN lp = s;
-    lp[0] = evaltyp(t_STR)|evallg(4);
+    lp[0] = evaltyp(t_STR)|_evallg(4);
     strcpy(GSTR(lp), cols[i]);
     gel(c,i) = lp;
   }
@@ -991,7 +1031,7 @@ default_gp_data(void)
   static pari_timer __T, __Tw;
 
   D->flags       = 0;
-  D->primelimit  = 500000;
+  D->factorlimit = D->primelimit = 1UL << 20;
 
   /* GP-specific */
   D->breakloop   = 1;
@@ -1018,7 +1058,7 @@ default_gp_data(void)
   init_path(D->sopath, "");
   init_pp(D);
   init_graphs(D);
-  D->plothsizes = cgetalloc(t_VECSMALL, 1);
+  D->plothsizes = cgetalloc(1, t_VECSMALL);
   D->prompt_comment = (char*)"comment> ";
   D->prompt = pari_strdup("? ");
   D->prompt_cont = pari_strdup("");
