@@ -26,9 +26,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 
 #include "pari.h"
 #include "paripriv.h"
-#ifdef __EMSCRIPTEN__
-#include "../systems/emscripten/emscripten.h"
-#endif
 
 /********************************************************************/
 /**                                                                **/
@@ -39,13 +36,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 void
 pari_skip_space(char **s) {
   char *t = *s;
-  while (isspace((int)*t)) t++;
+  while (isspace((unsigned char)*t)) t++;
   *s = t;
 }
 void
 pari_skip_alpha(char **s) {
   char *t = *s;
-  while (isalpha((int)*t)) t++;
+  while (isalpha((unsigned char)*t)) t++;
   *s = t;
 }
 
@@ -215,11 +212,20 @@ pari_center(const char *s)
 static void
 community(void)
 {
+  const char *pari_docdir;
+#if defined(_WIN32)
+  /* for some reason, the documentation on windows is not in datadir */
+  if (paricfg_datadir[0]=='@' && paricfg_datadir[1]==0)
+    pari_docdir = win32_basedir();
+  else
+#endif
+    pari_docdir = pari_datadir;
+
   print_text("The PARI/GP distribution includes a reference manual, a \
 tutorial, a reference card and quite a few examples. They have been installed \
 in the directory ");
   pari_puts("  ");
-  pari_puts(pari_datadir);
+  pari_puts(pari_docdir);
   pari_puts("\nYou can also download them from http://pari.math.u-bordeaux.fr/.\
 \n\nThree mailing lists are devoted to PARI:\n\
   - pari-announce (moderated) to announce major version changes.\n\
@@ -253,10 +259,10 @@ gentypes(void)
   t_VEC    : row vector        [ code ] [  x_1  ] ... [  x_k  ]\n\
   t_COL    : column vector     [ code ] [  x_1  ] ... [  x_k  ]\n\
   t_MAT    : matrix            [ code ] [ col_1 ] ... [ col_k ]\n\
-  t_LIST   : list              [ cod1 ] [ cod2 ][ vec ]\n\
+  t_LIST   : list              [ cod1 ] [ cod2 ] [ vec ]\n\
   t_STR    : string            [ code ] [ man_1 ] ... [ man_k ]\n\
   t_VECSMALL: vec. small ints  [ code ] [ x_1 ] ... [ x_k ]\n\
-  t_CLOSURE: functions [ code ] [ arity ] [ code ] [ operand ] [ data ] [ text ]\n\
+  t_CLOSURE: functions         [ code ] [ arity ] [ proto ] [ operand ] ... \n\
   t_ERROR  : error context     [ code ] [ errnum ] [ dat_1 ] ... [ dat_k ]\n\
   t_INFINITY: a*infinity       [ code ] [ a ]\n\
 \n");
@@ -329,6 +335,7 @@ slash_commands(void)
 \\t      : print the list of PARI types\n\
 \\u      : print the list of user-defined functions\n\
 \\um     : print the list of user-defined member functions\n\
+\\uv     : print the list of user-defined variables, excluding closures\n\
 \\v      : print current version of GP\n\
 \\w {nf} : write to a file\n\
 \\x {n}  : print complete inner structure of result\n\
@@ -346,8 +353,8 @@ a1-a6, b2-b8, c4-c6 : coeff. of the curve.         ell\n\
 area : area                                        ell\n\
 bid  : big ideal                     bid,                     bnr\n\
 bnf  : big number field                                   bnf,bnr\n\
-clgp : class group                   bid,                 bnf,bnr\n\
-cyc  : cyclic decomposition (SNF)    bid,     clgp,ell,   bnf,bnr\n\
+clgp : class group              quad,bid,                 bnf,bnr\n\
+cyc  : cyclic decomposition     quad,bid,     clgp,ell,   bnf,bnr\n\
 diff, codiff: different and codifferent                nf,bnf,bnr\n\
 disc : discriminant                                ell,nf,bnf,bnr,rnf\n\
 e, f : inertia/residue  degree           prid\n\
@@ -360,13 +367,14 @@ j    : j-invariant                                 ell\n");
   pari_puts("\
 mod  : modulus                       bid,                     bnr,    gal\n\
 nf   : number field                                    nf,bnf,bnr,rnf\n\
-no   : number of elements            bid,     clgp,ell,   bnf,bnr\n\
+no   : number of elements       quad,bid,     clgp,ell,   bnf,bnr\n\
+normfu:                         quad\n\
 omega, eta: [w1,w2] and [eta1, eta2]               ell\n\
 orders: relative orders of generators                                 gal\n\
 p    : rational prime                    prid,     ell,nf,bnf,bnr,rnf,gal\n\
 pol  : defining polynomial                             nf,bnf,bnr,    gal\n\
 polabs: defining polynomial over Q                                rnf\n\
-reg  : regulator                                          bnf\n\
+reg  : regulator                quad,                     bnf\n\
 roots: roots                                       ell,nf,bnf,bnr,    gal\n\
 sign,r1,r2 : signature                                 nf,bnf,bnr\n\
 t2   : t2 matrix                                       nf,bnf,bnr\n\
@@ -422,7 +430,7 @@ nl_read(char *s) { size_t l = strlen(s); return s[l-1] == '\n'; }
 
 /* query external help program for s. num < 0 [keyword] or chapter number */
 static void
-external_help(const char *s, int num)
+external_help(const char *s, long num)
 {
   long nbli = term_height()-3, li = 0;
   char buf[256], *str;
@@ -430,9 +438,7 @@ external_help(const char *s, int num)
   char *t, *help = GP_DATA->help;
   pariFILE *z;
   FILE *f;
-#ifdef __EMSCRIPTEN__
-  pari_emscripten_help(s);
-#endif
+  if (cb_pari_long_help) { cb_pari_long_help(s, num); return; }
 
   if (!has_ext_help()) pari_err(e_MISC,"no external help program");
   t = filter_quotes(s);
@@ -498,6 +504,9 @@ gphelp_keyword_list(void)
   "sums",
   "products",
   "integrals",
+  "gchar",
+  "grossencharacter",
+  "Grossencharacter",
   NULL};
   return L;
 }
@@ -508,7 +517,7 @@ ok_external_help(char **s)
   const char **L;
   long n;
   if (!**s) return 1;
-  if (!isalpha((int)**s)) return 3; /* operator or section number */
+  if (!isalpha((unsigned char)**s)) return 3; /* operator or section number */
   if (!strncmp(*s,"t_",2)) { *s += 2; return 2; } /* type name */
 
   L = gphelp_keyword_list();
@@ -569,12 +578,12 @@ help(const char *s0, int flag)
   entree *ep;
   char *s = get_sep(s0);
 
-  if (isdigit((int)*s)) { digit_help(s,flag); return; }
+  if (isdigit((unsigned char)*s)) { digit_help(s,flag); return; }
   if (flag & h_APROPOS) { external_help(s,-1); return; }
   /* Get meaningful answer on '\ps 5' (e.g. from <F1>) */
-  if (*s == '\\' && isalpha((int)*(s+1)))
+  if (*s == '\\' && isalpha((unsigned char)*(s+1)))
   { char *t = s+1; pari_skip_alpha(&t); *t = '\0'; }
-  if (isalpha((int)*s))
+  if (isalpha((unsigned char)*s))
   {
     char *t = s;
     if (!strncmp(s, "default", 7))
@@ -589,7 +598,7 @@ help(const char *s0, int flag)
     }
     if (!strncmp(s, "refcard-", 8)) t += 8;
     else if (!strncmp(s, "tutorial-", 9)) t += 9;
-    cut_trailing_garbage(t);
+    if (strncmp(s, "se:", 3)) cut_trailing_garbage(t);
   }
 
   if (long_help && (n = ok_external_help(&s))) { external_help(s,n); return; }
@@ -836,8 +845,8 @@ static ulong
 read_uint(char **s)
 {
   long v = atol(*s);
-  if (!isdigit((int)**s)) err_gprc("not an integer", *s, *s);
-  while (isdigit((int)**s)) (*s)++;
+  if (!isdigit((unsigned char)**s)) err_gprc("not an integer", *s, *s);
+  while (isdigit((unsigned char)**s)) (*s)++;
   return v;
 }
 static ulong
@@ -1416,50 +1425,14 @@ prettyp_init(void)
   sd_output("1", d_SILENT);
   return 0;
 }
-
-/* n = history number. if n = 0 no history */
-int
-tex2mail_output(GEN z, long n)
+/* assume prettyp_init() was called */
+static void
+prettyp_GEN(GEN z)
 {
+  FILE *log = pari_logfile, *out = GP_DATA->pp->file->file;
   pariout_t T = *(GP_DATA->fmt); /* copy */
-  FILE *log = pari_logfile, *out;
-
-  if (!prettyp_init()) return 0;
-  out = GP_DATA->pp->file->file;
-  /* Emit first: there may be lines before the prompt */
-  if (n) term_color(c_OUTPUT);
-  pari_flush();
-  T.prettyp = f_TEX;
-  /* history number */
-  if (n)
-  {
-    pari_sp av = avma;
-    const char *c_hist = term_get_color(NULL, c_HIST);
-    const char *c_out = term_get_color(NULL, c_OUTPUT);
-    if (!(GP_DATA->flags & gpd_QUIET))
-    {
-      if (*c_hist || *c_out)
-        fprintf(out, "\\LITERALnoLENGTH{%s}\\%%%ld =\\LITERALnoLENGTH{%s} ",
-                     c_hist, n, c_out);
-      else
-        fprintf(out, "\\%%%ld = ", n);
-    }
-    if (log) {
-      switch (pari_logstyle) {
-      case logstyle_plain:
-        fprintf(log, "%%%ld = ", n);
-        break;
-      case logstyle_color:
-        fprintf(log, "%s%%%ld = %s", c_hist, n, c_out);
-        break;
-      case logstyle_TeX:
-        fprintf(log, "\\PARIout{%ld}", n);
-        break;
-      }
-    }
-    set_avma(av);
-  }
   /* output */
+  T.prettyp = f_TEX;
   fputGEN_pariout(z, &T, out);
   /* flush and restore, output to logfile */
   prettyp_wait(out);
@@ -1474,8 +1447,129 @@ tex2mail_output(GEN z, long n)
     }
     fputc('\n', log); fflush(log);
   }
-  if (n) term_color(c_NONE);
-  pari_flush(); return 1;
+}
+/* assume prettyp_init() was called. */
+static void
+prettyp_output(long n)
+{
+  FILE *log = pari_logfile, *out = GP_DATA->pp->file->file;
+  pari_sp av = avma;
+  const char *c_hist = term_get_color(NULL, c_HIST);
+  const char *c_out = term_get_color(NULL, c_OUTPUT);
+  GEN z = pari_get_hist(n);
+  /* Emit first: there may be lines before the prompt */
+  term_color(c_OUTPUT); pari_flush();
+  /* history number */
+  if (!(GP_DATA->flags & gpd_QUIET))
+  {
+    if (*c_hist || *c_out)
+      fprintf(out, "\\LITERALnoLENGTH{%s}\\%%%ld =\\LITERALnoLENGTH{%s} ",
+                   c_hist, n, c_out);
+    else
+      fprintf(out, "\\%%%ld = ", n);
+  }
+  if (log) switch (pari_logstyle)
+  {
+    case logstyle_plain:
+      fprintf(log, "%%%ld = ", n);
+      break;
+    case logstyle_color:
+      fprintf(log, "%s%%%ld = %s", c_hist, n, c_out);
+      break;
+    case logstyle_TeX:
+      fprintf(log, "\\PARIout{%ld}", n);
+      break;
+  }
+  set_avma(av); prettyp_GEN(z);
+  term_color(c_NONE); pari_flush();
+}
+
+/*******************************************************************/
+/**                                                               **/
+/**                   FORMAT GP OUTPUT                            **/
+/**                                                               **/
+/*******************************************************************/
+
+#define COLOR_LEN 16
+
+static void
+str_lim_lines(pari_str *S, char *s, long n, long max_lin)
+{
+  long lin, col, width;
+  char COL[COLOR_LEN];
+  char c;
+  if (!*s) return;
+  width = term_width();
+  lin = 1;
+  col = n;
+
+  if (lin > max_lin) return;
+  while ( (c = *s++) )
+  {
+    if (lin >= max_lin)
+      if (c == '\n' || col >= width-5)
+      {
+        pari_sp av = avma;
+        str_puts(S, term_get_color(COL, c_ERR)); set_avma(av);
+        str_puts(S,"[+++]"); return;
+      }
+    if (c == '\n')         { col = -1; lin++; }
+    else if (col == width) { col =  0; lin++; }
+    pari_set_last_newline(c=='\n');
+    col++; str_putc(S, c);
+  }
+}
+void
+str_display_hist(pari_str *S, long n)
+{
+  long l = 0;
+  char col[COLOR_LEN];
+  char *s;
+  /* history number */
+  if (n)
+  {
+    char buf[64];
+    if (!(GP_DATA->flags & gpd_QUIET))
+    {
+      str_puts(S, term_get_color(col, c_HIST));
+      sprintf(buf, "%%%ld = ", n);
+      str_puts(S, buf);
+      l = strlen(buf);
+    }
+  }
+  /* output */
+  str_puts(S, term_get_color(col, c_OUTPUT));
+  s = GENtostr(pari_get_hist(n));
+  if (GP_DATA->lim_lines)
+    str_lim_lines(S, s, l, GP_DATA->lim_lines);
+  else
+    str_puts(S, s);
+  pari_free(s);
+  str_puts(S,term_get_color(col, c_NONE));
+}
+
+static void
+gp_classic_output(long n)
+{
+  pari_sp av = avma;
+  pari_str S;
+  str_init(&S, 1);
+  str_display_hist(&S, n);
+  str_putc(&S, 0);
+  pari_puts(S.string);
+  pari_putc('\n'); pari_flush();
+  set_avma(av);
+}
+
+void
+gp_display_hist(long n)
+{
+  if (cb_pari_display_hist)
+    cb_pari_display_hist(n);
+  else if (GP_DATA->fmt->prettyp == f_PRETTY && prettyp_init())
+    prettyp_output(n);
+  else
+    gp_classic_output(n);
 }
 
 /*******************************************************************/
@@ -1703,17 +1797,19 @@ pari_print_version(void)
   pari_sp av = avma;
   char *buf, *ver = what_cc();
   const char *kver = pari_kernel_version();
-  const char *date = paricfg_compiledate;
+  const char *date = paricfg_compiledate, *mt = paricfg_mt_engine;
+  ulong t = pari_mt_nbthreads;
 
   pari_center(paricfg_version);
   buf = stack_malloc(strlen(paricfg_buildinfo) + 2 + strlen(kver));
   (void)sprintf(buf, paricfg_buildinfo, kver);
   pari_center(buf);
-  buf = stack_malloc(strlen(date) + 32 + (ver? strlen(ver): 0));
+  buf = stack_malloc(128 + strlen(date) + (ver? strlen(ver): 0));
   if (ver) (void)sprintf(buf, "compiled: %s, %s", date, ver);
   else     (void)sprintf(buf, "compiled: %s", date);
   pari_center(buf);
-  sprintf(buf, "threading engine: %s",paricfg_mt_engine);
+  if (t > 1) sprintf(buf, "threading engine: %s, nbthreads = %lu",mt,t);
+  else       sprintf(buf, "threading engine: %s",mt);
   pari_center(buf);
   ver = what_readline();
   buf = stack_malloc(strlen(ver) + 64);
@@ -1728,33 +1824,26 @@ cmp_epname(void *E, GEN e, GEN f)
   (void)E;
   return strcmp(((entree*)e)->name, ((entree*)f)->name);
 }
+/* if fun is set print only closures, else only non-closures
+ * if member is set print only member functions, else only non-members */
 static void
-print_all_user_fun(int member)
+print_all_user_obj(int fun, int member)
 {
   pari_sp av = avma;
-  long iL = 0, lL = 1024;
+  long i, iL = 0, lL = 1024;
   GEN L = cgetg(lL+1, t_VECSMALL);
   entree *ep;
-  int i;
   for (i = 0; i < functions_tblsz; i++)
     for (ep = functions_hash[i]; ep; ep = ep->next)
-    {
-      const char *f;
-      int is_member;
-      if (EpVALENCE(ep) != EpVAR || typ((GEN)ep->value)!=t_CLOSURE) continue;
-      f = ep->name;
-      is_member = (f[0] == '_' && f[1] == '.');
-      if (member != is_member) continue;
-
-      if (iL >= lL)
+      if (EpVALENCE(ep) == EpVAR && fun == (typ((GEN)ep->value) == t_CLOSURE))
       {
-        GEN oL = L;
-        long j;
-        lL *= 2; L = cgetg(lL+1, t_VECSMALL);
-        for (j = 1; j <= iL; j++) gel(L,j) = gel(oL,j);
+        const char *f = ep->name;
+        if (member == (f[0] == '_' && f[1] == '.'))
+        {
+          if (iL >= lL) { lL *= 2; L = vecsmall_lengthen(L, lL); }
+          L[++iL] = (long)ep;
+        }
       }
-      L[++iL] = (long)ep;
-    }
   if (iL)
   {
     setlg(L, iL+1);
@@ -1800,13 +1889,17 @@ escape(const char *tch, int ismain)
       else
       {
         d = atol(s); if (*s == '-') s++;
-        while (isdigit((int)*s)) s++;
+        while (isdigit((unsigned char)*s)) s++;
       }
       x = pari_get_hist(d);
       switch (c)
       {
         case 'B': /* prettyprinter */
-          if (tex2mail_output(x,0)) break;
+          if (prettyp_init())
+          {
+            pari_flush(); prettyp_GEN(x);
+            pari_flush(); break;
+          }
         case 'b': /* fall through */
         case 'm': matbrute(x, GP_DATA->fmt->format, -1); break;
         case 'a': brute(x, GP_DATA->fmt->format, -1); break;
@@ -1825,22 +1918,22 @@ escape(const char *tch, int ismain)
       if (!*s) s = (GP_DATA->echo)? "0": "1";
       (void)sd_echo(s,d_ACKNOWLEDGE); break;
     case 'g':
-        if (isdigit(*s))
+        if (isdigit((unsigned char)*s))
         {
           const char *t = s + 1;
-          if (isdigit(*t)) t++; /* atol(s) < 99 */
+          if (isdigit((unsigned char)*t)) t++; /* atol(s) < 99 */
           t = get_name(t);
           if (*t) { d = atol(s); ack_setdebug(t, d); break; }
         }
-        else if (*s == '"' || isalpha(*s))
+        else if (*s == '"' || isalpha((unsigned char)*s))
         {
           char *t = get_name(s);
-          if (t[1] && !isdigit(t[1]))
+          if (t[1] && !isdigit((unsigned char)t[1]))
           {
             char *T = t + strlen(t) - 1;
-            if (isdigit(*T))
+            if (isdigit((unsigned char)*T))
             {
-              if (isdigit(T[-1])) T--; /* < 99 */
+              if (isdigit((unsigned char)T[-1])) T--; /* < 99 */
               d = atol(T); *T = 0;
               ack_setdebug(get_name(t), d); /* get_name in case of ".." */
             }
@@ -1906,9 +1999,21 @@ escape(const char *tch, int ismain)
     case 's': dbg_pari_heap(); break;
     case 't': gentypes(); break;
     case 'u':
-      print_all_user_fun((*s == 'm')? 1: 0);
-      break;
-    case 'v': pari_print_version(); break;
+      switch(*s)
+      {
+        case 'v':
+          if (*++s) break;
+          print_all_user_obj(0, 0); return;
+        case 'm':
+          if (*++s) break;
+          print_all_user_obj(1, 1); return;
+        case '\0':
+          print_all_user_obj(1, 0); return;
+      }
+      pari_err(e_SYNTAX,"unexpected character", s,tch-1); break;
+    case 'v':
+      if (*s) pari_err(e_SYNTAX,"unexpected character", s,tch-1);
+      pari_print_version(); break;
     case 'y':
       s = get_sep(s);
       if (!*s) s = (GP_DATA->simplify)? "0": "1";
@@ -1953,3 +2058,12 @@ gp_meta(const char *buf, int ismain)
   }
   return 1;
 }
+
+void
+pari_breakpoint(void)
+{
+  if (!pari_last_was_newline()) pari_putc('\n');
+  closure_err(0);
+  if (cb_pari_break_loop && cb_pari_break_loop(-1)) return;
+  cb_pari_err_recover(e_MISC);
+}
