@@ -183,7 +183,7 @@ rombint(void *E, GEN (*eval)(void*, GEN), GEN a, GEN b, long bit)
 }
 
 GEN
-intnumromb_bitprec(void *E, GEN (*f)(void *, GEN), GEN a,GEN b, long fl, long B)
+intnumromb(void *E, GEN (*f)(void *, GEN), GEN a,GEN b, long fl, long B)
 {
   pari_sp av = avma;
   GEN z;
@@ -198,11 +198,8 @@ intnumromb_bitprec(void *E, GEN (*f)(void *, GEN), GEN a,GEN b, long fl, long B)
   return gerepileupto(av, z);
 }
 GEN
-intnumromb(void *E, GEN (*f)(void *, GEN), GEN a, GEN b, long flag, long prec)
-{ return intnumromb_bitprec(E,f,a,b,flag,prec2nbits(prec));}
-GEN
-intnumromb0_bitprec(GEN a, GEN b, GEN code, long flag, long bit)
-{ EXPR_WRAP(code, intnumromb_bitprec(EXPR_ARG, a, b, flag, bit)); }
+intnumromb0(GEN a, GEN b, GEN code, long flag, long bit)
+{ EXPR_WRAP(code, intnumromb(EXPR_ARG, a, b, flag, bit)); }
 
 /********************************************************************/
 /**             NUMERICAL INTEGRATION (Gauss-Legendre)             **/
@@ -255,29 +252,33 @@ GEN
 intnumgaussinit(long N, long prec)
 {
   pari_sp av;
-  long N2, j, k, l, bit;
+  long N2, j, k, l, lV;
   GEN res, V, W, F, S;
   double d;
 
   prec += EXTRAPREC64;
-  bit = prec2nbits(prec);
-  if (N <= 0) { N = bit >> 2; if (odd(N)) N++; }
+  if (N <= 0) { N = prec >> 2; if (odd(N)) N++; }
   if (N == 1) retmkvec2(mkvec(gen_0), mkvec(gen_2));
+  l = prec2lg(prec);
   res = cgetg(3, t_VEC);
   if (N == 2)
   {
-    GEN z = cgetr(prec);
+    GEN z = cgetg(l, t_REAL);
     gel(res,1) = mkvec(z);
     gel(res,2) = mkvec(gen_1);
     av = avma; affrr(divru(sqrtr(utor(3,prec)), 3), z);
     return gc_const(av, res);
   }
-  N2 = N >> 1; l = (N+3)>> 1;
-  gel(res,1) = V = cgetg(l, t_VEC);
-  gel(res,2) = W = cgetg(l, t_VEC);
-  gel(V,1) = odd(N)? gen_0: cgetr(prec);
-  gel(W,1) = cgetr(prec);
-  for (k = 2; k < l; k++) { gel(V,k) = cgetr(prec); gel(W,k) = cgetr(prec); }
+  N2 = N >> 1; lV = (N+3)>> 1;
+  gel(res,1) = V = cgetg(lV, t_VEC);
+  gel(res,2) = W = cgetg(lV, t_VEC);
+  gel(V,1) = odd(N)? gen_0: cgetg(l, t_REAL);
+  gel(W,1) = cgetg(l, t_REAL);
+  for (k = 2; k < lV; k++)
+  {
+    gel(V,k) = cgetg(l, t_REAL);
+    gel(W,k) = cgetg(l, t_REAL);
+  }
   av = avma; S = vecpowuu(N, 2); F = sqrr(mpfactr(N-1, prec));
   if (!odd(N)) k = 1;
   else
@@ -291,7 +292,7 @@ intnumgaussinit(long N, long prec)
   for (j = 4*N2-1; j >= 3; k++, j -= 4)
   {
     pari_sp av2 = avma;
-    GEN zw = Legendreroot(N, d * cos(M_PI * j / (4*N+2)), S, bit);
+    GEN zw = Legendreroot(N, d * cos(M_PI * j / (4*N+2)), S, prec);
     GEN z = gel(zw,1), w = gel(zw,2);
     affrr(z, gel(V,k));
     w = mulrr(F, divrr(subsr(1, sqrr(z)), sqrr(w)));
@@ -423,8 +424,8 @@ checktab(GEN tab)
 static void
 intinit_start(intdata *D, long m, double TUNE, long prec)
 {
-  long l, n, bitprec = prec2nbits(prec);
-  double d = bitprec*LOG10_2;
+  long l, n;
+  double d = prec*LOG10_2;
   GEN h, nh, pi = mppi(prec);
 
   n = (long)ceil(d*log(d) / TUNE); /* heuristic */
@@ -433,7 +434,7 @@ intinit_start(intdata *D, long m, double TUNE, long prec)
   h = divru(nh, n);
   if (m > 0) { h = gmul2n(h,-m); n <<= m; }
   D->h = h;
-  D->bit = bitprec;
+  D->bit = prec;
   D->l = l = n+1;
   D->tabxp = cgetg(l, t_VEC);
   D->tabwp = cgetg(l, t_VEC);
@@ -465,7 +466,7 @@ static GEN
 inittanhsinh(long m, long prec)
 {
   GEN e, ei, ek, eik, pi = mppi(prec);
-  long k, nt = -1;
+  long k, l, nt = -1;
   intdata D;
 
   intinit_start(&D, m, 1.86, prec);
@@ -473,12 +474,13 @@ inittanhsinh(long m, long prec)
   D.tabw0 = Pi2n(-1,prec);
   e = mpexp(D.h); ek = mulrr(pi, e);
   ei = invr(e); eik = mulrr(pi, ei);
+  l = prec2lg(prec);
   for (k = 1; k < D.l; k++)
   {
     GEN xp, wp, ct, st, z;
     pari_sp av;
-    gel(D.tabxp,k) = cgetr(prec);
-    gel(D.tabwp,k) = cgetr(prec); av = avma;
+    gel(D.tabxp,k) = cgetg(l, t_REAL);
+    gel(D.tabwp,k) = cgetg(l, t_REAL); av = avma;
     ct = divr2_ip(addrr(ek, eik)); /* Pi ch(kh) */
     st = subrr(ek, ct); /* Pi sh(kh) */
     z = invr( addrs(mpexp(st), 1) );
@@ -498,18 +500,19 @@ initsinhsinh(long m, long prec)
 {
   pari_sp av;
   GEN et, ct, st, ex;
-  long k, nt = -1;
+  long k, l, nt = -1;
   intdata D;
 
   intinit_start(&D, m, 0.666, prec);
   D.tabx0 = real_0(prec);
   D.tabw0 = real_1(prec);
   et = ex = mpexp(D.h);
+  l = prec2lg(prec);
   for (k = 1; k < D.l; k++)
   {
     GEN xp, wp, ext, exu;
-    gel(D.tabxp,k) = cgetr(prec);
-    gel(D.tabwp,k) = cgetr(prec); av = avma;
+    gel(D.tabxp,k) = cgetg(l, t_REAL);
+    gel(D.tabwp,k) = cgetg(l, t_REAL); av = avma;
     ct = divr2_ip(addrr(et, invr(et)));
     st = subrr(et, ct);
     ext = mpexp(st);
@@ -529,17 +532,18 @@ initsinh(long m, long prec)
 {
   pari_sp av;
   GEN et, ex, eti, xp, wp;
-  long k, nt = -1;
+  long k, l, nt = -1;
   intdata D;
 
   intinit_start(&D, m, 1.0, prec);
   D.tabx0 = real_0(prec);
   D.tabw0 = real2n(1, prec);
   et = ex = mpexp(D.h);
+  l = prec2lg(prec);
   for (k = 1; k < D.l; k++)
   {
-    gel(D.tabxp,k) = cgetr(prec);
-    gel(D.tabwp,k) = cgetr(prec); av = avma;
+    gel(D.tabxp,k) = cgetg(l, t_REAL);
+    gel(D.tabwp,k) = cgetg(l, t_REAL); av = avma;
     eti = invr(et);
     xp = subrr(et, eti);
     wp = addrr(et, eti);
@@ -584,20 +588,21 @@ initexpexp(long m, long prec)
 {
   pari_sp av;
   GEN et, ex;
-  long k, nt = -1;
+  long k, l, nt = -1;
   intdata D;
 
   intinit_start(&D, m, 1.76, prec);
   D.tabx0 = mpexp(real_m1(prec));
   D.tabw0 = gmul2n(D.tabx0, 1);
   et = ex = mpexp(negr(D.h));
+  l = prec2lg(prec);
   for (k = 1; k < D.l; k++)
   {
     GEN xp, xm, wp, wm, eti, kh;
-    gel(D.tabxp,k) = cgetr(prec);
-    gel(D.tabwp,k) = cgetr(prec);
-    gel(D.tabxm,k) = cgetr(prec);
-    gel(D.tabwm,k) = cgetr(prec); av = avma;
+    gel(D.tabxp,k) = cgetg(l, t_REAL);
+    gel(D.tabwp,k) = cgetg(l, t_REAL);
+    gel(D.tabxm,k) = cgetg(l, t_REAL);
+    gel(D.tabwm,k) = cgetg(l, t_REAL); av = avma;
     eti = invr(et); kh = mulur(k,D.h);
     xp = mpexp(subrr(kh, et));
     xm = mpexp(negr(addrr(kh, eti)));
@@ -618,7 +623,7 @@ initnumsine(long m, long prec)
 {
   pari_sp av;
   GEN invh, et, eti, ex, pi = mppi(prec);
-  long exh, k, nt = -1;
+  long exh, k, l, nt = -1;
   intdata D;
 
   intinit_start(&D, m, 0.666, prec);
@@ -627,13 +632,14 @@ initnumsine(long m, long prec)
   D.tabw0 = gmul2n(D.tabx0,-1);
   exh = expo(invh); /*  expo(1/h) */
   et = ex = mpexp(D.h);
+  l = prec2lg(prec);
   for (k = 1; k < D.l; k++)
   {
     GEN xp,xm, wp,wm, ct,st, extp,extp1,extp2, extm,extm1,extm2, kct, kpi;
-    gel(D.tabxp,k) = cgetr(prec);
-    gel(D.tabwp,k) = cgetr(prec);
-    gel(D.tabxm,k) = cgetr(prec);
-    gel(D.tabwm,k) = cgetr(prec); av = avma;
+    gel(D.tabxp,k) = cgetg(l, t_REAL);
+    gel(D.tabwp,k) = cgetg(l, t_REAL);
+    gel(D.tabxm,k) = cgetg(l, t_REAL);
+    gel(D.tabwm,k) = cgetg(l, t_REAL); av = avma;
     eti = invr(et); /* exp(-kh) */
     ct = divr2_ip(addrr(et, eti)); /* ch(kh) */
     st = divr2_ip(subrr(et, eti)); /* sh(kh) */
@@ -1470,13 +1476,21 @@ QD(GEN M, long lim)
   e = zerovec(lim);
   c = zerovec(lim+1); gel(c, 1) = gel(M, 1);
   q = cgetg(lim+1, t_VEC);
-  for (k = 1; k <= lim; ++k) gel(q, k) = gdiv(gel(M, k+1), gel(M, k));
+  for (k = 1; k <= lim; ++k)
+  {
+    if (gequal0(gel(M, k))) return gc_NULL(av);
+    gel(q, k) = gdiv(gel(M, k+1), gel(M, k));
+  }
   for (j = 1; j <= lim2; ++j)
   {
     long l = lim - 2*j;
     gel(c, 2*j) = gneg(gel(q, 1));
     for (k = 0; k <= l; ++k)
-      gel(e, k+1) = gsub(gadd(gel(e, k+2), gel(q, k+2)), gel(q, k+1));
+    {
+      GEN E = gsub(gadd(gel(e, k+2), gel(q, k+2)), gel(q, k+1));
+      if (gequal0(E)) return gc_NULL(av);
+      gel(e, k+1) = E;
+    }
     for (k = 0; k < l; ++k)
       gel(q, k+1) = gdiv(gmul(gel(q, k+2), gel(e, k+2)), gel(e, k+1));
     gel(c, 2*j+1) = gneg(gel(e, 1));
@@ -1516,16 +1530,27 @@ GEN
 quodif(GEN M, long n)
 {
   pari_sp av = avma;
-  return gerepilecopy(av, quodif_i(M, n));
+  GEN C = quodif_i(M, n);
+  if (!C) pari_err(e_MISC, "0 divisor in QD algorithm");
+  return gerepilecopy(av, C);
 }
 GEN
 contfracinit(GEN M, long n)
 {
   pari_sp av = avma;
   GEN C = quodif_i(M, n);
+  if (!C) pari_err(e_MISC, "0 divisor in QD algorithm");
   if (lg(C) < 3) { set_avma(av); retmkvec2(cgetg(1,t_VEC), cgetg(1,t_VEC)); }
   return gerepilecopy(av, contfrac_Euler(C));
 }
+GEN
+contfracinit_i(GEN M, long n)
+{
+  GEN C = quodif_i(M, n);
+  if (!C) return NULL;
+  if (lg(C) < 3) return mkvec2(cgetg(1,t_VEC), cgetg(1,t_VEC));
+  return contfrac_Euler(C);
+}
 
 /* Evaluate at 1/tinv the nlim first terms of the continued fraction output by
  * contfracinit. Shallow. */
@@ -1594,7 +1619,7 @@ monrefine(GEN Q, GEN QP, GEN z, long prec)
     if (gcmp(gabs(prnew, prec), gabs(pr, prec)) >= 0) break;
     pr = prnew;
   }
-  z = gprec_wensure(z, 2*prec-2);
+  z = gprec_wensure(z, precdbl(prec));
   z = gsub(z, gdiv(poleval(Q, z), poleval(QP, z)));
   return gerepileupto(av, z);
 }
@@ -1622,6 +1647,7 @@ Pade(GEN M, GEN *pP, GEN *pQ)
   pari_sp av = avma;
   long n = lg(M)-2, i;
   GEN v = QD(M, n), P = pol_0(0), Q = pol_1(0);
+  if (!v) pari_err(e_MISC, "0 divisor in QD algorithm");
   /* evaluate continued fraction => Pade approximants */
   for (i = n-1; i >= 1; i--)
   { /* S = P/Q: S -> v[i]*x / (1+S) */
@@ -1641,7 +1667,7 @@ Pade(GEN M, GEN *pP, GEN *pQ)
 static GEN
 veczetaprime(GEN a, GEN b, long N, long prec)
 {
-  long B = prec2nbits(prec) / 2;
+  long B = prec / 2;
   GEN v, h = mkcomplex(gen_0, real2n(-B, prec));
   v = veczeta(a, gadd(b, h), N, prec);
   return gmul2n(imag_i(v), B);
@@ -1705,7 +1731,7 @@ static GEN
 sumnummonieninit_i(GEN a, GEN b, GEN w, GEN n0, long prec)
 {
   GEN c, M, P, Q, Qp, vr, vabs, vwt, ga = gadd(a, b);
-  double bit = 2*prec2nbits(prec) / gtodouble(ga), D = bit*M_LN2;
+  double bit = 2*prec / gtodouble(ga), D = bit*M_LN2;
   double da = maxdd(1., gtodouble(a));
   long n = (long)ceil(D / (da*(log(D)-1)));
   long j, prec2, prec0 = prec + EXTRAPREC64;
@@ -1718,8 +1744,8 @@ sumnummonieninit_i(GEN a, GEN b, GEN w, GEN n0, long prec)
   prec = nbits2prec(maxdd(2.05*da*bit, bit0));
   prec2 = nbits2prec(maxdd(1.3*da*bit, bit0));
   S.w = w;
-  S.a = a = gprec_wensure(a, 2*prec-2);
-  S.b = b = gprec_wensure(b, 2*prec-2);
+  S.a = a = gprec_wensure(a, precdbl(prec));
+  S.b = b = gprec_wensure(b, precdbl(prec));
   S.n = n;
   S.j = 1;
   S.prec = prec;
@@ -1855,13 +1881,13 @@ sumnuminit(GEN fast, long prec)
 {
   pari_sp av;
   GEN s, v, d, C, res = cgetg(6, t_VEC);
-  long bitprec = prec2nbits(prec), N, k, k2, m;
+  long N, k, k2, m;
   double w;
 
   gel(res, 1) = d = mkfrac(gen_1, utoipos(4)); /* 1/4 */
   av = avma;
   w = gtodouble(glambertW(ginv(d), 0, LOWDEFAULTPREC));
-  N = (long)ceil(M_LN2*bitprec/(w*(1+w))+5);
+  N = (long)ceil(M_LN2*prec/(w*(1+w))+5);
   k = (long)ceil(N*w); if (k&1) k--;
 
   prec += EXTRAPREC64;
@@ -1969,12 +1995,12 @@ intnumgauexpinit(long prec)
 {
   pari_sp ltop = avma;
   GEN V, N, E, P, Q, R, vabs, vwt;
-  long l, n, k, j, prec2, prec0 = prec + EXTRAPREC64, bit = prec2nbits(prec);
+  long l, n, k, j, prec2, prec0 = prec + EXTRAPREC64;
 
-  n = (long)ceil(bit*0.226);
+  n = (long)ceil(prec*0.226);
   n |= 1; /* make n odd */
-  prec = nbits2prec(1.5*bit + 32);
-  prec2 = maxss(prec0, nbits2prec(1.15*bit + 32));
+  prec2 = maxss(prec0, nbits2prec(1.15*prec + 32));
+  prec = nbits2prec(1.5*prec + 32);
   constbern(n+3);
   V = cgetg(n + 4, t_VEC);
   for (k = 1; k <= n + 3; ++k)
@@ -2060,7 +2086,7 @@ sumnumap(void *E, GEN (*eval)(void*,GEN), GEN a, GEN tab, long prec)
   if (!tab) tab = sumnumapinit(fast, prec);
   else if (typ(tab) != t_VEC || lg(tab) != 3) pari_err_TYPE("sumnumap",tab);
   as = itos(a);
-  T.N = N = maxss(as + 1, (long)ceil(prec2nbits(prec)*0.327));
+  T.N = N = maxss(as + 1, (long)ceil(prec*0.327));
   T.E = E;
   T.f = eval;
   gN = stoi(N);
@@ -2108,23 +2134,26 @@ ratpolemax2(GEN F)
 static GEN
 sercoeff(GEN x, long n)
 {
-  long N = n - valp(x);
+  long N = n - valser(x);
   return (N < 0)? gen_0: gel(x,N+2);
 }
+static GEN
+rfrac_gtofp(GEN F, long prec)
+{ return mkrfrac(gel(F,1), RgX_gtofp(gel(F,2), prec)); }
 
 /* Compute the integral from N to infinity of a rational function F, deg(F) < -1
  * We must have N > 2 * r, r = max(1, |poles F|). */
 static GEN
 intnumainfrat(GEN F, long N, double r, long prec)
 {
-  long B = prec2nbits(prec), v, k, lim;
+  long v, k, lim;
   GEN S, ser;
   pari_sp av = avma;
 
-  lim = (long)ceil(B/log2(N/r));
-  ser = gmul(F, real_1(prec + EXTRAPREC64));
-  ser = rfracrecip_to_ser_absolute(ser, lim+2);
-  v = valp(ser);
+  lim = (long)ceil(prec / log2(N/r));
+  F = rfrac_gtofp(F, prec + EXTRAPREC64);
+  ser = rfracrecip_to_ser_absolute(F, lim+2);
+  v = valser(ser);
   S = gdivgu(sercoeff(ser,lim+1), lim*N);
   /* goes down to 2, but coeffs are 0 in degree < v */
   for (k = lim; k >= v; k--) /* S <- (S + coeff(ser,k)/(k-1)) / N */
@@ -2192,13 +2221,13 @@ get_kN(long r, long B, long *pk, long *pN)
 static GEN
 sumnumrat_i(GEN F, GEN F0, GEN vF, long prec)
 {
-  long B = prec2nbits(prec), vx, j, k, N;
+  long vx, j, k, N;
   GEN S, S1, S2, intf, _1;
   double r;
   if (poldegree(F, -1) > -2) pari_err(e_MISC, "sum diverges in sumnumrat");
   vx = varn(gel(F,2));
   r = ratpolemax(F);
-  get_kN((long)ceil(r), B, &k,&N);
+  get_kN((long)ceil(r), prec, &k,&N);
   intf = intnumainfrat(F, N, r, prec);
   /* N > ratpolemax(F) is not a pole */
   _1 = real_1(prec);
@@ -2267,7 +2296,7 @@ GEN
 prodnumrat(GEN F, long a, long prec)
 {
   pari_sp ltop = avma;
-  long B = prec2nbits(prec), j, k, m, N, vx;
+  long j, k, m, N, vx;
   GEN S, S1, S2, intf, G;
   double r;
 
@@ -2283,11 +2312,11 @@ prodnumrat(GEN F, long a, long prec)
   vx = varn(gel(F,2));
   if (a) F = gsubst(F, vx, gaddgs(pol_x(vx), a));
   r = ratpolemax2(F);
-  get_kN((long)ceil(r), B, &k,&N);
+  get_kN((long)ceil(r), prec, &k,&N);
   G = gdiv(deriv(F, vx), F);
   intf = intnumainfrat(gmul(pol_x(vx),G), N, r, prec);
   intf = gneg(gadd(intf, gmulsg(N, glog(gsubst(F, vx, stoi(N)), prec))));
-  S = gmul(real_1(prec), gsubst(G, vx, gaddgs(pol_x(vx), N)));
+  S = rfrac_gtofp(gsubst(G, vx, gaddgs(pol_x(vx), N)), prec);
   S = rfrac_to_ser_i(S, k + 2);
   S1 = gsqrt(gsubst(F, vx, utoipos(N)), prec);
   for (m = 0; m < N; m++) S1 = gmul(S1, gsubst(F, vx, utoi(m)));
@@ -2307,22 +2336,52 @@ sdmob(GEN s, long n, GEN fan)
     S = gadd(S, gdivgs(sercoeff(s, n/labs(D[i])), D[i]));
   return S;
 }
+
+/* z * (1 - p^-s); ms = -s or NULL (in which case s is a t_INT) */
+static GEN
+auxeuler(GEN z, GEN p, GEN s, GEN ms, long prec)
+{ return ms? gsub(z, gmul(z, gpow(p, ms, prec)))
+           : gsub(z, gdiv(z, powii(p, s))); }
+
 /* log (zeta(s) * prod_i (1 - P[i]^-s) */
 static GEN
-logzetan(GEN s, GEN P, long prec)
+logzetan(GEN s, GEN P, long N, long prec)
 {
-  GEN Z = gzeta(s, prec);
-  long i, l = lg(P);
-  for (i = 1; i < l; i++) Z = gsub(Z, gdiv(Z, gpow(gel(P,i), s, prec)));
+  GEN Z, ms = NULL;
+  pari_sp av;
+  if (typ(s) != t_INT) ms = gneg(s);
+  av = avma; Z = gzeta(s, prec);
+  if (P)
+  {
+    long i, l = lg(P);
+    for (i = 1; i < l; i++)
+    {
+      Z = auxeuler(Z, gel(P,i), s, ms, prec);
+      if (gc_needed(av,2)) Z = gerepileupto(av, Z);
+    }
+  }
+  else
+  {
+    forprime_t T;
+    GEN p;
+    forprime_init(&T, gen_2, utoi(N)); av = avma;
+    while ((p = forprime_next(&T)))
+    {
+      Z = auxeuler(Z, p, s, ms, prec);
+      if (gc_needed(av,2)) Z = gerepileupto(av, Z);
+    }
+  }
   return glog(Z, prec);
 }
 static GEN
-sumlogzeta(GEN ser, GEN s, GEN P, double rs, double lN, long vF, long lim,
-           long prec)
+sumlogzeta(GEN ser, GEN s, GEN P, long N, double rs, double lN, long vF,
+           long lim, long prec)
 {
-  GEN z = gen_0, v = vecfactoru_i(vF,lim);
-  pari_sp av = avma;
+  GEN z = gen_0, v;
+  pari_sp av;
   long i, n;
+  if (vF > lim) return z;
+  v = vecfactoru_i(vF,lim); av = avma;
   if (typ(s) == t_INT) constbern((itos(s) * lim + 1) >> 1);
   for (n = lim, i = lg(v)-1; n >= vF; n--, i--)
   {
@@ -2330,7 +2389,7 @@ sumlogzeta(GEN ser, GEN s, GEN P, double rs, double lN, long vF, long lim,
     if (!gequal0(t))
     { /* (n Re(s) - 1) log2(N) bits cancel in logzetan */
       long prec2 = prec + nbits2extraprec((n*rs-1) * lN);
-      GEN L = logzetan(gmulsg(n,gprec_wensure(s,prec2)), P, prec2);
+      GEN L = logzetan(gmulsg(n,gprec_wensure(s,prec2)), P, N, prec2);
       z = gerepileupto(av, gadd(z, gmul(L, t)));
       z = gprec_wensure(z, prec);
     }
@@ -2344,22 +2403,38 @@ rfrac_evalfp(GEN F, GEN x, long prec)
   GEN N = gel(F,1), D = gel(F,2), a, b = poleval(D, x);
   a = (typ(N) == t_POL && varn(N) == varn(D))? poleval(N, x): N;
   if (typ(a) != t_INT || typ(b) != t_INT ||
-      (lgefint(a) <= prec && lgefint(b) <= prec)) return gdiv(a, b);
-  return rdivii(a, b, prec + EXTRAPRECWORD);
+      (lg2prec(lgefint(a)) <= prec && lg2prec(lgefint(b)) <= prec)) return gdiv(a, b);
+  return rdivii(a, b, prec + EXTRAPREC64);
 }
 
-/* { F(p^s): p in P, p >= a }, F t_RFRAC */
+/* op_p F(p^s): p in P, p >= a, F t_RFRAC */
 static GEN
-vFps(GEN P, long a, GEN F, GEN s, long prec)
+opFps(GEN(*op)(GEN,GEN), GEN P, long N, long a, GEN F, GEN s, long prec)
 {
-  long i, j, l = lg(P);
-  GEN v = cgetg(l, t_VEC);
-  for (i = j = 1; i < l; i++)
+  GEN S = op == gadd? gen_0: gen_1;
+  pari_sp av = avma;
+  if (P)
   {
-    GEN p = gel(P,i); if (cmpiu(p, a) < 0) continue;
-    gel(v, j++) = rfrac_evalfp(F, gpow(p, s, prec), prec);
+    long i, j, l = lg(P);
+    for (i = j = 1; i < l; i++)
+    {
+      GEN p = gel(P,i); if (cmpiu(p, a) < 0) continue;
+      S = op(S, rfrac_evalfp(F, gpow(p, s, prec), prec));
+      if (gc_needed(av,2)) S = gerepileupto(av, S);
+    }
   }
-  setlg(v, j); return v;
+  else
+  {
+    forprime_t T;
+    GEN p;
+    forprime_init(&T, utoi(a), utoi(N)); av = avma;
+    while ((p = forprime_next(&T)))
+    {
+      S = op(S, rfrac_evalfp(F, gpow(p, s, prec), prec));
+      if (gc_needed(av,2)) S = gerepileupto(av, S);
+    }
+  }
+  return S;
 }
 
 static void
@@ -2380,7 +2455,7 @@ sumeulerrat(GEN F, GEN s, long a, long prec)
   pari_sp av = avma;
   GEN ser, z, P;
   double r, rs, RS, lN;
-  long B = prec2nbits(prec), prec2 = prec + EXTRAPREC64, vF, N, lim;
+  long prec2 = prec + EXTRAPREC64, vF, N, lim;
 
   euler_set_Fs(&F, &s);
   switch(typ(F))
@@ -2396,16 +2471,18 @@ sumeulerrat(GEN F, GEN s, long a, long prec)
   vF = -poldegree(F, -1);
   if (vF <= 0) pari_err(e_MISC, "sum diverges in sumeulerrat");
   r = polmax(gel(F,2));
-  N = maxss(30, a); lN = log2((double)N);
+  N = a; /* >= 2 */
+  /* if s not integral, computing p^-s at increased accuracy is too expensive */
+  if (typ(s) == t_INT) N = maxss(30, N);
+  lN = log2((double)N);
   RS = maxdd(1./vF, log2(r) / lN);
   if (rs <= RS)
     pari_err_DOMAIN("sumeulerrat", "real(s)", "<=",  dbltor(RS), dbltor(rs));
-  lim = (long)ceil(B / (rs*lN - log2(r)));
-  ser = gmul(real_1(prec2), F);
-  ser = rfracrecip_to_ser_absolute(ser, lim+1);
-  P = primes_interval(gen_2, utoipos(N));
-  z = sumlogzeta(ser, s, P, rs, lN, vF, lim, prec);
-  z = gadd(z, vecsum(vFps(P, a, F, s, prec)));
+  lim = (long)ceil(prec / (rs*lN - log2(r)));
+  ser = rfracrecip_to_ser_absolute(rfrac_gtofp(F, prec2), lim+1);
+  P = N < 1000000? primes_interval(gen_2, utoipos(N)): NULL;
+  z = sumlogzeta(ser, s, P, N, rs, lN, vF, lim, prec);
+  z = gadd(z, opFps(&gadd, P, N, a, F, s, prec));
   return gerepilecopy(av, gprec_wtrunc(z, prec));
 }
 
@@ -2430,7 +2507,7 @@ prodeulerrat(GEN F, GEN s, long a, long prec)
   pari_sp ltop = avma;
   GEN DF, NF, ser, P, z;
   double r, rs, RS, lN;
-  long B = prec2nbits(prec), prec2 = prec + EXTRAPREC64, vF, N, lim;
+  long prec2 = prec + EXTRAPREC64, vF, N, lim;
 
   euler_set_Fs(&F, &s);
   switch(typ(F))
@@ -2446,19 +2523,21 @@ prodeulerrat(GEN F, GEN s, long a, long prec)
   vF = - rfracm1_degree(NF, DF);
   if (rs * vF <= 1) pari_err(e_MISC, "product diverges in prodeulerrat");
   r = ratpolemax2(F);
-  N = maxss(maxss(30, a), (long)ceil(2*r)); lN = log2((double)N);
+  N = maxss(a, (long)ceil(2*r));
+  if (typ(s) == t_INT) N = maxss(N, 30);
+  lN = log2((double)N);
   RS = maxdd(1./vF, log2(r) / lN);
   if (rs <= RS)
     pari_err_DOMAIN("prodeulerrat", "real(s)", "<=",  dbltor(RS), dbltor(rs));
-  lim = (long)ceil(B / (rs*lN - log2(r)));
+  lim = (long)ceil(prec / (rs*lN - log2(r)));
   (void)rfracrecip(&NF, &DF); /* returned value is 0 */
   if (!RgX_is_ZX(DF) || !is_pm1(gel(DF,2))
-      || lim * log2(r) > 4 * B) NF = gmul(NF, real_1(prec2));
+      || lim * log2(r) > 4 * prec) NF = gmul(NF, real_1(prec2));
   ser = integser(rfrac_to_ser_i(rfrac_logderiv(NF,DF), lim+3));
   /* ser = log f, f = F(1/x) + O(x^(lim+1)) */
-  P = primes_interval(gen_2, utoipos(N));
-  z = gexp(sumlogzeta(ser, s, P, rs, lN, vF, lim, prec), prec);
-  z = gmul(z, vecprod(vFps(P, a, F, s, prec)));
+  P = N < 1000000? primes_interval(gen_2, utoipos(N)): NULL;
+  z = gexp(sumlogzeta(ser, s, P, N, rs, lN, vF, lim, prec), prec);
+  z = gmul(z, opFps(&gmul, P, N, a, F, s, prec));
   return gerepilecopy(ltop, gprec_wtrunc(z, prec));
 }
 
@@ -2471,13 +2550,13 @@ sumnumlagrange1init(GEN c1, long flag, long prec)
   pari_sp av = avma;
   GEN V, W, T;
   double c1d;
-  long B = prec2nbits(prec), prec2;
+  long prec2;
   ulong n, N;
   c1d = c1 ? gtodouble(c1) : 0.332;
   if (c1d <= 0)
     pari_err_DOMAIN("sumnumlagrangeinit", "c1", "<=", gen_0, c1);
-  N = (ulong)ceil(c1d*B); if ((N&1L) == 0) N++;
-  prec2 = nbits2prec(B+(long)ceil(1.8444*N) + 16);
+  N = (ulong)ceil(c1d*prec); if ((N&1L) == 0) N++;
+  prec2 = nbits2prec(prec+(long)ceil(1.8444*N) + 16);
   W = vecbinomial(N);
   T = vecpowuu(N, N);
   V = cgetg(N+1, t_VEC); gel(V,N) = gel(T,N);
@@ -2499,11 +2578,11 @@ sumnumlagrange2init(GEN c1, long flag, long prec)
   pari_sp av = avma;
   GEN V, W, T, told;
   double c1d = c1 ? gtodouble(c1) : 0.228;
-  long B = prec2nbits(prec), prec2;
+  long prec2;
   ulong n, N;
 
-  N = (ulong)ceil(c1d*B); if ((N&1L) == 0) N++;
-  prec2 = nbits2prec(B+(long)ceil(1.18696*N) + 16);
+  N = (ulong)ceil(c1d*prec); if ((N&1L) == 0) N++;
+  prec2 = nbits2prec(prec+(long)ceil(1.18696*N) + 16);
   W = vecbinomial(2*N);
   T = vecpowuu(N, 2*N);
   V = cgetg(N+1, t_VEC); gel(V, N) = told = gel(T,N);
@@ -2532,7 +2611,7 @@ sumnumlagrangeinit_i(GEN al, GEN c1, long flag, long prec)
   pari_sp av = avma;
   GEN V, W;
   double c1d = 0.0, c2;
-  long B = prec2nbits(prec), B1, prec2, dal;
+  long prec2, dal;
   ulong j, n, N;
 
   if (typ(al) == t_INT)
@@ -2561,9 +2640,8 @@ sumnumlagrangeinit_i(GEN al, GEN c1, long flag, long prec)
     if (c1d <= 0)
       pari_err_DOMAIN("sumnumlagrangeinit", "c1", "<=", gen_0, c1);
   }
-  N = (ulong)ceil(c1d*B); if ((N&1L) == 0) N++;
-  B1 = B + (long)ceil(c2*N) + 16;
-  prec2 = nbits2prec(B1);
+  N = (ulong)ceil(c1d * prec); if ((N&1L) == 0) N++;
+  prec2 = nbits2prec(prec + (long)ceil(c2*N) + 16);
   V = vecpowug(N, al, prec2);
   W = cgetg(N+1, t_VEC);
   for (n = 1; n <= N; ++n)
@@ -2687,10 +2765,9 @@ sumnumsidi(void *E, GEN (*f)(void*, GEN, long), GEN a, double mu, long prec)
 {
   pari_sp av;
   GEN M, N, Wkeep = gen_0, W = gen_0, _1, S, t, Wp;
-  long bit = prec2nbits(prec), newbit = (long)(mu * bit) + 33;
-  long n, s, fail = 0, BIG = LONG_MAX, ekeep = LONG_MAX;
+  long n, s, fail = 0, BIG = LONG_MAX, ekeep = LONG_MAX, bit = prec;
 
-  prec = nbits2prec(newbit); _1 = real_1(prec);
+  prec = nbits2prec((long)(mu * prec) + 33); _1 = real_1(prec);
   av = avma; S = real_0(prec); t = Wp = f(E, a, prec);
   M = N = cgetg(1, t_VEC);
   for (n = 1;; n++)
