@@ -411,7 +411,7 @@ forstep(GEN a, GEN b, GEN s, GEN code)
     case t_INTMOD:
       if (typ(a) != t_INT) a = gceil(a);
       a = addii(a, modii(subii(gel(s,2),a), gel(s,1)));
-      s = gel(s,1);
+      s = gel(s,1); /* FALL THROUGH */
     default: ss = gsigne(s);
   }
   if (!ss) pari_err_DOMAIN("forstep","step","=",gen_0,s);
@@ -599,6 +599,48 @@ _next_lt(forvec_t *d)
     if (--i <= 0) return NULL;
   }
 }
+
+/* on Z^n /(cyc Z^n) [over integers]
+ * torsion (cyc>0) and free (cyc=0) components may be interleaved */
+static GEN
+_next_mod_cyc(forvec_t *d)
+{ /* keep free components indices t1 < t2 last nonzero < t3 */
+  long t, t1 = 0, t2 = 0, t3 = 0;
+  if (d->first) { d->first = 0; return (GEN)d->a; }
+  for (t = d->n; t > 0; t--)
+  {
+    if (signe(d->M[t]) > 0)
+    { /* torsion component */
+      d->a[t] = incloop(d->a[t]);
+      if (cmpii(d->a[t], d->M[t]) < 0) return (GEN)d->a;
+      d->a[t] = resetloop(d->a[t], gen_0);
+    }
+    else
+    { /* set or update t1,t2,t3 */
+      if (t2 && !t1) t1 = t;
+      if (!t2 && signe(d->a[t])) t2 = t;
+      if (!t2) t3 = t;
+    }
+  }
+  if (!t3 && !t2) return NULL; /* no free component, stop */
+  if (!t2) d->a[t3] = resetloop(d->a[t3], gen_m1);
+  else if (!t3 && signe(d->a[t2]) < 0) togglesign(d->a[t2]);
+  else if (signe(d->a[t2]) < 0)
+  {
+    d->a[t2] = incloop(d->a[t2]);
+    d->a[t3] = resetloop(d->a[t3], gen_m1);
+  }
+  else if (!t1) { d->a[t2] = incloop(d->a[t2]); togglesign(d->a[t2]); }
+  else
+  {
+    if (signe(d->a[t1]) < 0)
+    { d->a[t2] = incloop(d->a[t2]); togglesign(d->a[t2]); }
+    else
+    { togglesign(d->a[t2]); d->a[t2] = incloop(d->a[t2]); }
+    d->a[t1] = incloop(d->a[t1]);
+  }
+  return (GEN)d->a;
+}
 /* for forvec(v=[],) */
 static GEN
 _next_void(forvec_t *d)
@@ -606,6 +648,32 @@ _next_void(forvec_t *d)
   if (d->first) { d->first = 0; return (GEN)d->a; }
   return NULL;
 }
+static int
+RgV_is_ZV_nonneg(GEN x)
+{
+  long i;
+  for (i = lg(x)-1; i > 0; i--)
+    if (typ(gel(x,i)) != t_INT || signe(gel(x, i)) < 0) return 0;
+  return 1;
+}
+/* x assumed to be cyc vector, l>1 */
+static int
+forvec_mod_cyc_init(forvec_t *d, GEN x)
+{
+  long i, tx = typ(x), l = lg(x);
+  d->a = (GEN*)cgetg(l,tx); /* current */
+  d->M = (GEN*)cgetg(l,tx); /* cyc */
+  for (i = 1; i < l; i++)
+  {
+    d->a[i] = setloop(gen_0);
+    d->M[i] = setloop(gel(x, i));
+  }
+  d->first = 1;
+  d->n = l-1;
+  d->m = NULL;
+  d->next = &_next_mod_cyc;
+  return 1;
+}
 
 /* Initialize minima (m) and maxima (M); guarantee M[i] - m[i] integer and
  *   if flag = 1: m[i-1] <= m[i] <= M[i] <= M[i+1]
@@ -616,6 +684,8 @@ forvec_init(forvec_t *d, GEN x, long flag)
 {
   long i, tx = typ(x), l = lg(x), t = t_INT;
   if (!is_vec_t(tx)) pari_err_TYPE("forvec [not a vector]", x);
+  if (l > 1 && RgV_is_ZV_nonneg(x))
+      return forvec_mod_cyc_init(d, x);
   d->first = 1;
   d->n = l - 1;
   d->a = (GEN*)cgetg(l,tx);
@@ -752,7 +822,7 @@ sum_init(GEN x0, GEN t)
 }
 
 GEN
-suminf_bitprec(void *E, GEN (*eval)(void *, GEN), GEN a, long bit)
+suminf(void *E, GEN (*eval)(void *, GEN), GEN a, long bit)
 {
   long fl = 0, G = bit + 1;
   pari_sp av0 = avma, av;
@@ -780,11 +850,8 @@ suminf_bitprec(void *E, GEN (*eval)(void *, GEN), GEN a, long bit)
   return gerepileupto(av0, gsub(x, _1));
 }
 GEN
-suminf(void *E, GEN (*eval)(void *, GEN), GEN a, long prec)
-{ return suminf_bitprec(E, eval, a, prec2nbits(prec)); }
-GEN
-suminf0_bitprec(GEN a, GEN code, long bit)
-{ EXPR_WRAP(code, suminf_bitprec(EXPR_ARG, a, bit)); }
+suminf0(GEN a, GEN code, long bit)
+{ EXPR_WRAP(code, suminf(EXPR_ARG, a, bit)); }
 
 GEN
 sumdivexpr(GEN num, GEN code)
@@ -876,7 +943,7 @@ prodinf(void *E, GEN (*eval)(void *, GEN), GEN a, long prec)
   if (typ(a) != t_INT) pari_err_TYPE("prodinf",a);
   a = setloop(a);
   av = avma;
-  fl=0; G = -prec2nbits(prec)-5;
+  fl=0; G = -prec-5;
   for(;;)
   {
     p1 = eval(E, a); if (gequal0(p1)) { x = p1; break; }
@@ -901,7 +968,7 @@ prodinf1(void *E, GEN (*eval)(void *, GEN), GEN a, long prec)
   if (typ(a) != t_INT) pari_err_TYPE("prodinf1",a);
   a = setloop(a);
   av = avma;
-  fl=0; G = -prec2nbits(prec)-5;
+  fl=0; G = -prec-5;
   for(;;)
   {
     p2 = eval(E, a); p1 = gaddgs(p2,1);
@@ -1171,7 +1238,7 @@ polzag(long n, long m)
 /*0.39322 > 1/log_2(3+sqrt(8))*/
 static ulong
 sumalt_N(long prec)
-{ return (ulong)(0.39322*(prec2nbits(prec) + 7)); }
+{ return (ulong)(0.39322*(prec + 7)); }
 
 GEN
 sumalt(void *E, GEN (*eval)(void *, GEN), GEN a, long prec)
@@ -1211,7 +1278,7 @@ sumalt2(void *E, GEN (*eval)(void *, GEN), GEN a, long prec)
   GEN s, dn, pol;
 
   if (typ(a) != t_INT) pari_err_TYPE("sumalt",a);
-  N = (long)(0.307073*(prec2nbits(prec) + 5)); /*0.307073 > 1/log_2(\beta_B)*/
+  N = (long)(0.307073*(prec + 5)); /*0.307073 > 1/log_2(\beta_B)*/
   pol = ZX_div_by_X_1(polzag1(N,N>>1), &dn);
   a = setloop(a);
   N = degpol(pol);
@@ -1219,7 +1286,7 @@ sumalt2(void *E, GEN (*eval)(void *, GEN), GEN a, long prec)
   av2 = avma;
   for (k=0; k<=N; k++)
   {
-    GEN t = itor(gel(pol,k+2), prec+EXTRAPRECWORD);
+    GEN t = itor(gel(pol,k+2), prec+EXTRAPREC64);
     s = gadd(s, gmul(t, eval(E, a)));
     if (k == N) break;
     a = incloop(a); /* in place! */
@@ -1288,7 +1355,7 @@ static GEN
 sumpos_init(void *E, GEN (*f)(void *, GEN), GEN a, long N, long prec)
 {
   GEN S = cgetg(N+1,t_VEC);
-  long k, G = -prec2nbits(prec) - 5;
+  long k, G = -prec - 5;
   for (k=1; k<=N; k+=2) binsum(S,k, E,f, a,G,prec);
   return S;
 }
@@ -1331,7 +1398,7 @@ sumpos2(void *E, GEN (*eval)(void *, GEN), GEN a, long prec)
 
   if (typ(a) != t_INT) pari_err_TYPE("sumpos2",a);
   a = subiu(a,1);
-  N = (ulong)(0.31*(prec2nbits(prec) + 5));
+  N = (ulong)(0.31*(prec + 5));
 
   if (odd(N)) N++; /* extra precision for free */
   S = sumpos_init(E, eval, a, N, prec);
@@ -1426,7 +1493,7 @@ zbrent(void *E, GEN (*eval)(void *, GEN), GEN a, GEN b, long prec)
   if (gsigne(fa)*gsigne(fb) > 0)
     pari_err_DOMAIN("solve", "f(a)f(b)", ">", gen_0, mkvec2(fa, fb));
 SOLVE:
-  bit0 = -prec2nbits(prec); bit = 3+bit0; itmax = 1 - 2*bit0;
+  bit0 = -prec; bit = 3+bit0; itmax = 1 - 2*bit0;
   fc = fb; e = d = NULL;
   for (iter = 1; iter <= itmax; ++iter)
   { /* b = current best guess, a and c auxiliary points */
@@ -1498,7 +1565,7 @@ solvestep(void *E, GEN (*f)(void *,GEN), GEN a, GEN b, GEN step, long flag, long
   const long ITMAX = 10;
   pari_sp av = avma;
   GEN fa, a0, b0;
-  long sa0, it, bit = prec2nbits(prec) / 2, ct = 0, s = gcmp(a,b);
+  long sa0, it, bit = prec / 2, ct = 0, s = gcmp(a,b);
 
   if (!s) return gequal0(f(E, a)) ? gcopy(mkvec(a)): cgetg(1,t_VEC);
   if (s > 0) swap(a, b);
@@ -1661,7 +1728,7 @@ chk_ord(long m)
   if (m < 0)
     pari_err_DOMAIN("derivnumk", "derivation order", "<", gen_0, stoi(m));
 }
-/* m! / N! for m in ind; vecmax(ind) <= N */
+/* m! / N! for m in ind; vecmax(ind) <= N. Result not a GEN if ind contains 0. */
 static GEN
 vfact(GEN ind, long N, long prec)
 {
@@ -1735,9 +1802,9 @@ derivnumk(void *E, GEN (*eval)(void *, GEN, long), GEN x, GEN ind0, long prec)
   }
 
   p = precision(x);
-  fpr = p ? prec2nbits(p): prec2nbits(prec);
+  fpr = p ? p: prec;
   e = (fpr + 3*M*log2((double)M)) / (2*M);
-  ex = gexpo(x);
+  ex = gexpo(real_i(x));
   if (ex < 0) ex = 0; /* near 0 */
   pr = (long)ceil(fpr + e * M); /* ~ 3fpr/2 */
   newprec = nbits2prec(pr + (emax - emin) + ex + BITS_IN_LONG);
@@ -1861,11 +1928,11 @@ laurentseries(void *E, GEN (*f)(void*,GEN x, long), long M, long v, long prec)
   {
     long i, dr, vr;
     GEN s;
-    s = cgetg(d+2, t_SER); s[1] = evalsigne(1) | evalvalp(1) | evalvarn(v);
+    s = cgetg(d+2, t_SER); s[1] = evalsigne(1) | evalvalser(1) | evalvarn(v);
     gel(s, 2) = gen_1; for (i = 3; i <= d+1; i++) gel(s, i) = gen_0;
     s = f(E, s, prec);
     if (typ(s) != t_SER || varn(s) != v) pari_err_TYPE("laurentseries", s);
-    vr = valp(s);
+    vr = valser(s);
     if (M < vr) { set_avma(av); return zeroser(v, M); }
     dr = lg(s) + vr - 3 - M;
     if (dr >= 0) return gerepileupto(av, s);
@@ -2005,9 +2072,8 @@ get_c(GEN a)
 static void
 limit_Nprec(struct limit *L, GEN alpha, long prec)
 {
-  long bit = prec2nbits(prec);
-  L->N = ceil(get_c(alpha) * bit);
-  L->prec = nbits2prec(bit + (long)ceil(get_accu(alpha) * L->N));
+  L->N = ceil(get_c(alpha) * prec);
+  L->prec = nbits2prec(prec + (long)ceil(get_accu(alpha) * L->N));
 }
 /* solve x - a log(x) = b; a, b >= 3 */
 static double
