@@ -27,7 +27,11 @@ static long
 vecgroup_sumorders(GEN L)
 {
   long i, s = 0;
-  for(i=1; i<lg(L); i++) s += group_order(gel(L,i));
+  for (i=1; i<lg(L); i++)
+  {
+    GEN G = gel(L,i);
+    s += (lg(G)==3 && typ(gel(G,1))==t_VEC) ? group_order(G): lg(G)-1;
+  }
   return s;
 }
 
@@ -78,17 +82,12 @@ indexgroupcentre(GEN G, GEN Z, const long *good, const long *bad)
 static GEN
 vecgroup_idxlist(GEN L, long order)
 {
-  pari_sp ltop=avma;
-  GEN V;
-  long i,j,n;
-  for (n=0,i=1; i<lg(L); i++)
-    if (group_order(gel(L,i))==order)
-      n++;
-  V=cgetg(n+1,t_VECSMALL);
-  for(i=1,j=1; j<=n; i++)
-    if (group_order(gel(L,i))==order)
-      V[j++]=group_ident(gel(L,i),NULL);
-  return gerepileupto(ltop,vecsmall_uniq(V));
+  pari_sp av = avma;
+  long i, j, l = lg(L);
+  GEN V = cgetg(l, t_VECSMALL);
+  for (i = j = 1; i < l; i++)
+    if (group_order(gel(L,i)) == order) V[j++] = group_ident(gel(L,i),NULL);
+  setlg(V, j); return gerepileuptoleaf(av, vecsmall_uniq(V));
 }
 
 /* Group identification code.
@@ -282,7 +281,7 @@ group_ident_i(GEN G, GEN S)
       -1};
     long i;
     const long *t;
-    GEN Z = groupelts_center(S), L = group_subgroups(G);
+    GEN Z = groupelts_center(S), L = galoissubgroups(G);
     long scenter = groupelts_sumorders(Z), svecgroup = vecgroup_sumorders(L);
     long u = svecgroup+10000*scenter; /*This is used as a hash value*/
 
