@@ -15,6 +15,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #include "pari.h"
 #include "paripriv.h"
 
+#define DEBUGLEVEL DEBUGLEVEL_padicfields
+
 /************************************************************/
 /*     Computation of all the extensions of a given         */
 /*               degree of a p-adic field                   */
@@ -242,7 +244,7 @@ FieldData(KRASNER_t *data, FAD_t *fdata, GEN eis, GEN topx)
     /* First lift to a root mod p */
     for (;;) {
       p2 = FpX_FpXQ_eval(data->T, z, fdata->top, data->p);
-      if (gcmp0(p2)) break;
+      if (gequal0(p2)) break;
       p3 = FpX_FpXQ_eval(p1, z, fdata->top, data->p);
       z  = FpX_sub(z, FpXQ_div(p2, p3, fdata->top, data->p), data->p);
     }
@@ -849,7 +851,7 @@ checkpols(GEN p, GEN EFJ, GEN pols)
     p1 = poldisc0(pol, -1);
     if (gvaluation(p1, p) != f*(e+j-1)) pari_err_BUG("Discriminant is incorrect");
     /* only compute a p-maximal order */
-    p1 = nfinitall(mkvec2(pol, mkvec(p)), 0, DEFAULTPREC);
+    p1 = nfinit0(mkvec2(pol, mkvec(p)), 0, DEFAULTPREC);
     p2 = idealprimedec(p1, p);
     if(lg(p2) > 2) pari_err_BUG("Prime p is split");
     p2 = gel(p2, 1);
@@ -877,7 +879,7 @@ checkpols(GEN p, GEN EFJ, GEN pols)
     p1 = gmael(p1, 1, 1);
     if (poldegree(p1, -1) == e*f) pari_err_BUG("fields are isomorphic");
     /*
-      p1 = nfinitall(mkvec2(pol, mkvec(p)), 0, DEFAULTPREC);
+      p1 = nfinit0(mkvec2(pol, mkvec(p)), 0, DEFAULTPREC);
       p2 = idealprimedec_galois(p1, p);
       if (!cmpis(mulii(gel(p2, 3), gel(p2, 4)), e*f)) pari_err_BUG("fields are isomorphic");
     */
