@@ -13,7 +13,8 @@ with the package; see the file 'COPYING'. If not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 
 BEGINEXTERN
-void pari_emscripten_help(const char *s);
+void pari_emscripten_help(const char *s, long n);
 void pari_emscripten_wget(const char *s);
 void pari_emscripten_plot_init(long width, long height);
+void pari_emscripten_init(long rsize, long vsize);
 ENDEXTERN
